package com.yihu.hos.app.controller;

import com.yihu.hos.app.model.SystemServiceEndpoint;
import com.yihu.hos.app.service.AppServiceManage;
import com.yihu.hos.core.constants.MicroServices;
import com.yihu.hos.core.constants.ServiceApi;
import com.yihu.hos.web.framework.model.Envelop;
import com.yihu.hos.web.framework.util.controller.BaseController;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;

/**
 * 应用管理/服务管理
 */
@RequestMapping(MicroServices.ApiVersion.Version1_0)
@Controller
public class AppServiceController extends BaseController {

    @Resource(name = AppServiceManage.BEAN_ID)
    private AppServiceManage appServiceManage;

    @RequestMapping(value = ServiceApi.AppService.GetById, method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "获取App服务")
    public SystemServiceEndpoint getAppService(
            @ApiParam(name = "id", value = "id")
            @PathVariable(value = "id") String id) throws Exception {
        SystemServiceEndpoint service = appServiceManage.getById(id);
        return service;
    }

    @RequestMapping(value = ServiceApi.AppService.Create, method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ResponseBody
    @ApiOperation(value = "创建App服务")
    public SystemServiceEndpoint create(
            @ApiParam(name = "json", value = "对象JSON结构体", allowMultiple = true)
            @RequestBody String json) throws Exception {
        SystemServiceEndpoint service = toEntity(json, SystemServiceEndpoint.class);
        appServiceManage.create(service);
        return service;
    }

    @RequestMapping(value = ServiceApi.AppService.Update, method = RequestMethod.PUT, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ApiOperation(value = "更新App服务")
    @ResponseBody
    public SystemServiceEndpoint update(
            @ApiParam(name = "app", value = "对象JSON结构体", allowMultiple = true)
            @RequestBody String json) throws Exception {
        SystemServiceEndpoint service = toEntity(json, SystemServiceEndpoint.class);
        return appServiceManage.update(service);
    }

    @RequestMapping(value = ServiceApi.AppService.DeleteById, method = RequestMethod.DELETE)
    @ApiOperation(value = "删除App服务")
    @ResponseBody
    public Envelop delete(
            @ApiParam(name = "id", value = "id")
            @PathVariable(value = "id") String id) throws Exception {
        appServiceManage.deleteById(id);
        Envelop envelop = new Envelop();
        envelop.setSuccessFlg(true);
        return envelop;
    }

    /**
     * 应用服务列表
     *
     * @param request
     * @return
     */
    @RequestMapping(value =ServiceApi.AppService.List, method = RequestMethod.GET)
    @ApiOperation(value = "获取App服务列表")
    @ResponseBody
    public Envelop getAppList(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段", defaultValue = "id,name,secret,url,createTime,icon,releaseFlag")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档", defaultValue = "+name,+createTime")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "size", value = "分页大小", defaultValue = "15")
            @RequestParam(value = "size") Integer size,
            @ApiParam(name = "page", value = "页码", defaultValue = "1")
            @RequestParam(value = "page") Integer page,
            HttpServletRequest request,
            HttpServletResponse response)throws Exception  {

        List<SystemServiceEndpoint> appServiceList =appServiceManage.search(fields,filters,sorts,page,size);
        List<SystemServiceEndpoint> appServices = (List)convertToModels(appServiceList, new ArrayList<>(appServiceList.size()), SystemServiceEndpoint.class, fields);
        Envelop envelop = new Envelop();
        envelop.setSuccessFlg(true);
        envelop.setDetailModelList(appServices);
        if(page!=null && size!=null){//都不为空,则为分页查询  -->>计算总页数,总条数等
            long count = appServiceManage.getCount(filters);
            int cou = (int)count;
            envelop.setTotalCount((int)count);//总条数
            envelop.setTotalPage((cou-1)/size+1);//计算总页数
        }
        return envelop;
    }

}
