package com.yihu.hos.app.service;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.hos.app.dao.AppFeatureDao;
import com.yihu.hos.app.model.AppFeature;
import com.yihu.hos.core.log.Logger;
import com.yihu.hos.core.log.LoggerFactory;
import com.yihu.hos.web.framework.baseservice.BaseService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;
import java.util.List;

/**
 * Created by chenweida on 2016/1/21.
 */

@Service("appFeatureService")
public class AppFeatureService extends BaseService<AppFeature> {
    public static final String BEAN_ID = "appFeatureService";

    static final Logger logger = LoggerFactory.getLogger(AppFeatureService.class);

    @Resource(name = AppFeatureDao.BEAN_ID)
    private AppFeatureDao appFeatureDao;

    @Transactional
    public AppFeature addFeature(AppFeature obj) throws Exception {
        Integer sort = obj.getSort();
        if(sort==null||sort==0){
            obj.setSort(1);
        }
        obj.setId(null);
        appFeatureDao.saveEntity(obj);
        return obj;
    }

    public List<AppFeature> getFeatureByCode(String code) throws Exception {
        return appFeatureDao.findByCode(code);
    }


    public AppFeature getById(Integer id) throws Exception {
        return appFeatureDao.getEntity(AppFeature.class, id);
    }

    @Transactional
    public void delete(Integer id) throws Exception {
        AppFeature feature = appFeatureDao.getEntity(AppFeature.class, id);
        appFeatureDao.deleteEntity(feature);
    }

    @Transactional
    public AppFeature update(AppFeature obj) throws Exception {
        appFeatureDao.updateEntity(obj);
        return obj;
    }

    /**
     * 生成菜单JSON对象字符串
     * @return AppFeature
     */
    public AppFeature joinMenuItemJsonStr(AppFeature appFeature) throws JsonProcessingException {
        ObjectMapper objectMapper = new ObjectMapper();
        Map<String, Object> contentMap = new HashMap<>();
        contentMap.put("id", appFeature.getId());
        contentMap.put("level", appFeature.getLevel());
        contentMap.put("text", appFeature.getName());
        if (!"1".equals(appFeature.getLevel())) {
            contentMap.put("pid", appFeature.getParentId());
        }
        if (appFeature.getUrl().startsWith("/")) {
            contentMap.put("url", "${contextRoot}" + appFeature.getUrl());
        }
        appFeature.setContent(objectMapper.writeValueAsString(contentMap));
        return appFeature;
    }
}
