package com.yihu.hos.app.service;

import com.yihu.hos.app.dao.AppDao;
import com.yihu.hos.app.model.App;
import com.yihu.hos.core.log.Logger;
import com.yihu.hos.core.log.LoggerFactory;
import com.yihu.hos.web.framework.baseservice.BaseService;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Random;

/**
 * Created by chenweida on 2016/1/21.
 */

@Service("appManager")
public class AppManager extends BaseService<App> {
    public static final String BEAN_ID = "appManager";
    static final Logger logger = LoggerFactory.getLogger(AppManager.class);
    @Resource(name = AppDao.BEAN_ID)
    private AppDao appDao;

    @Value("${spring.data.mongodb.gridFsDatabase}")
    private String dbName;

    private static String getRandomString(int length) {
        String str = "abcdefghigklmnopkrstuvwxyzABCDEFGHIGKLMNOPQRSTUVWXYZ0123456789";
        StringBuffer buffer = new StringBuffer();

        Random random = new Random();
        for (int i = 0; i < length; i++) {
            int number = random.nextInt(str.length() - 1);//0~61
            buffer.append(str.charAt(number));
        }

        return buffer.toString();
    }


    public App getAppById(String id) throws Exception {
        return appDao.getEntity(App.class, id);
    }

    public List<App> getAppByCode(String code) throws Exception {
        return appDao.findByCode(code);
    }

    @Transactional
    public App addApp(App obj) throws Exception {

        obj.setId(getRandomString(10));
        obj.setSecret(getRandomString(16));
        obj.setCreateTime(new Date());
        obj.setStatus("WaitingForApprove");
        List<App> appByCode = getAppByCode(obj.getCode());

        if (appByCode != null && appByCode.size() > 0) {
            throw new Exception("保存失败，应用Code已存在");
        }
        appDao.saveEntity(obj);
        return obj;
    }

    @Transactional
    public App updateApp(App obj) throws Exception {
        appDao.updateEntity(obj);
        return obj;
    }

    @Transactional
    public void deleteApp(String id) throws Exception {
        App systemApp = appDao.getEntity(App.class, id);
        appDao.deleteEntity(systemApp);
    }

    public List<App> getListByCondition(String appIds, String catalog, String manageType) {
        return appDao.getListByCondition(appIds, catalog, manageType);
    }
}
