package com.yihu.hos.app.service;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.hos.app.dao.AppServiceDao;
import com.yihu.hos.app.model.SystemServiceEndpoint;
import com.yihu.hos.core.log.Logger;
import com.yihu.hos.core.log.LoggerFactory;
import com.yihu.hos.web.framework.baseservice.BaseService;
import com.yihu.hos.web.framework.constant.ContextAttributes;
import com.yihu.hos.web.framework.constant.EndPointConstant;
import com.yihu.hos.web.framework.thread.LocalContext;
import io.zbus.mq.Broker;
import io.zbus.mq.Message;
import io.zbus.mq.Producer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.Date;
import java.util.List;

/**
 * Created by chenweida on 2016/1/21.
 */

@Service("appService")
public class AppServiceManage extends BaseService<SystemServiceEndpoint> {

    public static final String BEAN_ID = "appService";

    static final Logger logger = LoggerFactory.getLogger(AppServiceManage.class);

    @Resource(name = AppServiceDao.BEAN_ID)
    private AppServiceDao appServiceDao;

    @Autowired
    private ObjectMapper objectMapper;

    @Autowired
    private Broker broker;

    public SystemServiceEndpoint getById(String id) throws Exception {
        return appServiceDao.getEntity(SystemServiceEndpoint.class, id);
    }

    public SystemServiceEndpoint create(SystemServiceEndpoint service) throws Exception {
        service.setCreateDate(new Date());
        appServiceDao.saveEntity(service);
        this.sendMsg(EndPointConstant.ADD_ENDPOINT, service);
        return service;
    }

    public List<SystemServiceEndpoint> getByCode(String code) throws Exception {
        return appServiceDao.findByCode(code);
    }

    @Transactional
    public SystemServiceEndpoint update(SystemServiceEndpoint obj) throws Exception {
        appServiceDao.updateEntity(obj);
        this.sendMsg(EndPointConstant.ADD_ENDPOINT, obj);
        return obj;
    }

    @Transactional
    public void deleteById(String id) throws Exception {
        SystemServiceEndpoint service = appServiceDao.getEntity(SystemServiceEndpoint.class, id);
        appServiceDao.deleteEntity(service);
        this.sendMsg(EndPointConstant.DELETE_ENDPOINT, service);
    }

    private void sendMsg(String event, SystemServiceEndpoint endpoint) {
        if (broker == null) {
            logger.error("broker is null.");
            return;
        }

        try {
            String msg = objectMapper.writeValueAsString(endpoint);
            String tenant = LocalContext.getContext().getAttachment(ContextAttributes.TENANT_NAME);

            String topic = EndPointConstant.ZBUS_MQ + "@" + tenant;
            Producer producer = new Producer(broker);
            //确定为创建消息队列需要显示调用
            producer.declareTopic(topic);
            Message message = new Message();
            message.setTopic(topic);
            message.setHeader("event", event);
            message.setHeader("tenant", tenant);
            message.setMethod("POST");
            message.setBody(msg);
            producer.publish(message);
        } catch (IOException | InterruptedException e) {
            logger.error(e.getMessage());
            e.printStackTrace();
        }
    }
}
