package com.yihu.hos.ehr.standard.cdaversion.controller;

import com.yihu.hos.common.controller.ExtendEndPoint;
import com.yihu.hos.core.constants.MicroServices;
import com.yihu.hos.core.constants.ServiceApi;
import com.yihu.hos.model.standard.MSTDVersion;
import com.yihu.hos.standard.model.standard.STDVersion;
import com.yihu.hos.standard.service.standard.STDVersionService;
import com.yihu.hos.web.framework.constant.ErrorCode;
import com.yihu.hos.web.framework.exception.EsbApiException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

/**
 * @author lincl
 * @version 1.0
 * @created 2016.2.3
 */
@RestController
@RequestMapping(MicroServices.ApiVersion.Version1_0)
@Api(value = "version", description = "版本服务")
public class VersionEndPoint extends ExtendEndPoint<MSTDVersion> {

    @Autowired
    private STDVersionService versionService;

//    @Autowired
//    private DispatchService dispatchService;

    protected EsbApiException errNotFound(String version) {
        return errNotFound("标准版本", version);
    }

    private STDVersion findVersion(String version) throws ParseException {
        STDVersion cdaVersion = versionService.getVersion(version);
        if (cdaVersion == null)
            throw errNotFound(version);
        return cdaVersion;
    }

    private STDVersion findVersionInstage(String version) throws ParseException {
        STDVersion cdaVersion = findVersion(version);
        if (!cdaVersion.isInStage()) {
            throw new EsbApiException(ErrorCode.InValidCDAVersionStage);
        }
        return cdaVersion;
    }

    @RequestMapping(value = ServiceApi.Standards.Versions, method = RequestMethod.GET)
    @ApiOperation(value = "适配采集标准")
    public Collection<MSTDVersion> searchCDAVersions(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段", defaultValue = "version,versionName,commitTime,baseVersion,inStage")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件", defaultValue = "")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档", defaultValue = "+version")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "size", value = "分页大小", defaultValue = "15")
            @RequestParam(value = "size", required = false) int size,
            @ApiParam(name = "page", value = "页码", defaultValue = "1")
            @RequestParam(value = "page", required = false) int page,
            HttpServletRequest request,
            HttpServletResponse response) throws Exception {

        List versions = versionService.search(fields, filters, sorts, page, size);
        Long dataSetInt = versionService.getCount(filters);
        pagedResponse(request, response, dataSetInt, page, size);
        return convertToModels(versions, new ArrayList<>(versions.size()), MSTDVersion.class, fields);
    }


    @RequestMapping(value = ServiceApi.Standards.VersionLatestExistence, method = RequestMethod.GET)
    @ApiOperation(value = "判断是否最新版本")
    public boolean isLatestVersion(
            @ApiParam(name = "version", value = "版本号", defaultValue = "")
            @PathVariable(value = "version") String version) throws Exception {

        STDVersion cdaVersion = versionService.getLatestVersion();
        String latestVersion = cdaVersion.getVersion();
        return latestVersion != null && latestVersion.equals(version);
    }

    //新增标准版本的最新版本信息

    @RequestMapping(value = ServiceApi.Standards.Versions, method = RequestMethod.POST)
    @ApiOperation(value = "新增cda版本")
    public MSTDVersion addVersion(
            @ApiParam(name = "userLoginCode", value = "用户登录名")
            @RequestParam(value = "userLoginCode") String userLoginCode) throws Exception {

        STDVersion baseVersion = versionService.getLatestVersion();
        STDVersion xcdaVersion = versionService.createStageVersion(baseVersion, userLoginCode);
        if (xcdaVersion != null)
            return getModel(xcdaVersion);
        return null;
    }


    @RequestMapping(value = ServiceApi.Standards.Version, method = RequestMethod.DELETE)
    @ApiOperation(value = "删除版本")
    public boolean dropCDAVersion(
            @ApiParam(name = "version", value = "版本号", defaultValue = "")
            @PathVariable(value = "version") String version) throws Exception {

        versionService.dropVersion(findVersion(version));
        return true;
    }


    @RequestMapping(value = ServiceApi.Standards.VersionRevert, method = RequestMethod.DELETE)
    @ApiOperation(value = "删除编辑状态的版本")
    public boolean revertVersion(
            @ApiParam(name = "version", value = "版本号", defaultValue = "")
            @PathVariable(value = "version") String version) throws Exception {

        versionService.revertVersion(findVersionInstage(version));
        return true;
    }


    @RequestMapping(value = ServiceApi.Standards.VersionCommit, method = RequestMethod.PUT)
    @ApiOperation(value = "发布新版本")
    public boolean commitVersion(
            @ApiParam(name = "version", value = "版本号", defaultValue = "")
            @PathVariable(value = "version") String version) throws Exception {

        STDVersion cdaVersion = findVersionInstage(version);
//        Map<String, Object> mapResult = dispatchService.createFullVersionFile(cdaVersion.getVersion());
//        if (mapResult == null) return false;

        versionService.commitVersion(cdaVersion);
        return true;
    }


    @RequestMapping(value = ServiceApi.Standards.VersionBackStage, method = RequestMethod.PUT)
    @ApiOperation(value = "将最新的已发布版本回滚为编辑状态")
    public boolean rollbackToStage(
            @ApiParam(name = "version", value = "版本号", defaultValue = "")
            @PathVariable(value = "version") String version) throws Exception {

        STDVersion cdaVersion = versionService.getVersion(version);
        versionService.rollbackToStage(cdaVersion);

        return true;
    }


    @RequestMapping(value = ServiceApi.Standards.Version, method = RequestMethod.PUT)
    @ApiOperation(value = "修改版本信息")
    public MSTDVersion updateVersion(
            @ApiParam(name = "version", value = "版本号", defaultValue = "")
            @PathVariable(value = "version") String version,
            @ApiParam(name = "versionName", value = "版本名称", defaultValue = "")
            @RequestParam(value = "versionName") String versionName,
            @ApiParam(name = "userCode", value = "操作者", defaultValue = "")
            @RequestParam(value = "userCode") String userCode,
            @ApiParam(name = "inStage", value = "编辑状态", defaultValue = "")
            @RequestParam(value = "inStage") int inStage,
            @ApiParam(name = "baseVersion", value = "父版本", defaultValue = "")
            @RequestParam(value = "baseVersion") String baseVersion) throws Exception {

        STDVersion cdaVersion = findVersion(version);
        cdaVersion.setVersionName(versionName);
        cdaVersion.setAuthor(userCode);
        cdaVersion.setCommitTime(new Date());
        cdaVersion.setInStage(inStage == 1);
        cdaVersion.setBaseVersion(baseVersion);
        versionService.saveEntity(cdaVersion);
        return getModel(cdaVersion);
    }


    @RequestMapping(value = ServiceApi.Standards.VersionNameExistence, method = RequestMethod.GET)
    @ApiOperation(value = "判断版本名称是否已存在")
    public boolean checkVersionName(
            @ApiParam(name = "versionName", value = "版本名称", defaultValue = "")
            @RequestParam(value = "versionName") String versionName) throws Exception {

        int num = versionService.checkVersionName(versionName);
        return num > 0;
    }


    @RequestMapping(value = ServiceApi.Standards.VersionInStageExist, method = RequestMethod.GET)
    @ApiOperation(value = "检查是否存在处于编辑状态的版本")
    public boolean existInStage() throws Exception {
        int num = versionService.searchInStage();
        return num > 0;
    }

    @RequestMapping(value = ServiceApi.Standards.Version, method = RequestMethod.GET)
    @ApiOperation(value = "获取版本信息")
    public MSTDVersion getVersion(
            @ApiParam(name = "version", value = "版本号", defaultValue = "")
            @PathVariable(value = "version") String version) throws Exception {

        STDVersion cdaVersion = versionService.getVersion(version);
        return getModel(cdaVersion);
    }


    @RequestMapping(value = ServiceApi.Standards.VersionCache, method = RequestMethod.PUT)
    @ApiOperation(value = "向Redis中缓存标准数据")
    public void cache(
            @ApiParam(name = "version", value = "版本号", defaultValue = "")
            @PathVariable(value = "version") String version) {

    }

    @RequestMapping(value = ServiceApi.Standards.VersionLatest, method = RequestMethod.GET)
    @ApiOperation(value = "获取最新版本")
    public MSTDVersion getLatestVersion()throws Exception {
        STDVersion cdaVersion = versionService.getLatestVersion();
        return getModel(cdaVersion);
    }

}