package com.yihu.hos.ehr.standard.datasets.controller;

import com.fasterxml.jackson.core.type.TypeReference;
import com.yihu.hos.common.controller.ExtendEndPoint;
import com.yihu.hos.core.constants.MicroServices;
import com.yihu.hos.core.constants.ServiceApi;
import com.yihu.hos.model.standard.MStdDataSet;
import com.yihu.hos.standard.model.standard.StdDataSetModel;
import com.yihu.hos.standard.service.standard.StdDatasetService;
import com.yihu.hos.web.framework.model.Result;
import com.yihu.hos.web.framework.model.standard.StandardVersion;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * @author lincl
 * @version 1.0
 * @created 2016.2.15
 */
@RestController
@RequestMapping(MicroServices.ApiVersion.Version1_0)
@Api(value = "Dataset", description = "数据集服务")
public class DataSetEndPoint extends ExtendEndPoint<MStdDataSet> {

    @Autowired
    private StdDatasetService dataSetService;


    @RequestMapping(value = ServiceApi.Standards.DataSets, method = RequestMethod.GET)
    @ApiOperation(value = "查询数据集的方法")
    public Collection<MStdDataSet> searchDataSets(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段", defaultValue = "")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件", defaultValue = "")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档", defaultValue = "")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "size", value = "分页大小", defaultValue = "15")
            @RequestParam(value = "size", required = false) int size,
            @ApiParam(name = "page", value = "页码", defaultValue = "1")
            @RequestParam(value = "page", required = false) int page,
            @ApiParam(name = "version", value = "版本", defaultValue = "")
            @RequestParam(value = "version") String version,
            HttpServletRequest request,
            HttpServletResponse response) throws Exception{

        List ls = dataSetService.searchPage(StdDataSetModel.class, StandardVersion.DataSetTable,version, filters, sorts, page, size,true);
        List all = dataSetService.searchPage(StdDataSetModel.class, StandardVersion.DataSetTable, version, filters, sorts, null, null,true);
        long total = Long.valueOf(all.size());
        pagedResponse(request, response, total, page, size);
        return convertToModels(ls, new ArrayList<>(ls.size()), MStdDataSet.class, fields);
    }


    @RequestMapping(value = ServiceApi.Standards.NoPageDataSets, method = RequestMethod.GET)
    @ApiOperation(value = "标准数据集不分页搜索")
    public Collection<MStdDataSet> searchSourcesWithoutPaging(
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件", defaultValue = "")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "version", value = "版本", defaultValue = "")
            @RequestParam(value = "version") String version) throws Exception {
        List dataSetList = dataSetService.searchVersionData(StdDataSetModel.class,version,StandardVersion.DataSetTable,filters);
        return convertToModels(dataSetList, new ArrayList<>(dataSetList.size()), MStdDataSet.class, "");
    }


    @RequestMapping(value = ServiceApi.Standards.DataSet, method = RequestMethod.DELETE)
    @ApiOperation(value = "删除数据集信息")
    public boolean deleteDataSet(
            @ApiParam(name = "id", value = "数据集编号", defaultValue = "")
            @PathVariable(value = "id") long id,
            @ApiParam(name = "version", value = "版本", defaultValue = "")
            @RequestParam(value = "version") String version) throws Exception{
        dataSetService.delete(version,id);
        return true;
    }


    @RequestMapping(value = ServiceApi.Standards.DataSets, method = RequestMethod.DELETE)
    @ApiOperation(value = "删除数据集信息")
    public boolean deleteDataSet(
            @ApiParam(name = "ids", value = "数据集编号", defaultValue = "")
            @RequestParam(value = "ids") String ids,
            @ApiParam(name = "version", value = "版本", defaultValue = "")
            @RequestParam(value = "version") String version) throws Exception{
        if(StringUtils.isNotEmpty(ids)){
            for(String id:ids.split(",")){
                Long id_ = Long.valueOf(id);
                dataSetService.delete(version,id_);
            }
        }
        return true;
    }


    @RequestMapping(value = ServiceApi.Standards.DataSet, method = RequestMethod.GET)
    @ApiOperation(value = "获取数据集信息")
    public MStdDataSet getDataSet(
            @ApiParam(name = "id", value = "数据集编号", defaultValue = "")
            @PathVariable(value = "id") long id,
            @ApiParam(name = "version", value = "版本", defaultValue = "")
            @RequestParam(value = "version") String version) {
        StandardVersion ver = new StandardVersion(version);
        return getModel(dataSetService.retrieve(StdDataSetModel.class,ver.getDataSetTableName() ,id));
    }

    @RequestMapping(value = ServiceApi.Standards.DataSetsIds, method = RequestMethod.GET)
    @ApiOperation(value = "根据数据集ids(用逗号隔开)获取数据集信息")
    public Collection<MStdDataSet> getDataSets(
            @ApiParam(name = "ids", value = "数据集编号", defaultValue = "")
            @PathVariable(value = "ids") String ids,
            @ApiParam(name = "version", value = "版本", defaultValue = "")
            @RequestParam(value = "version") String version) throws IOException {
        List<StdDataSetModel> list = dataSetService.searchVersionData(StdDataSetModel.class,version,StandardVersion.DataSetTable, "id="+ids);
        return convertToModels(list, new ArrayList<>(list.size()), MStdDataSet.class, "");
    }

    @RequestMapping(value = ServiceApi.Standards.DataSets, method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ApiOperation(value = "新增数据集信息")
    public MStdDataSet saveDataSet(
            @ApiParam(name = "version", value = "标准版本", defaultValue = "")
            @RequestParam(value = "version") String version,
            @ApiParam(name = "model", value = "json数据模型", defaultValue = "")
            @RequestBody String model) throws Exception{


        StdDataSetModel dataSet =  jsonToObj(model, StdDataSetModel.class);
        Result result = dataSetService.add(version, model);
        if(result.isSuccessFlg()){
            return getModel(dataSet);
        }
        return null;
    }


    @RequestMapping(value = ServiceApi.Standards.DataSet, method = RequestMethod.PUT, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ApiOperation(value = "修改数据集信息")
    public MStdDataSet updateDataSet(
            @ApiParam(name = "version", value = "标准版本", defaultValue = "")
            @RequestParam(value = "version") String version,
            @ApiParam(name = "id", value = "数据集编号", defaultValue = "")
            @PathVariable(value = "id") long id,
            @ApiParam(name = "model", value = "json数据模型", defaultValue = "")
            @RequestBody String model) throws Exception{

        StdDataSetModel dataSetModel =  jsonToObj(model, StdDataSetModel.class);
//        StdDataSetModel dataSet = dataSetService.retrieve(id, entityClass);
//        if(!dataSet.getInnerCode().equals(dataSetModel.getInnerCode())){
//            if(dataSetService.isExistByField("code", dataSetModel.getInnerCode(), entityClass))
//                throw new ApiException(ErrorCode.RapeatDataSetCode, "代码重复！");
//        }
        dataSetModel.setId(id);
        Result result = dataSetService.update(version, toJson(dataSetModel));
        if(result.isSuccessFlg()){
            return getModel(dataSetModel);
        }
        return null;
    }


    @RequestMapping(value = ServiceApi.Standards.DataSetsName, method = RequestMethod.GET)
    @ApiOperation(value = "获取数据集 id-name : map集")
    public Map<Long,String> getDataSetMapByIds(
            @ApiParam(name = "version", value = "版本号", defaultValue = "")
            @RequestParam(value = "version") String version,
            @ApiParam(name = "ids", value = "数据集编号", defaultValue = "")
            @RequestParam(value = "ids",required = false) String ids) throws Exception{

        return dataSetService.getDataSetMapByIds(ids==null || ids.trim().length()==0? new String[]{} : ids.split(","), version);
    }

    @RequestMapping(value = ServiceApi.Standards.DataSetCodeIsExist,method = RequestMethod.GET)
    public boolean isExistCode(
            @RequestParam(value = "code")String code,
            @RequestParam(value = "version_code")String versionCode) throws IOException {
        return dataSetService.isExistByField(StdDataSetModel.class,versionCode,StandardVersion.DataSetTable,"code", code);
    }

    @RequestMapping(value = ServiceApi.Standards.DataSetsBatch, method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ApiOperation(value = "批量创建标准数据集以及数据元", notes = "批量创建标准数据集以及数据元")
    public boolean createDictAndEntries(
            @RequestParam(value = "version") String version,
            @RequestBody String jsonData) throws Exception {

        List models = objectMapper.readValue(jsonData, new TypeReference<List>() {});
        dataSetService.batchInsertDictsAndEntry(models, version);
        return true;
    }

    @RequestMapping(value = ServiceApi.Standards.AdaptMeta, method = RequestMethod.GET)
    @ApiOperation(value = "获取ehr适配的数据元", notes = "获取ehr适配的数据元")
    public List<Map<String, Object>> getEhrAdapterMetadata( @RequestParam(value = "version")String version){
        return dataSetService.getEhrAdapterMetadata(version);
    }
}