package com.yihu.hos.ehr.standard.datasets.controller;

import com.yihu.ehr.constants.ApiVersion;
import com.yihu.ehr.constants.ServiceApi;
import com.yihu.hos.common.controller.ExtendEndPoint;
import com.yihu.hos.crawler.model.standard.StdMetaData;
import com.yihu.hos.model.standard.MStdMetaData;
import com.yihu.hos.standard.model.standard.StdDataSetModel;
import com.yihu.hos.standard.model.standard.StdDictionaryModel;
import com.yihu.hos.standard.model.standard.StdMetaDataModel;
import com.yihu.hos.standard.service.standard.StdDatasetService;
import com.yihu.hos.standard.service.standard.StdDictService;
import com.yihu.hos.standard.service.standard.StdMetadataService;
import com.yihu.hos.web.framework.model.standard.StandardVersion;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.collections.CollectionUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URLDecoder;
import java.util.*;

/**
 * @author lincl
 * @version 1.0
 * @created 2016.2.3
 */
@RestController
@RequestMapping(ApiVersion.Version1_0)
@Api(value = "Meta data", description = "数据元服务")
public class MetaDataEndPoint extends ExtendEndPoint<MStdMetaData> {

    @Autowired
    private StdMetadataService metaDataService;


    @RequestMapping(value = ServiceApi.Standards.MetaDatas, method = RequestMethod.GET)
    @ApiOperation(value = "查询数据元")
    public Collection<MStdMetaData> searchMetaDatas(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段", defaultValue = "")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件", defaultValue = "")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档", defaultValue = "")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "size", value = "分页大小", defaultValue = "15")
            @RequestParam(value = "size", required = false) int size,
            @ApiParam(name = "page", value = "页码", defaultValue = "1")
            @RequestParam(value = "page", required = false) int page,
            @ApiParam(name = "version", value = "版本", defaultValue = "")
            @RequestParam(value = "version") String version,
            HttpServletRequest request,
            HttpServletResponse response) throws Exception {

        List ls;
        if (size == -1){
            ls = metaDataService.searchVersionData(StdMetaDataModel.class, version, StandardVersion.MetaDataTable, filters);
        } else {
            ls = metaDataService.searchPage(StdMetaDataModel.class,  StandardVersion.MetaDataTable,version, filters, sorts, page, size,true);
            List all = metaDataService.searchPage(StdMetaDataModel.class, StandardVersion.MetaDataTable, version, filters, sorts, null, null,true);
            long total = Long.valueOf(all.size());
            pagedResponse(request, response, total, page, size);
        }
        return convertToModels(ls, new ArrayList<>(ls.size()), MStdMetaData.class, fields);
    }

    @RequestMapping(value = ServiceApi.Standards.MetaDatas, method = RequestMethod.DELETE)
    @ApiOperation(value = "批量删除数据元")
    public boolean deleteMetaDatas(
            @ApiParam(name = "ids", value = "编号集", defaultValue = "")
            @RequestParam(value = "ids") String ids,
            @ApiParam(name = "version", value = "版本", defaultValue = "")
            @RequestParam(value = "version") String version) throws Exception {

        metaDataService.delete(StdMetaDataModel.class, strToLongArr(ids), StandardVersion.MetaDataTable, version);
        return true;
    }


    @RequestMapping(value = ServiceApi.Standards.MetaDatasWithDataSet, method = RequestMethod.DELETE)
    @ApiOperation(value = "删除数据集关联的数据元")
    public boolean deleteMetaDataByDataSet(
            @ApiParam(name = "data_set_id", value = "数据集编号", defaultValue = "")
            @PathVariable(value = "data_set_id") long dataSetId,
            @ApiParam(name = "version", value = "版本", defaultValue = "")
            @RequestParam(value = "version") String version) throws Exception {

        return metaDataService.deleteByField("dataSetId", new Object[]{dataSetId}, StandardVersion.MetaDataTable, version);
    }


    @RequestMapping(value = ServiceApi.Standards.MetaData, method = RequestMethod.DELETE)
    @ApiOperation(value = "删除数据元")
    public boolean deleteMetaData(
            @ApiParam(name = "id", value = "编号集", defaultValue = "")
            @PathVariable(value = "id") long id,
            @ApiParam(name = "version", value = "版本", defaultValue = "")
            @RequestParam(value = "version") String version) throws Exception {

        metaDataService.delete(StdMetaData.class, new Object[]{id}, StandardVersion.MetaDataTable, version);
        return true;
    }


    @RequestMapping(value = ServiceApi.Standards.MetaData, method = RequestMethod.GET)
    @ApiOperation(value = "获取数据元")
    public MStdMetaData getMetaData(
            @ApiParam(name = "id", value = "数据元编号", defaultValue = "")
            @PathVariable(value = "id") long id,
            @ApiParam(name = "version", value = "版本", defaultValue = "")
            @RequestParam(value = "version") String version) throws Exception {
        StandardVersion ver = new StandardVersion(version);
        return getModel(metaDataService.retrieve(StdMetaDataModel.class,ver.getMetaDataTableName(),id));
    }

    @RequestMapping(value = ServiceApi.Standards.MetaDatasWithDataSet, method = RequestMethod.GET)
    @ApiOperation(value = "根据数据集id获取数据元")
    public Collection<MStdMetaData> getMetaDataByDataSetId(
            @ApiParam(name = "data_set_id", value = "数据元编号", defaultValue = "")
            @PathVariable(value = "data_set_id") long dataSetIs,
            @ApiParam(name = "version", value = "版本", defaultValue = "")
            @RequestParam(value = "version") String version) throws Exception {
        List<StdMetaDataModel> list = metaDataService.searchVersionData(StdMetaDataModel.class,version,StandardVersion.MetaDataTable, "dataSetId=" + dataSetIs);
        return convertToModels(list, new ArrayList<>(list.size()), MStdMetaData.class, "");
    }

    @RequestMapping(value = ServiceApi.Standards.MetaData, method = RequestMethod.PUT, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ApiOperation(value = "更新数据元")
    public MStdMetaData updataMetaSet(
            @ApiParam(name = "version", value = "版本", defaultValue = "")
            @RequestParam(value = "version") String version,
            @ApiParam(name = "id", value = "数据元编号", defaultValue = "")
            @PathVariable(value = "id") long id,
            @ApiParam(name = "model", value = "数据源模型", defaultValue = "")
            @RequestBody String model) throws Exception {

        StdMetaDataModel metaDataModel =  jsonToObj(model, StdMetaDataModel.class);
//        StdMetaDataModel metaData = metaDataService.retrieve(id, entityClass);
//        if(metaData.getId()==0)
//            throw errNotFound("数据元", metaDataModel.getId());

        metaDataModel.setId(id);
        metaDataService.saveEntity(metaDataModel);
        return getModel(metaDataModel);
    }


    @RequestMapping(value = ServiceApi.Standards.MetaDatas, method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ApiOperation(value = "新增数据元")
    public MStdMetaData saveMetaSet(
            @ApiParam(name = "version", value = "版本", defaultValue = "")
            @RequestParam(value = "version") String version,
            @ApiParam(name = "model", value = "数据源模型", defaultValue = "")
            @RequestBody String model) throws Exception {

        StdMetaDataModel metaData =  jsonToObj(model,StdMetaDataModel.class);
        StandardVersion standardVersion = new StandardVersion(version);
        return getModel(metaDataService.add(standardVersion,metaData));
    }


    @RequestMapping(value = ServiceApi.Standards.MetaDataCodeExistence, method = RequestMethod.GET)
    @ApiOperation(value = "验证数据元代码是否重复")
    public boolean validateInnerCode(
            @ApiParam(name = "version", value = "版本号", defaultValue = "")
            @RequestParam(value = "version") String version,
            @ApiParam(name = "data_set_id", value = "数据集编号", defaultValue = "")
            @PathVariable(value = "data_set_id") long dataSetId,
            @ApiParam(name = "inner_code", value = "查询代码", defaultValue = "")
            @RequestParam(value = "inner_code") String innerCode) throws Exception {

        return metaDataService.isColumnValExsit(dataSetId, "innerCode", innerCode,StdMetaDataModel.class,version,StandardVersion.MetaDataTable);
    }


    @RequestMapping(value = ServiceApi.Standards.MetaDataNameExistence, method = RequestMethod.GET)
    @ApiOperation(value = "验证数据元名称是否重复")
    public boolean validatorName(
            @ApiParam(name = "version", value = "版本号", defaultValue = "")
            @RequestParam(value = "version") String version,
            @ApiParam(name = "data_set_id", value = "数据集编号", defaultValue = "")
            @RequestParam(value = "data_set_id") long dataSetId,
            @ApiParam(name = "name", value = "查询名称", defaultValue = "")
            @RequestParam(value = "name") String name) throws Exception {

        return metaDataService.isColumnValExsit(dataSetId, "name", name, StdMetaDataModel.class,version,StandardVersion.MetaDataTable);
    }


    @RequestMapping(value = ServiceApi.Standards.MetaDatasName, method = RequestMethod.POST)
    @ApiOperation(value = "获取数据元 id-name : map集")
    public Map<Long,Object> getMetaDataMapByIds(
            @ApiParam(name = "parmModel", value = "参数模型", defaultValue = "")
            @RequestBody String parmModel) throws IOException {

        parmModel = URLDecoder.decode(parmModel, "UTF-8");
        Map<String, Object> parms = jsonToObj(parmModel, Map.class);
        List<String> metaIds = (List) parms.get("metaIds");
        Object[] rs = null;
        if (metaIds != null && metaIds.size() > 0) {
            rs = new Object[metaIds.size()];
            for (int i = 0; i < metaIds.size(); i++) {
                rs[i] = metaIds.get(i);
            }
        }
        return metaDataService.getMetaDataMapByIds(rs, parms.get("version")+"");
    }


    @Autowired
    private StdDatasetService dataSetService;
    @Autowired
    private StdDictService dictService;

    @RequestMapping(value = "importMetadata", method = RequestMethod.GET)
    @ApiOperation(value = "导入数据元啊")
    public boolean importFile() throws IOException {
        File file = new File("D:/aa.xls.");
        InputStream inputStream = new FileInputStream(file);
        //根据文件后缀名不同(xls和xlsx)获得不同的Workbook实现类对象
        Workbook workbook = new HSSFWorkbook(inputStream);
        Iterator<Sheet> sheets = workbook.sheetIterator();
        int jj=0;
        while (sheets.hasNext()) {
            Sheet sheet = sheets.next();
            int rows = sheet.getLastRowNum();
            for(int i=1;i<rows;i++){
                Cell cell = sheet.getRow(i).getCell(0);
                String datasetCode = cell.getStringCellValue();
                cell = sheet.getRow(i).getCell(1);
                String metadataCode = cell.getStringCellValue();
                //通过 这2个code值查找是否存在在当前标准中,存在则不存,不存在则存入数据库中
                List<StdDataSetModel> datasets = dataSetService.searchPage(StdDataSetModel.class, StandardVersion.DataSetTable, "59083976eebd", "code="+datasetCode, null, null, null,true);
                if(CollectionUtils.isEmpty(datasets)){
                    throw new RuntimeException("查找不到数据集:"+datasetCode);
                }
                if(datasets.size()>1){
                    throw new RuntimeException("查找到多个数据集:"+datasetCode+",个数为:"+datasets.size());
                }
                Long id = datasets.get(0).getId();
                List all = metaDataService.searchPage(StdMetaDataModel.class, StandardVersion.MetaDataTable, "59083976eebd", "innerCode="+metadataCode+";dataSetId="+id, null, null, null,true);
                boolean needAdd = false;
                if(all==null || all.size()==0){
                    needAdd=true;
                }else if(all.size()==1){
                    needAdd=false;
                }else if(all.size()>1){
                    throw new RuntimeException("查找出来的数据元多个datasetCode:"+datasetCode+",metadatacode:"+metadataCode);
                }
                if(needAdd){
                    //查找eip版本
                    List<StdDataSetModel> _datasets = dataSetService.searchPage(StdDataSetModel.class, StandardVersion.DataSetTable, "5a619250f719", "code="+datasetCode, null, null, null,true);

                    if(CollectionUtils.isEmpty(_datasets) && !"HDSB05_81".equalsIgnoreCase(datasetCode)){
                        throw new RuntimeException("eip查找不到数据集:"+datasetCode);
                    }
                    if(datasets.size()>1){
                        throw new RuntimeException("eip查找到多个数据集:"+datasetCode+",个数为:"+datasets.size());
                    }
                    Long _id = null;
                    if("HDSB05_81".equalsIgnoreCase(datasetCode)){
                        _id=11111L;
                    }else{
                        _id = _datasets.get(0).getId();
                    }

                    List<StdMetaDataModel> _all = metaDataService.searchPage(StdMetaDataModel.class, StandardVersion.MetaDataTable, "5a619250f719", "innerCode="+metadataCode+";dataSetId="+_id, null, null, null,true);
                    if("JDSA00_01_001".equalsIgnoreCase(metadataCode) && CollectionUtils.isEmpty(_all)){
                        _all = metaDataService.searchPage(StdMetaDataModel.class, StandardVersion.MetaDataTable, "5a619250f719", "innerCode="+metadataCode+";dataSetId=64", null, null, null,true);
                    }
                    if(CollectionUtils.isEmpty(_all)){
                        throw new RuntimeException("异常了啊.........dataset:"+datasetCode+",metadacode:"+metadataCode);
                    }else if(_all.size()==1){
                        StdMetaDataModel metaData =  _all.get(0);
                        metaData.setId(null);
                        metaData.setStandardId(10L);
                        metaData.setDataSetId(id);
                        Long _dictId = metaData.getDictId();

                        if(_dictId!=null && _dictId==0){
                            //通过此dictId查找对应的dictCode,
                            List<StdDictionaryModel> _is = dictService.searchPage(StdDictionaryModel.class, StandardVersion.DictTable,"5a619250f719", "id="+_dictId, null, null, null,true);
                            if(CollectionUtils.isEmpty(_is) || _is.size()>1){
                                throw new RuntimeException("eip错了啊.......查找出了多条字典");
                            }
                            String dictCode = _is.get(0).getCode();
                            //通过dictCode查找ehr版本的dictid
                            List<StdDictionaryModel> is = dictService.searchPage(StdDictionaryModel.class, StandardVersion.DictTable,"5a619250f719", "id="+_dictId, null, null, null,true);
                            if(CollectionUtils.isEmpty(is) || is.size()>1){
                                throw new RuntimeException("错了啊.......查找出了多条字典");
                            }
                            Long dictId = is.get(0).getId();
                            metaData.setDictId(dictId);
                        }

                        StandardVersion standardVersion = new StandardVersion("59083976eebd");
                        metaDataService.add(standardVersion,metaData);
                        jj++;
                    }else if(_all.size()>1){
                        throw new RuntimeException("异常了啊.........dataset:"+datasetCode+",metadacode:"+metadataCode+"=========多条啊");
                    }
                }

            }

        }
        System.out.println("插入了辣么多条的数据啊:"+jj);
        return true;
    }

}