package com.yihu.hos.ehr.standard.dict.controller;

import com.fasterxml.jackson.core.type.TypeReference;
import com.yihu.hos.common.controller.ExtendEndPoint;
import com.yihu.hos.core.constants.MicroServices;
import com.yihu.hos.core.constants.ServiceApi;
import com.yihu.hos.model.standard.MStdDict;
import com.yihu.hos.standard.model.standard.StdDictionaryModel;
import com.yihu.hos.standard.service.standard.StdDictService;
import com.yihu.hos.web.framework.model.Result;
import com.yihu.hos.web.framework.model.standard.StandardVersion;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * @author lincl
 * @version 1.0
 * @created 2016.2.1
 */
@RestController
@RequestMapping(MicroServices.ApiVersion.Version1_0)
@Api(value = "Dictionary", description = "字典服务")
public class DictEndPoint extends ExtendEndPoint<MStdDict> {

    @Autowired
    private StdDictService dictService;



    @RequestMapping(value = ServiceApi.Standards.Dictionaries, method = RequestMethod.GET)
    @ApiOperation(value = "查询字典")
    public Collection<MStdDict> searchDict(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段", defaultValue = "")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件", defaultValue = "")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档", defaultValue = "")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "size", value = "分页大小", defaultValue = "15")
            @RequestParam(value = "size", required = false) int size,
            @ApiParam(name = "page", value = "页码", defaultValue = "1")
            @RequestParam(value = "page", required = false) int page,
            @ApiParam(name = "version", value = "版本", defaultValue = "")
            @RequestParam(value = "version") String version,
            HttpServletRequest request,
            HttpServletResponse response) throws Exception{

        List<StdDictionaryModel> ls = dictService.searchPage(StdDictionaryModel.class, StandardVersion.DictTable,version, filters, sorts, page, size,true);
        List list = dictService.searchPage(StdDictionaryModel.class, StandardVersion.DictTable, version, filters, sorts, null, null,true);
        long count  =(long)list.size();
        pagedResponse(request, response, count, page, size);
        return convertToModels(ls, new ArrayList<>(ls.size()), MStdDict.class, fields);
    }


    @RequestMapping(value = ServiceApi.Standards.NoPageDictionaries, method = RequestMethod.GET)
    @ApiOperation(value = "标准字典不分页搜索")
    public Collection<MStdDict> searchSourcesWithoutPaging(
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件", defaultValue = "")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "version", value = "版本", defaultValue = "")
            @RequestParam(value = "version") String version) throws Exception {
        List dictList = dictService.searchVersionData(StdDictionaryModel.class,version,StandardVersion.DictTable,filters);
        return convertToModels(dictList, new ArrayList<>(dictList.size()), MStdDict.class, "");
    }


    @RequestMapping(value = ServiceApi.Standards.Dictionaries, method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ApiOperation(value = "新增字典")
    public MStdDict addDict(
            @ApiParam(name = "version", value = "标准版本", defaultValue = "")
            @RequestParam(value = "version") String version,
            @ApiParam(name = "model", value = "json数据模型", defaultValue = "")
            @RequestBody String model) throws Exception{

        StdDictionaryModel dict =  jsonToObj(model, StdDictionaryModel.class);
//        if(dictService.isExistByField("code", dict.getInnerCode(), entityClass))
//            throw errRepeatCode();

        Result result = dictService.add(version, model);
        if(result.isSuccessFlg()){
            return getModel(dict);
        }
        return null;
    }


    @RequestMapping(value = ServiceApi.Standards.Dictionary, method = RequestMethod.PUT, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ApiOperation(value = "修改字典")
    public MStdDict updateDict(
            @ApiParam(name = "version", value = "标准版本", defaultValue = "")
            @RequestParam(value = "version") String version,
            @ApiParam(name = "id", value = "编号", defaultValue = "")
            @PathVariable(value = "id") Long id,
            @ApiParam(name = "model", value = "json数据模型", defaultValue = "")
            @RequestBody String model) throws Exception{

        StdDictionaryModel dictModel =  jsonToObj(model, StdDictionaryModel.class);
//        StdDictionaryModel dict = dictService.retrieve(id, entityClass);
//        if(!dict.getInnerCode().equals(dictModel.getInnerCode()) &&
//                dictService.isExistByField("code", dictModel.getInnerCode(), entityClass))
//            throw errRepeatCode();
        dictModel.setId(id);
        Result result = dictService.update(version, toJson(dictModel));
        if(result.isSuccessFlg()){
            return getModel(dictModel);
        }
        return null;
    }


    @RequestMapping(value = ServiceApi.Standards.Dictionary, method = RequestMethod.DELETE)
    @ApiOperation(value = "删除字典")
    public boolean deleteDict(
            @ApiParam(name = "version", value = "cda版本号", defaultValue = "")
            @RequestParam(value = "version") String version,
            @ApiParam(name = "id", value = "编号", defaultValue = "")
            @PathVariable(value = "id") long id) throws Exception{
        dictService.delete(version,id);
        return true;
    }


    @RequestMapping(value = ServiceApi.Standards.Dictionaries, method = RequestMethod.DELETE)
    @ApiOperation(value = "批量删除字典")
    public boolean deleteDicts(
            @ApiParam(name = "version", value = "cda版本号", defaultValue = "")
            @RequestParam(value = "version") String version,
            @ApiParam(name = "ids", value = "编号", defaultValue = "")
            @RequestParam(value = "ids") String ids) throws Exception{
        if(StringUtils.isNotEmpty(ids)){
            for(String id:ids.split(",")){
                Long id_ = Long.valueOf(id);
                dictService.delete(version,id_);
            }
        }
        return true;
    }


    @RequestMapping(value = ServiceApi.Standards.Dictionary, method = RequestMethod.GET)
    @ApiOperation(value = "获取字典详细信息")
    public MStdDict getCdaDictInfo(
            @ApiParam(name = "id", value = "字典编号", defaultValue = "")
            @PathVariable(value = "id") long id,
            @ApiParam(name = "version", value = "版本编号", defaultValue = "")
            @RequestParam(value = "version") String version) throws Exception{
        Result result =  dictService.get(version, id);
        if(result.isSuccessFlg()){
            //StdDictionaryModel re = (StdDictionaryModel)result;
            //MStdDict mStdDict = convertToModel(re, MStdDict.class, "");
            return getModel(result);
            //return mStdDict;
        }
        return null;
    }


    @RequestMapping(value = ServiceApi.Standards.DictionaryCode, method = RequestMethod.GET)
    @ApiOperation(value = "获取字典详细信息")
    public MStdDict getDictByCode(
            @ApiParam(name = "code", value = "字典代码", defaultValue = "")
            @RequestParam(value = "code") String code,
            @ApiParam(name = "version", value = "版本编号", defaultValue = "")
            @RequestParam(value = "version") String version) throws Exception{

        StdDictionaryModel baseDict = dictService.getByCode(code, version);
        return baseDict==null? null : getModel(baseDict);
    }


    @RequestMapping(value = ServiceApi.Standards.MetaDataWithDict, method = RequestMethod.GET)
    @ApiOperation(value = "获取字典 map集")
    public Map getDictMapByIds(
            @ApiParam(name = "version", value = "版本号", defaultValue = "")
            @RequestParam(value = "version") String version,
            @ApiParam(name = "data_set_id", value = "数据集编号", defaultValue = "")
            @PathVariable(value = "data_set_id") Long dataSetId,
            @ApiParam(name = "meta_data_id", value = "数据元编号", defaultValue = "")
            @PathVariable(value = "meta_data_id") Long metaDataId) {

        return dictService.getDictMapByIds(version, dataSetId, metaDataId);
    }

    @RequestMapping(value = ServiceApi.Standards.DictCodeIsExist,method = RequestMethod.GET)
    public boolean isExistDictCode(
            @RequestParam(value = "dict_code")String dictCode,
            @RequestParam("version_code")String versionCode) throws IOException {
        return dictService.isExistByField(StdDictionaryModel.class, versionCode,StandardVersion.DictTable,"code",dictCode);
    }


    @RequestMapping(value = ServiceApi.Standards.DictParent, method = RequestMethod.GET)
    @ApiOperation(value = "根据当前类别获取自己的父级以及同级以及同级所在父级类别列表")
    public List<MStdDict> getCdaTypeExcludeSelfAndChildren(
            @ApiParam(name = "id", value = "id")
            @RequestParam(value = "id") long id,
            @ApiParam(name = "version", value = "版本编号", defaultValue = "")
            @RequestParam(value = "version") String version) throws Exception {
        List<StdDictionaryModel> dicts = new ArrayList<>();
        StandardVersion ver = new StandardVersion(version);
        StdDictionaryModel dict = dictService.retrieve(StdDictionaryModel.class,ver.getDictTableName(), id);
        dicts.add(dict);
        String childrenIds = getChildIncludeSelfByParentsAndChildrenIds(dicts,"",version);   //递归获取
        List<StdDictionaryModel> returnDicts = dictService.getCdaTypeExcludeSelfAndChildren(childrenIds,version);

        return  (List<MStdDict>)convertToModels(returnDicts,new ArrayList<MStdDict>(returnDicts.size()),MStdDict.class,"");
    }


    /**
     * 根据父级类别获取父级类别所在以下所有子集类别（包括当前父级列表）
     * @param parents 父级信息
     * @param childrenIds   子级ID
     */
    public String getChildIncludeSelfByParentsAndChildrenIds(List<StdDictionaryModel> parents, String childrenIds, String version) {
        for (int i = 0; i < parents.size(); i++) {
            StdDictionaryModel dict = parents.get(i);
            childrenIds+=dict.getId()+",";
            List<StdDictionaryModel> listChild = dictService.getChildrensByParentId(dict.getId(),version);
            if(listChild.size()>0){
                childrenIds = getChildIncludeSelfByParentsAndChildrenIds(listChild,childrenIds,version);
            }
        }
        return childrenIds;
    }

    @RequestMapping(value = ServiceApi.Standards.DictEntryBatch, method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ApiOperation(value = "批量创建标准字典以及字典项", notes = "批量创建标准字典以及字典项")
    public boolean createDictAndEntries(
            @RequestBody String jsonData, @RequestParam(value = "version")String version) throws Exception {

        List models = objectMapper.readValue(jsonData, new TypeReference<List>() {});
        dictService.batchInsertDictsAndEntry(models,version);
        return true;
    }

    @RequestMapping(value = ServiceApi.Standards.DictCodesExistence,method = RequestMethod.POST)
    @ApiOperation("获取已存在字典编码")
    public List<String> codeExistence(
            @RequestBody String codes, @RequestParam(value = "version")String version) throws Exception {
        List existCodes = dictService.codeExist(toEntity(codes, String[].class),version);
        return existCodes;
    }

    @RequestMapping(value = ServiceApi.Standards.AdaptDict, method = RequestMethod.GET)
    @ApiOperation(value = "获取ehr适配的数据元", notes = "获取ehr适配的数据元")
    public List<Map<String, Object>> getEhrAdapterMetadata( @RequestParam(value = "version")String version){
        return dictService.getEhrAdapterDict(version);
    }

}