package com.yihu.hos.ehr.standard.dict.controller;

import com.yihu.ehr.constants.ApiVersion;
import com.yihu.ehr.constants.ServiceApi;
import com.yihu.hos.common.controller.ExtendEndPoint;
import com.yihu.hos.model.standard.MStdDictEntry;
import com.yihu.hos.standard.model.standard.StdDictionaryEntryModel;
import com.yihu.hos.standard.service.standard.StdDictEntryService;
import com.yihu.hos.web.framework.model.Result;
import com.yihu.hos.web.framework.model.standard.StandardVersion;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * @author lincl
 * @version 1.0
 * @created 2016.2.1
 */
@RestController
@RequestMapping(ApiVersion.Version1_0)
@Api(value = "Entry", description = "字典项服务")
public class DictEntryEndPoint extends ExtendEndPoint<MStdDictEntry> {

    @Autowired
    private StdDictEntryService dictEntryService;



    @RequestMapping(value = ServiceApi.Standards.Entry, method = RequestMethod.PUT, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ApiOperation(value = "修改字典项")
    public MStdDictEntry updateDictEntry(
            @ApiParam(name = "version", value = "标准版本", defaultValue = "")
            @RequestParam(value = "version") String version,
            @ApiParam(name = "id", value = "标准版本", defaultValue = "")
            @PathVariable(value = "id") Long id,
            @ApiParam(name = "model", value = "json数据模型", defaultValue = "")
            @RequestBody String model) throws Exception{

        StdDictionaryEntryModel dictEntryModel =  jsonToObj(model, StdDictionaryEntryModel.class);
//        StdDictionaryEntryModel dictEntry = dictEntryService.retrieve(id, entityClass);
//        if(!dictEntry.getInnerCode().equals(dictEntryModel.getInnerCode())
//                && dictEntryService.isExistByField("code", dictEntryModel.getInnerCode(), entityClass))
//            throw errRepeatCode();

        dictEntryModel.setId(id);
        Result result = dictEntryService.update(version, toJson(dictEntryModel));
        if(result.isSuccessFlg()){
            return getModel(version);
        }
        return null;
    }


    @RequestMapping(value = ServiceApi.Standards.Entries, method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ApiOperation(value = "新增字典项")
    public MStdDictEntry addDictEntry(
            @ApiParam(name = "version", value = "cda版本号", defaultValue = "")
            @RequestParam(value = "version") String version,
            @ApiParam(name = "model", value = "json数据模型", defaultValue = "")
            @RequestBody String model) throws Exception{

        StdDictionaryEntryModel dictEntry =  jsonToObj(model, StdDictionaryEntryModel.class);
//        if(dictEntryService.isExistByField("code", dictEntry.getInnerCode(), entityClass))
//            throw errRepeatCode();

        Result result = dictEntryService.add(version, model);
        if(result.isSuccessFlg()){
            return getModel(dictEntry);
        }
        return null;
    }


    @RequestMapping(value = ServiceApi.Standards.Entry, method = RequestMethod.DELETE)
    @ApiOperation(value = "删除字典项")
    public boolean deleteDictEntry(
            @ApiParam(name = "version", value = "cda版本号", defaultValue = "")
            @RequestParam(value = "version") String version,
            @ApiParam(name = "id", value = "字典项编号", defaultValue = "")
            @PathVariable(value = "id") long id) throws Exception{
        dictEntryService.delete(version,id);
        return true;
    }


    @RequestMapping(value = ServiceApi.Standards.Entries, method = RequestMethod.DELETE)
    @ApiOperation(value = "删除字典项")
    public boolean deleteDictEntrys(
            @ApiParam(name = "version", value = "cda版本号", defaultValue = "")
            @RequestParam(value = "version") String version,
            @ApiParam(name = "ids", value = "字典项编号", defaultValue = "")
            @RequestParam(value = "ids") String ids) throws Exception{

        if(StringUtils.isNotEmpty(ids)){
            for(String id:ids.split(",")){
                Long id_ = Long.valueOf(id);
                dictEntryService.delete(version,id_);
            }
        }
        return true;
    }


    @RequestMapping(value = ServiceApi.Standards.EntriesWithDictionary, method = RequestMethod.DELETE)
    @ApiOperation(value = "删除字典关联的所有字典项")
    public boolean deleteDictEntryList(
            @ApiParam(name = "version", value = "cda版本号", defaultValue = "")
            @RequestParam(value = "version") String version,
            @ApiParam(name = "dict_id", value = "字典编号", defaultValue = "")
            @PathVariable(value = "dict_id") long dictId) throws Exception{

        return dictEntryService.deleteByField("dictId", new Object[]{dictId}, StandardVersion.DictEntryTable,version);
    }


    @RequestMapping(value = ServiceApi.Standards.Entries, method = RequestMethod.GET)
    @ApiOperation(value = "查询字典项")
    public Collection<MStdDictEntry> searchDictEntry(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段", defaultValue = "")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件", defaultValue = "")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档", defaultValue = "")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "size", value = "分页大小", defaultValue = "15")
            @RequestParam(value = "size", required = false) int size,
            @ApiParam(name = "page", value = "页码", defaultValue = "1")
            @RequestParam(value = "page", required = false) int page,
            @ApiParam(name = "version", value = "版本", defaultValue = "")
            @RequestParam(value = "version") String version,
            HttpServletRequest request,
            HttpServletResponse response) throws Exception{

        List ls = dictEntryService.searchPage(StdDictionaryEntryModel.class,StandardVersion.DictEntryTable,version, filters, sorts, page, size,true);
        List list = dictEntryService.searchPage(StdDictionaryEntryModel.class,StandardVersion.DictEntryTable,version, filters, null, null, null,true);
        long count =(long) list.size();
        pagedResponse(request, response, count, page, size);
        return convertToModels(ls, new ArrayList<>(ls.size()), MStdDictEntry.class, fields);
    }


    @RequestMapping(value = ServiceApi.Standards.Entry, method = RequestMethod.GET)
    @ApiOperation(value = "获取字典项")
    public MStdDictEntry getDictEntry(
            @ApiParam(name = "id", value = "字典项编号", defaultValue = "")
            @PathVariable(value = "id") long id,
            @ApiParam(name = "version", value = "版本编号", defaultValue = "")
            @RequestParam(value = "version") String version) throws Exception{
        Result result =  dictEntryService.get(version, id);
        if(result.isSuccessFlg()){
            return getModel(result);
        }
        return null;
    }

    @RequestMapping(value = ServiceApi.Standards.EntryCodeIsExist,method = RequestMethod.GET)
    public boolean isExistEntryCode(
            @RequestParam(value = "dict_id")long dictId,
            @RequestParam(value = "code")String code,
            @RequestParam(value = "version_code")String versionCode) throws IOException {
        return dictEntryService.isExistByFields(StdDictionaryEntryModel.class,versionCode,StandardVersion.DictEntryTable,new String[]{"dictId","code"}, new Object[]{dictId,code});
    }
}
