package com.yihu.hos.h3cadapter;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.hos.standard.model.adapter.AdapterDatasetModel;
import com.yihu.hos.standard.model.adapter.AdapterMetadataModel;
import com.yihu.hos.standard.service.adapter.AdapterDatasetService;
import com.yihu.hos.standard.service.adapter.AdapterMetadataService;
import com.yihu.hos.web.framework.dao.SQLGeneralDAO;
import com.yihu.hos.web.framework.model.Result;
import com.yihu.hos.web.framework.model.standard.AdapterVersion;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.util.List;

/**
 * h3c & jkzl 标准转换-- 测试
 */
@Transactional
@Service("H3cTransform")
public class H3cTransform extends SQLGeneralDAO {

    public static final String BEAN_ID = "H3cTransform";

    @Resource(name = AdapterDatasetService.BEAN_ID)
    private AdapterDatasetService adapterDatasetService;
    @Resource(name = AdapterMetadataService.BEAN_ID)
    private AdapterMetadataService adapterMetadataService;

    @Autowired
    private ObjectMapper objectMapper;


    //添加outTable step信息
    public Result addStep(String schemaXmlPath, String toXmlPath, String version, Long stdDatasetId) {
        try {
            //xml修改
            Document schemaXML = genStepNode(schemaXmlPath, toXmlPath, version, stdDatasetId);
            boolean b1 = false;
            b1 = writeToXml(schemaXML, toXmlPath);

            if (b1) {
                return Result.success("修改xml完成！");
            } else {
                return Result.error("修改xml失败！！");
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
        return Result.error("修改异常，请处理！！");
    }

    /**
     * 添加step的映射节点 （jkzl to h3c)
     *
     * @param schemaXmlPath h3c基础XMl
     * @param toXmlPath     新生成的xml路径
     * @param version       适配版本
     * @param stdDatasetId  数据集ID
     * @return
     */
    public Document genStepNode(String schemaXmlPath, String toXmlPath, String version, Long stdDatasetId) {
        Document document = null;
        List<AdapterMetadataModel> adapterMetadataNotNullList = adapterMetadataService.getAdapterMetadataNotNullList(new AdapterVersion(version), stdDatasetId, null);
        AdapterDatasetModel adapterDataset = adapterDatasetService.getAdapterDatasetByStdId(version, stdDatasetId);
        File file = new File(schemaXmlPath);
        try {
            SAXReader reader = new SAXReader();
            document = reader.read(file);
            //获取文档根节点
            Element root = document.getRootElement();
            //输出根标签的名字
            System.out.println(root.getName());
            //获取根节点下面的所有子节点（不包过子节点的子节点）
            List<Element> list = root.elements();
            //遍历List的方法
            list.stream().filter(e -> "step".equals(e.getName()))
                    .filter(step -> "TableOutput".equals(step.element("type").getStringValue()))
                    .forEach(outStep -> {
                        outStep.element("table").setText(adapterDataset.getStdDatasetCode());
                        Element element = outStep.element("fields");
                        //TODO 添加field节点
                        for (AdapterMetadataModel metadataModel : adapterMetadataNotNullList) {
                            //TODO 设置h3c的xml内容
                            Element field = DocumentHelper.createElement("field");
                            field.addElement("column_name").setText(metadataModel.getStdMetadataCode());
                            field.addElement("stream_name").setText(metadataModel.getAdapterMetadataCode());
//                            field.addAttribute("column_name",metadataModel.getAdapterMetadataCode());
//                            field.addAttribute("stream_name",metadataModel.getStdMetadataCode());
                            element.add(field);
                            System.out.println(metadataModel);
                        }

                    });


        } catch (Exception e) {
            e.printStackTrace();
        }

        return document;
    }


    /**
     * 写xml文件
     *
     * @param document xml的document信息
     * @param toPath   保存地址
     * @return
     */
    public static boolean writeToXml(Document document, String toPath) {
        boolean succ = false;
        try {
            OutputFormat format = OutputFormat.createPrettyPrint();
            format.setEncoding("UTF-8");
            XMLWriter writer = new XMLWriter(
                    new OutputStreamWriter(new FileOutputStream(toPath)), format);
            writer.write(document);
            writer.close();
            succ = true;
        } catch (Exception e) {
            System.out.println("Can't write to file: " + toPath);
            e.printStackTrace();
        }
        return succ;
    }

}
