package com.yihu.hos.interceptor;

import com.yihu.hos.tenant.model.TenantModel;
import com.yihu.hos.tenant.service.TenantService;
import com.yihu.hos.web.framework.constant.ContextAttributes;
import com.yihu.hos.web.framework.thread.LocalContext;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 *  web 请求拦截
 * @author HZY
 * @vsrsion 1.0
 * Created at 2016/12/26.
 */
@Configuration
public class WebMvcInterceptor extends HandlerInterceptorAdapter {

    @Resource(name = TenantService.BEAN_ID)
    private TenantService tenantService;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler)
            throws Exception {
        LocalContext.getContext().removeAttachment(ContextAttributes.SCHEMA);
        LocalContext.getContext().removeAttachment(ContextAttributes.TENANT_NAME);
        Object tenant = request.getHeader(ContextAttributes.TENANT_NAME);
        if(tenant==null){ //如果header没有传 tenant_name,从通过 request.getParameter("tenant_name")  获取 (eip前端通过zuul代理出来的地址,不好将tenant_name设置在请求头中)
            tenant = request.getParameter("tenant_name");
        }
        if(tenant !=null){
            //通过tenant查找schema
            LocalContext.getContext().removeAttachment(ContextAttributes.SCHEMA);
            TenantModel tenantModel = tenantService.findTenantByName(tenant.toString());
            if(tenantModel!=null){
                LocalContext.getContext().setAttachment(ContextAttributes.TENANT_NAME, tenantModel.getLoginName());
                LocalContext.getContext().setAttachment(ContextAttributes.SCHEMA, tenantModel.getSchema());
            }
        }
        return true;
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex)
            throws Exception {
    }

}
