package com.yihu.hos.jwt.auth;


import com.yihu.hos.jwt.secruity.JwtAuthenticationResponse;
import com.yihu.hos.system.model.SystemUser;
import com.yihu.hos.web.framework.model.ActionResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.AuthenticationException;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import javax.servlet.http.HttpServletRequest;

/**
 * 登录认证-jwt
 */
//@RestController
@RequestMapping("/auth")
public class AuthController {
    @Value("${jwt.header}")
    private String tokenHeader;

    @Autowired
    private AuthService authService;


    /**
     *  登录认证
     * @param authenticationRequest
     * @return
     * @throws AuthenticationException
     */
    @RequestMapping(value = "login", produces = "application/json", method = RequestMethod.POST)
    public ResponseEntity<?> createAuthenticationToken(
            @RequestBody SystemUser authenticationRequest) throws AuthenticationException{
        final String token = authService.login(authenticationRequest.getLoginCode(), authenticationRequest.getPassword());

        // Return the token
        return ResponseEntity.ok(new JwtAuthenticationResponse(token));
    }

    /**
     * token 刷新
     * @param request
     * @return
     * @throws AuthenticationException
     */
    @RequestMapping(value = "/token/refresh", produces = "application/json", method = RequestMethod.GET)
    public ResponseEntity<?> refreshAndGetAuthenticationToken(
            HttpServletRequest request) throws AuthenticationException{
        String token = request.getHeader(tokenHeader);
        String refreshedToken = authService.refresh(token);
        if(refreshedToken == null) {
            return ResponseEntity.badRequest().body(null);
        } else {
            return ResponseEntity.ok(new JwtAuthenticationResponse(refreshedToken));
        }
    }

    /**
     * 注册用户
     * @param addedUser
     * @return
     * @throws AuthenticationException
     */
    @RequestMapping(value = "register", produces = "application/json",  method = RequestMethod.POST)
    public ActionResult register(@RequestBody SystemUser addedUser) throws AuthenticationException{
        SystemUser systemUser = authService.register(addedUser);
        ActionResult result = new ActionResult();
        if (systemUser==null){
            result.setSuccessFlg(false);
            result.setErrorCode(403);
            result.setMessage("用户已经被注册！");
        }else {
            result.setData(systemUser);
            result.setSuccessFlg(true);
            result.setMessage("用户注册成功！");
        }
        return result;
    }

}
