package com.yihu.hos.jwt.auth;


import com.yihu.hos.jwt.secruity.JwtTokenUtil;
import com.yihu.hos.jwt.secruity.JwtUser;
import com.yihu.hos.system.entity.SystemRole;
import com.yihu.hos.system.model.SystemUser;
import com.yihu.hos.system.service.SystemRoleService;
import com.yihu.hos.system.service.UserManager;
import com.yihu.hos.web.framework.constant.ContextAttributes;
import com.yihu.hos.web.framework.util.PKUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.authentication.encoding.Md5PasswordEncoder;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;

import javax.annotation.Resource;
import java.util.Date;

//@Service
public class AuthServiceImpl implements AuthService {

    @Autowired
    private AuthenticationManager authenticationManager;
    private UserDetailsService userDetailsService;
    private JwtTokenUtil jwtTokenUtil;

    @Value("${jwt.tokenHead}")
    private String tokenHead;

    @Autowired
    private SystemRoleService systemRoleService;

    @Resource(name = UserManager.BEAN_ID)
    private UserManager userManager;


    @Autowired
    private Md5PasswordEncoder md5PasswordEncoder;


    @Autowired
    public AuthServiceImpl(
            UserDetailsService userDetailsService,
            JwtTokenUtil jwtTokenUtil) {
        this.userDetailsService = userDetailsService;
        this.jwtTokenUtil = jwtTokenUtil;
    }

    @Override
    public SystemUser register(SystemUser userToAdd) {
        final String username = userToAdd.getLoginCode();
        if(userManager.getUserByLoginCode(username)!=null) {
            return null;
        }

        try {
            String saltValu = PKUtil.getUuid();
            userToAdd.setSaltValue(saltValu);
            userToAdd.setActivated(SystemUser.ACTIVATED_FORBIDDEN);//未激活
            final String encodePassword = md5PasswordEncoder.encodePassword(userToAdd.getPassword(), saltValu);
            userToAdd.setPassword(encodePassword);
            userToAdd.setLastLoginTime(new Date());
            userManager.addUser(userToAdd);

            SystemRole role = systemRoleService.getRoleByCode(ContextAttributes.ROLE_USER);
            if (role == null){
                role =  new SystemRole();
                //不存在普通用户角色时，添加用户角色
                role.setCode(ContextAttributes.ROLE_USER);
                role.setName("普通用户");
                role.setValid("true");
                systemRoleService.addRole(role);
            }

        } catch (Exception e) {
            e.printStackTrace();
        }



        return userToAdd;
    }

    @Override
    public String login(String loginCode, String password) {
        SystemUser user = userManager.getUserByLoginCode(loginCode);
        if (user == null){
            System.out.println(String.format("No user found with username '%s'.", loginCode));
            return null;

        }

        UsernamePasswordAuthenticationToken upToken = new UsernamePasswordAuthenticationToken(loginCode, password);
        // Perform the security
        final Authentication authentication = authenticationManager.authenticate(upToken);
        SecurityContextHolder.getContext().setAuthentication(authentication);

        // Reload password post-security so we can generate token
        final UserDetails userDetails = userDetailsService.loadUserByUsername(loginCode);
        final String token = jwtTokenUtil.generateToken(userDetails);
        return token;
    }

    @Override
    public String refresh(String oldToken) {
        final String token = oldToken.substring(tokenHead.length());
        String username = jwtTokenUtil.getUsernameFromToken(token);
        JwtUser user = (JwtUser) userDetailsService.loadUserByUsername(username);
        if (jwtTokenUtil.canTokenBeRefreshed(token, user.getLastPasswordResetDate())){
            return jwtTokenUtil.refreshToken(token);
        }
        return null;
    }
}
