package com.yihu.hos.jwt.secruity;

import com.yihu.hos.system.model.SystemUser;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;

import java.util.List;
import java.util.stream.Collectors;

public final class JwtUserFactory {

    private JwtUserFactory() {
    }

    /**
     * 创建jwt验证用户信息
     *
     * @param user
     * @return
     */
    public static JwtUser create(SystemUser user) {

        return new JwtUser(
                user.getId(),
                user.getLoginCode(),
                user.getPassword(),
                user.getUserEmail(),
                user.getSaltValue(),
                mapToGrantedAuthorities(user.getRoles()),
                user.getLastLoginTime()
        );
    }


    private static List<GrantedAuthority> mapToGrantedAuthorities(List<String> authorities) {
        if (authorities != null) {
            return authorities.stream()
                    .map(SimpleGrantedAuthority::new)
                    .collect(Collectors.toList());
        } else {
            return null;
        }

    }
}

