package com.yihu.hos.jwt.user;

import com.yihu.hos.system.model.SystemUser;
import com.yihu.hos.system.service.UserManager;
import com.yihu.hos.web.framework.model.Result;
import org.springframework.security.access.prepost.PostAuthorize;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

/**
 * TODO 测试接口，后面可删除
 * 在 @PreAuthorize 中我们可以利用内建的 SPEL 表达式：比如 'hasRole()' 来决定哪些用户有权访问。
 * 需注意的一点是 hasRole 表达式认为每个角色名字前都有一个前缀 'ROLE_'。所以这里的 'ADMIN' 其实在
 * 数据库中存储的是 'ROLE_ADMIN' 。这个 @PreAuthorize 可以修饰Controller也可修饰Controller中的方法。
 **/
@RestController("jwtUserController")
@RequestMapping("/users")
public class UserController {

    @Resource(name = UserManager.BEAN_ID)
    private UserManager userManager;


    @PostAuthorize("returnObject.loginCode == principal.loginCode or hasRole('ROLE_ADMIN')")
    @RequestMapping(value = "/{id}", produces = "application/json", method = RequestMethod.GET)
    public SystemUser getUser(@PathVariable String id) throws Exception {
        return userManager.getUserById(id);
    }

    @PreAuthorize("hasRole('ADMIN')")
    @RequestMapping(value = "/{id}", produces = "application/json", method = RequestMethod.PUT)
    Result updateUser(@PathVariable String id, @RequestBody SystemUser updatedUser) throws Exception {
        updatedUser.setId(id);
        return userManager.updateUser(updatedUser);
    }

    @PreAuthorize("hasRole('ADMIN')")
    @RequestMapping(value = "/{id}", produces = "application/json", method = RequestMethod.DELETE)
    Result removeUser(@PathVariable String id) throws Exception {
        Result deletedUser = userManager.deleteUser(id);
        return deletedUser;
    }

    @PostAuthorize("returnObject.loginCode == principal.loginCode or hasRole('ROLE_ADMIN')")
    @RequestMapping(value = "/", produces = "application/json", method = RequestMethod.GET)
    public SystemUser getUserByUsername(@RequestParam(value = "username") String username) {
        SystemUser user = userManager.getUserByLoginCode(username);
        return user;
    }
}
