package com.yihu.hos.qc.controller;

import com.google.common.base.Splitter;
import com.yihu.ehr.util.rest.Envelop;
import com.yihu.hos.qc.model.RuleModel;
import com.yihu.hos.qc.service.RuleService;
import com.yihu.hos.web.framework.model.Result;
import com.yihu.hos.web.framework.util.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.beanutils.BeanUtils;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 质控规则管理
 *
 * @author CD
 * @vsrsion 1.0
 * Created at 2017/05/05.
 */
@RequestMapping("/rule")
@Controller
@Api(description = "质控规则接口", tags = {"质控规则--质控规则接口"})
public class RuleController extends BaseController {

    @Resource(name = RuleService.BEAN_ID)
    private RuleService ruleService;


    /**
     * 质控规则管理界面
     *
     * @param model
     * @return
     */
    @RequestMapping("/initial")
    public String ruleInitial(Model model) {
        model.addAttribute("contentPage", "qc/rule");
        return "partView";
    }


    /**
     * 质控规则列表
     *
     * @param request
     * @return
     */
    @RequestMapping("/getRuleList")
    @ResponseBody
    public Result getAppList(HttpServletRequest request, String name, String valid) {
        try {
            Map<String, Object> params = new HashMap<>();
            params.put("name", name);
            String page = request.getParameter("page");
            String rows = request.getParameter("rows");

            params.put("page", page);
            params.put("rows", rows);
            Result result = ruleService.getRuleList(params);
            return result;
        } catch (Exception ex) {
            ex.printStackTrace();
            return Result.error(ex.getMessage());
        }
    }

    /**
     * 质控规则修改页面
     *
     * @param model
     * @param id
     * @return
     */
    @RequestMapping("/editorRule")
    public String editoruleRule(Model model, String id) {
        try {
            RuleModel ruleModel = null;
            if (id != null) {
                ruleModel = ruleService.getRuleById(id);
            } else {
                ruleModel = new RuleModel();
            }
            model.addAttribute("model", ruleModel);
            model.addAttribute("contentPage", "/qc/editorRule");
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "pageView";
    }

    /**
     * 质控规则详情页
     *
     * @param model
     * @param id
     * @return
     */
    @RequestMapping("/ruleDetail")
    public String ruleDetail(Model model, String id) {
        try {
            RuleModel ruleModel = null;
            if (id != null) {
                ruleModel = ruleService.getRuleById(id);
            } else {
                ruleModel = new RuleModel();
            }
            model.addAttribute("model", ruleModel);
            model.addAttribute("contentPage", "/rule/ruleDetail");
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "pageView";
    }


    /**
     * 新增质控规则s信息
     *
     * @param request
     * @return
     */
    @RequestMapping("addRule")
    @ResponseBody
    public Result addRule(HttpServletRequest request) {
        try {
            RuleModel obj = new RuleModel();
            BeanUtils.populate(obj, request.getParameterMap());
            return ruleService.addRule(obj);
        } catch (Exception ex) {
            ex.printStackTrace();
            return Result.error(ex.getMessage());
        }
    }

    /**
     * 删除质控规则信息
     *
     * @param request
     * @return
     */
    @RequestMapping("/deleteRule")
    @ResponseBody
    public Result deleteRule(HttpServletRequest request) {

        try {
            String id = request.getParameter("id");

            ruleService.deleteRule(id);
            return Result.success("删除成功！");
        } catch (Exception e) {
            e.printStackTrace();
            return Result.error("删除失败！");
        }
    }

    /**
     * 批量删除质控规则信息
     *
     * @param request
     * @return
     */
    @RequestMapping("/deleteRuleBatch")
    @ResponseBody
    public Result deleteRuleBatch(HttpServletRequest request) {
        try {
            String idList = request.getParameter("idList");
            List<String> ids = Splitter.on(",").trimResults().splitToList(idList);
            if (!CollectionUtils.isEmpty(ids)) {
                for (String id :
                        ids) {
                    ruleService.deleteRule(id);
                }
            }
            return Result.success("删除成功！");
        } catch (Exception e) {
            e.printStackTrace();
            return Result.error("删除失败！");
        }
    }

    /**
     * 修改质控规则信息
     */
    @RequestMapping("updateRule")
    @ResponseBody
    public Result updateRule(HttpServletRequest request) {
        try {
            RuleModel obj = new RuleModel();
            BeanUtils.populate(obj, request.getParameterMap());

            return ruleService.updateRule(obj);
        } catch (Exception ex) {
            ex.printStackTrace();
            return Result.error(ex.getMessage());
        }
    }

    /**
     * 绑定质控规则
     */
    @ApiOperation(" 根据数集ID&数据元编码绑定质控规则")
    @RequestMapping(value = "bindQcRule", method = RequestMethod.POST)
    @ResponseBody
    public Envelop bindQcRule(
            @RequestParam(name = "stdVersion") String stdVersion,
            @RequestParam(name = "stdDataSetId") Long stdDataSetId,
            @RequestParam(name = "dataElement") String dataElement,
            @RequestParam(name = "ruleList") String ruleList) {
        Envelop envelop = new Envelop();
        try {
            ruleService.bindQcRule(stdVersion,stdDataSetId,dataElement, ruleList);//绑定质控规则反射方法
            envelop.setSuccessFlg(true);
        } catch (Exception ex) {
            ex.printStackTrace();
            envelop.setSuccessFlg(false);
            envelop.setErrorMsg(ex.getMessage());
        }

        return envelop;
    }

    /**
     * 绑定质控规则
     */
    @ApiOperation("根据数据元编码批量绑定质控规则")
    @RequestMapping(value = "bindQcRuleBatch", method = RequestMethod.POST)
    @ResponseBody
    public Envelop bindQcRuleBatch(
            @RequestParam(name = "stdVersion") String stdVersion,
            @RequestParam(name = "dataElement") String dataElement,
            @RequestParam(name = "ruleList") String ruleList) {
        Envelop envelop = new Envelop();
        try {
            ruleService.bindQcRuleBatch(stdVersion,dataElement, ruleList);//绑定质控规则反射方法
            envelop.setSuccessFlg(true);
        } catch (Exception ex) {
            ex.printStackTrace();
            envelop.setSuccessFlg(false);
            envelop.setErrorMsg(ex.getMessage());
        }

        return envelop;
    }


    /**
     * 绑定质控规则
     */
    @ApiOperation(" 根据数集编码&数据元编码绑定质控规则")
    @RequestMapping(value = "bindQcRuleByCode", method = RequestMethod.POST)
    @ResponseBody
    public Envelop bindQcRule(
            @RequestParam(name = "stdVersion") String stdVersion,
            @RequestParam(name = "stdDataSetCode") String stdDataSetCode,
            @RequestParam(name = "dataElement") String dataElement,
            @RequestParam(name = "ruleList") String ruleList) {
        Envelop envelop = new Envelop();
        try {

            ruleService.bindQcRule(stdVersion,stdDataSetCode,dataElement, ruleList);
            envelop.setSuccessFlg(true);
        } catch (Exception ex) {
            ex.printStackTrace();
            envelop.setSuccessFlg(false);
            envelop.setErrorMsg(ex.getMessage());
        }

        return envelop;
    }


    /**
     * 获取绑定质控规则
     */
    @RequestMapping(value = "bindQcRule", method = RequestMethod.GET)
    @ResponseBody
    public Envelop getBindQcRule(
            @RequestParam(name = "stdVersion") String stdVersion,
            @RequestParam(name = "stdDataSetId") Long stdDataSetId,
            @RequestParam(name = "dataElement") String dataElement) {
        Envelop envelop = new Envelop();
        try {
            List<RuleModel> bindQcRule = ruleService.getBindQcRule(stdVersion,stdDataSetId,dataElement);
            envelop.setDetailModelList(bindQcRule);
            envelop.setSuccessFlg(true);
        } catch (Exception ex) {
            ex.printStackTrace();
            envelop.setSuccessFlg(false);
            envelop.setErrorMsg(ex.getMessage());
        }

        return envelop;
    }


    /**
     * 获取绑定质控规则
     */
    @RequestMapping(value = "bindQcRuleByCode", method = RequestMethod.GET)
    @ResponseBody
    public Envelop bindQcRuleByCode(
            @RequestParam(name = "stdVersion") String stdVersion,
            @RequestParam(name = "stdDataSetCode") String stdDataSetCode,
            @RequestParam(name = "dataElement") String dataElement) {
        Envelop envelop = new Envelop();
        try {
            List<RuleModel> bindQcRule = ruleService.getBindQcRule(stdVersion,stdDataSetCode,dataElement);
            envelop.setDetailModelList(bindQcRule);
            envelop.setSuccessFlg(true);
        } catch (Exception ex) {
            ex.printStackTrace();
            envelop.setErrorMsg("绑定制控规则");
        }

        return envelop;
    }


    @ApiOperation("根据数据集，绑定其所有数据元的质控规则")
    @RequestMapping(value = "bindQcRuleByDataSet", method = RequestMethod.POST)
    @ResponseBody
    public Envelop bindQcRuleByDataSet(
            @ApiParam(name = "stdVersion", value = "标准版本号", required = true)
            @RequestParam(name = "stdVersion") String stdVersion,
            @ApiParam(name = "dataSetId", value = "数据集ID", required = true)
            @RequestParam(name = "dataSetId") Long dataSetId) {
        Envelop envelop = new Envelop();
        try {
            ruleService.bindQcRuleByDataSet(stdVersion, dataSetId);
            envelop.setSuccessFlg(true);
        } catch (Exception e) {
            e.printStackTrace();
            envelop.setSuccessFlg(false);
            envelop.setErrorMsg(e.getMessage());
        }
        return envelop;
    }

    @ApiOperation("根据标准版本，绑定其所有数据元的质控规则")
    @RequestMapping(value = "bindQcRuleByStdVersion", method = RequestMethod.POST)
    @ResponseBody
    public Envelop bindQcRuleByStdVersion(
            @ApiParam(name = "stdVersion", value = "标准版本", required = true)
            @RequestParam(name = "stdVersion") String stdVersion) {
        Envelop envelop = new Envelop();
        try {
            ruleService.batchBindRuleByStdVersion(stdVersion);
            envelop.setSuccessFlg(true);
            envelop.setErrorMsg("标准版本："+stdVersion +"绑定规则成功");
        } catch (Exception e) {
            e.printStackTrace();
            envelop.setSuccessFlg(false);
            envelop.setErrorMsg(e.getMessage());
        }
        return envelop;
    }

    @ApiOperation("根据标准版本，删除其所有数据元的质控规则")
    @RequestMapping(value = "delQcRuleByStdVersion", method = RequestMethod.DELETE)
    @ResponseBody
    public Envelop delQcRuleByStdVersion(
            @ApiParam(name = "stdVersion", value = "标准版本", required = true)
            @RequestParam(name = "stdVersion") String stdVersion) {
        Envelop envelop = new Envelop();
        try {
            ruleService.deleteQcMetaData(stdVersion);
            envelop.setSuccessFlg(true);
        } catch (Exception e) {
            e.printStackTrace();
            envelop.setSuccessFlg(false);
            envelop.setErrorMsg(e.getMessage());
        }
        return envelop;
    }

    /**
     * 获取绑定质控规则
     */
    @ApiOperation("获取数据元绑定质控规则(返回单个数据元）")
    @RequestMapping(value = "metaDataRule", method = RequestMethod.GET)
    @ResponseBody
    public Envelop metaDataRule(
            @RequestParam(name = "stdVersion") String stdVersion,
            @RequestParam(name = "stdDataSetCode") String stdDataSetCode,
            @RequestParam(name = "metaDataCode") String metaDataCode) {
        Envelop envelop = new Envelop();
        try {
            String bindQcRule = ruleService.getQcRule(stdVersion,stdDataSetCode,metaDataCode);
            envelop.setObj(bindQcRule);
            envelop.setSuccessFlg(true);
        } catch (Exception ex) {
            ex.printStackTrace();
            envelop.setErrorMsg("获取数据元质控绑定规则异常");
        }

        return envelop;
    }

    /**
     * 获取数据集下有绑定质控规则的数据元
     */
    @ApiOperation("获取数据集下有绑定质控规则的数据元")
    @RequestMapping(value = "metaDatas", method = RequestMethod.GET)
    @ResponseBody
    public Envelop getMetaDatasByDataSet(
            @RequestParam(name = "stdVersion") String stdVersion,
            @RequestParam(name = "stdDataSetCode") String stdDataSetCode) {
        Envelop envelop = new Envelop();
        try {
            List<String> metaDatas = ruleService.getMetaDataByDataSet(stdVersion,stdDataSetCode);
            envelop.setDetailModelList(metaDatas);
            envelop.setSuccessFlg(true);
        } catch (Exception ex) {
            ex.printStackTrace();
            envelop.setErrorMsg("获取数据元质控绑定规则异常");
        }

        return envelop;
    }
}
