package com.yihu.hos.qc.feign;

import com.yihu.ehr.constants.ApiVersion;
import com.yihu.ehr.constants.MicroServices;
import com.yihu.ehr.constants.ServiceApi;
import com.yihu.ehr.model.redis.MRedisMqSubscriber;
import com.yihu.ehr.util.rest.Envelop;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.netflix.feign.FeignClient;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import java.util.Collection;

/**
 * Redis消息订阅者 Client
 *
 * @author 张进军
 * @date 2017/11/13 15:14
 */
@FeignClient(name = MicroServices.Redis)
@RequestMapping(value = ApiVersion.Version1_0)
@ApiIgnore
public interface RedisMqSubscriberClient {

    @RequestMapping(value = ServiceApi.Redis.MqSubscriber.GetById, method = RequestMethod.GET)
    Envelop getById(
            @PathVariable(value = "id") Integer id);

    @RequestMapping(value = ServiceApi.Redis.MqSubscriber.Search, method = RequestMethod.GET)
    Envelop search(
            @RequestParam(value = "fields", required = false) String fields,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "sorts", required = false) String sorts,
            @RequestParam(value = "page", required = false) int page,
            @RequestParam(value = "size", required = false) int size);

    @RequestMapping(value = ServiceApi.Redis.MqSubscriber.Prefix, method = RequestMethod.GET)
    Collection<MRedisMqSubscriber> getSubscriberList(
            @RequestParam(value = "filters", required = false) String filters);

    @ApiOperation("新增消息订阅者")
    @RequestMapping(value = ServiceApi.Redis.MqSubscriber.Save, method = RequestMethod.POST)
    Envelop add(
            @RequestBody String entityJson);

    @ApiOperation("更新消息订阅者")
    @RequestMapping(value = ServiceApi.Redis.MqSubscriber.Save, method = RequestMethod.PUT)
    Envelop update(
            @RequestBody String entityJson);

    @RequestMapping(value = ServiceApi.Redis.MqSubscriber.Delete, method = RequestMethod.DELETE)
    Envelop delete(
            @RequestParam(value = "id") Integer id);

    @RequestMapping(value = ServiceApi.Redis.MqSubscriber.IsUniqueAppId, method = RequestMethod.GET)
    Envelop isUniqueAppId(
            @RequestParam(value = "id") Integer id,
            @RequestParam(value = "channel") String channel,
            @RequestParam(value = "appId") String appId);

    @RequestMapping(value = ServiceApi.Redis.MqSubscriber.IsUniqueSubscribedUrl, method = RequestMethod.GET)
    Envelop isUniqueSubscribedUrl(
            @RequestParam(value = "id") Integer id,
            @RequestParam(value = "channel") String channel,
            @RequestParam(value = "subscriberUrl") String subscriberUrl);

    @RequestMapping(value = ServiceApi.Redis.MqSubscriber.IsExist, method = RequestMethod.GET)
    Boolean isExist(
            @RequestParam(value = "channel") String channel,
            @RequestParam(value = "subscriber") String subscriber);

    @RequestMapping(value = ServiceApi.Redis.MqSubscriber.Unsubscribe, method = RequestMethod.POST)
    void unsubscribe(
            @RequestParam(value = "channel") String channel,
            @RequestParam(value = "subscriber", required = false) String subscriber);

}
