package com.yihu.hos.qc.service;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.hos.core.log.Logger;
import com.yihu.hos.core.log.LoggerFactory;
import com.yihu.hos.qc.dao.QcRuleMetadataDao;
import com.yihu.hos.qc.dao.RuleDao;
import com.yihu.hos.qc.feign.RedisMqChannelClient;
import com.yihu.hos.qc.feign.RedisMqPublisherClient;
import com.yihu.hos.qc.feign.RedisMqSubscriberClient;
import com.yihu.hos.qc.model.RuleModel;
import com.yihu.hos.redis.schema.std.StdQcMetaDataKeySchema;
import com.yihu.hos.standard.model.standard.STDVersion;
import com.yihu.hos.standard.model.standard.StdDataSetModel;
import com.yihu.hos.standard.model.standard.StdMetaDataModel;
import com.yihu.hos.standard.service.standard.STDVersionService;
import com.yihu.hos.standard.service.standard.StandardService;
import com.yihu.hos.standard.service.standard.StdDatasetService;
import com.yihu.hos.standard.service.standard.StdMetadataService;
import com.yihu.hos.web.framework.model.Result;
import com.yihu.hos.web.framework.model.standard.StandardVersion;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.IOException;
import java.io.Serializable;
import java.util.*;

/**
 * @author CD
 * @vsrsion 1.0
 * Created at 2017/05/05.
 */
@Service("ruleService")
public class RuleService {
    public static final String BEAN_ID = "ruleService";
    static final Logger logger = LoggerFactory.getLogger(RuleService.class);
    @Resource(name = RuleDao.BEAN_ID)
    private RuleDao ruleDao;
    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private StdMetadataService stdMetadataService;
    @Autowired
    private QcRuleMetadataDao qcRuleMetadataDao;
    @Autowired
    private STDVersionService stdVersionService;
    @Autowired
    private StdQcMetaDataKeySchema qcMetaDataKeySchema;
    @Autowired
    private StdDatasetService stdDatasetService;

    public Result getRuleList(Map<String, Object> params) throws Exception {
        return ruleDao.getRuleList(params);
    }


    public RuleModel getRuleById(String id) throws Exception {
        return ruleDao.getEntity(RuleModel.class, id);
    }

    @Transactional
    public Result addRule(RuleModel obj) throws Exception {
        ruleDao.saveEntity(obj);
        return Result.success("保存成功");
    }

    @Transactional
    public Result updateRule(RuleModel obj) throws Exception {
        ruleDao.updateEntity(obj);
        return Result.success("更新成功");
    }

    @Transactional
    public Result deleteRule(String id) throws Exception {
        RuleModel ruleModel = ruleDao.getEntity(RuleModel.class, id);
        ruleDao.deleteEntity(ruleModel);
        return Result.success("删除成功");
    }


    /**
     * 收集数据元需要绑定的质控规则
     *
     * @param metadata   数据元
     */
    private RuleModel collectQcRule(StdMetaDataModel metadata) {
        RuleModel ruleModel = null;
        String ruleCode = "";
        try {

            // 时间格式检验
            if (metadata.getFormat().contains("DT") || metadata.getColumnType().contains("DATE")) {
                ruleCode = qcRuleMetadataDao.findRuleCodeByType("type_rule");
            } else if (!metadata.getNullable()) {
                // 非空校验规则
                ruleCode = qcRuleMetadataDao.findRuleCodeByType("nullable");
            } else if (metadata.getDictId() != null && metadata.getDictId() != 0L) {
                // 值域校验规则
                ruleCode = qcRuleMetadataDao.findRuleCodeByType("dict_id");
            }

            if (!ruleCode.isEmpty()) {
                ruleModel = ruleDao.getByCode(ruleCode);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ruleModel;
    }

    /**
     * 收集数据元需要绑定的质控规则
     *
     * @param qcRuleList 质控规则容器
     * @param metadata   数据元
     */
    private void collectQcRule(List<RuleModel> qcRuleList, StdMetaDataModel metadata) {
        String ruleCode = "";
        try {

            // 时间格式检验
            if (metadata.getFormat().contains("DT") || metadata.getColumnType().contains("DATE")) {
                ruleCode = qcRuleMetadataDao.findRuleCodeByType("type_rule");
                qcRuleList.add(ruleDao.getByCode(ruleCode));
            }
            // 非空校验规则
            if (!metadata.getNullable()) {
                ruleCode = qcRuleMetadataDao.findRuleCodeByType("nullable");
                qcRuleList.add(ruleDao.getByCode(ruleCode));
            }
            // 值域校验规则
            if (metadata.getDictId() != null && metadata.getDictId() != 0L) {
                ruleCode = qcRuleMetadataDao.findRuleCodeByType("dict_id");
                qcRuleList.add(ruleDao.getByCode(ruleCode));
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    public List<RuleModel> getBindQcRule(String stdVersion,String stdDataSetCode,String dataElement) throws IOException {

        String qcRuleMethod = getQcRule(stdVersion, stdDataSetCode, dataElement);
        return ruleDao.getByRuleMethod(qcRuleMethod);
    }



    /* ******************************************   反射方式绑定质控规则 *************************************** */

    public List<RuleModel> getBindQcRule(String stdVersion,Long stdDataSetId,String dataElement) throws IOException {
        StdDataSetModel stdDataSetModel = stdDatasetService.get(new StandardVersion(stdVersion), stdDataSetId);
        if (stdDataSetModel != null) {
            String qcRuleMethod = getQcRule(stdVersion, stdDataSetModel.getCode(), dataElement);
            return ruleDao.getByRuleMethod(qcRuleMethod);
        } else {
            throw new RuntimeException("绑定规则的数据集不存在，dataSetId：" + stdDataSetId);
        }

    }

    public void bindQcRule(String version, String dataSetCode, String metadataInnerCode, String ruleList) throws IOException {
        List<RuleModel> rules = objectMapper.readValue(ruleList,
                new TypeReference<List<RuleModel>>() {
                });
        bindQcRule(version, dataSetCode, metadataInnerCode, rules);
    }

    //绑定数据元质控规则方法
    public void bindQcRule(String version, String dataSetCode, String metadataInnerCode, List<RuleModel> ruleList) {
        String ruleMethod = null;
        if (ruleList !=null && ruleList.size()>0) {
            ruleMethod = ruleList.get(0).getRuleMethod();
        }

        if (ruleMethod == null) {
            return;
        }
        qcMetaDataKeySchema.setQcMetaDataList(version, dataSetCode, metadataInnerCode, ruleMethod);
    }

    /**
     * 绑定数据元质控规则方法  (反射）
     *
     * @param version           标准版本号
     * @param dataSetId         标准数据集ID
     * @param metadataInnerCode 标准数据元编码
     * @param ruleList          质控规则列表
     */
    public void bindQcRule(String version, Long dataSetId, String metadataInnerCode, String ruleList) throws IOException {
        List<RuleModel> rules = objectMapper.readValue(ruleList,
                new TypeReference<List<RuleModel>>() {
                });
        String ruleMethod = null;
        if (rules !=null && rules.size()>0) {
            ruleMethod = rules.get(0).getRuleMethod();
        }

        if (ruleMethod == null) {
            return;
        }

        StdDataSetModel stdDataSetModel = stdDatasetService.get(new StandardVersion(version), dataSetId);
        if (stdDataSetModel != null) {
            qcMetaDataKeySchema.setQcMetaDataList(version, stdDataSetModel.getCode(), metadataInnerCode, ruleMethod);
        } else {
            throw new RuntimeException("绑定规则的数据集不存在，dataSetId：" + dataSetId);
        }
    }

    /**
     * 绑定数据元质控规则方法  (反射）
     *
     * @param version           标准版本号
     * @param metadataInnerCode 标准数据元编码
     * @param ruleList          质控规则列表
     */
    public void bindQcRuleBatch(String version, String metadataInnerCode, String ruleList) throws IOException {
        List<RuleModel> rules = objectMapper.readValue(ruleList,
                new TypeReference<List<RuleModel>>() {
                });
        String ruleMethod = null;
        if (rules !=null && rules.size()>0) {
            ruleMethod = rules.get(0).getRuleMethod();
        }

        if (ruleMethod == null) {
            return;
        }
        //通过数据元内部编码获取所在的数据集,然后循环绑定值
        List<StdMetaDataModel> list = stdMetadataService.searchPage(StdMetaDataModel.class, StandardVersion.MetaDataTable,version, "innerCode="+metadataInnerCode, null, null, null,true);

        if(CollectionUtils.isNotEmpty(list)){
            for(StdMetaDataModel model:list){
                Long dataSetId = model.getDataSetId();
                StdDataSetModel stdDataSetModel = stdDatasetService.get(new StandardVersion(version), dataSetId);
                if (stdDataSetModel != null) {
                    qcMetaDataKeySchema.setQcMetaDataList(version, stdDataSetModel.getCode(), metadataInnerCode, ruleMethod);
                } else {
                    throw new RuntimeException("绑定规则的数据集不存在，dataSetId：" + dataSetId);
                }
            }
        }
    }

    /**
     * 根据数据集，绑定其所有数据元的质控规则
     *
     * @param stdVersion 标准版本号
     * @param dataSetId  数据集ID
     */
    public void bindQcRuleByDataSet(String stdVersion, Long dataSetId) {
        try {
            Map<String, Serializable> map = null;
            StdDataSetModel stdDataSetModel = stdDatasetService.get(new StandardVersion(stdVersion), dataSetId);
            List<StdMetaDataModel> metadataList = stdMetadataService.searchPageData(StdMetaDataModel.class, StandardVersion.MetaDataTable, stdVersion, "dataSetId=" + dataSetId, null, null, null);
            RuleModel ruleModel = null;

            if (stdDataSetModel == null) {
                throw new RuntimeException("获取数据集失败version: " + stdVersion + " ,datasetId : " + dataSetId);
            }
            map = new HashMap<>();
            for (StdMetaDataModel metadata : metadataList) {
                // 遍历标准版本的数据元集合
                if (metadata != null) {
                    ruleModel = this.collectQcRule(metadata);
                    if (ruleModel !=null) {
                        String key = qcMetaDataKeySchema.getKey(stdVersion, stdDataSetModel.getCode(), metadata.getInnerCode());
                        map.put(key, ruleModel.getRuleMethod());
                    }
                } else {
                    continue;
                }
            }
            qcMetaDataKeySchema.multiSetQcMetaData(map);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 根据标准版本，绑定其所有数据元的质控规则存储到redis
     *
     * @param stdVersion 标准版本
     */
    public void bindQcRuleByVersion(String stdVersion) {
        try {
            // 标准的版本信息
            STDVersion stdVersionInfo = stdVersionService.getVersion(stdVersion);
            List<StdDataSetModel> dataSetModelList = stdDatasetService.searchPageData(StdDataSetModel.class, StandardVersion.DataSetTable, stdVersion, null, null, null, null);
            for (StdDataSetModel dataSetModel : dataSetModelList) {
                List<StdMetaDataModel> metadataList = stdMetadataService.getMetadataList(StdMetaDataModel.class, stdVersion, dataSetModel.getId(), null, null, null, null);
                // 遍历标准版本的数据元集合
                List<RuleModel> qcRuleList = new ArrayList<>();
                for (StdMetaDataModel metadata : metadataList) {
                    qcRuleList.clear();
                    this.collectQcRule(qcRuleList, metadata);
                    this.bindQcRule(stdVersion, dataSetModel.getCode(), metadata.getInnerCode(), qcRuleList);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 根据标准版本，绑定其所有数据元的质控规则
     *
     * @param stdVersion 标准版本
     */
    public void batchBindRuleByStdVersion(String stdVersion) {
        try {
            Map<String, Serializable> map = null;
            // 标准的版本信息
//            STDVersion stdVersionInfo = stdVersionService.getVersion(stdVersion);
            List<StdDataSetModel> dataSetModelList = stdDatasetService.searchPageData(StdDataSetModel.class, StandardVersion.DataSetTable, stdVersion, null, null, null, null);
            for (StdDataSetModel dataSetModel : dataSetModelList) {
                List<StdMetaDataModel> metadataList = stdMetadataService.getMetadataList(StdMetaDataModel.class, stdVersion, dataSetModel.getId(), null, null, null, null);
                // 遍历标准版本的数据元集合
                RuleModel ruleModel;
                map = new HashMap<>();
                for (StdMetaDataModel metadata : metadataList) {
                    if (metadata != null) {
                        ruleModel = this.collectQcRule(metadata);
                        //有规则，添加到redis缓存
                        if (ruleModel!=null) {
                            String key = qcMetaDataKeySchema.getKey(stdVersion, dataSetModel.getCode(), metadata.getInnerCode());
                            map.put(key, ruleModel.getRuleMethod());
                        }
                    } else {
                        continue;
                    }

                }
                //保存到redis
                if (map !=null && !map.isEmpty()) {
                    qcMetaDataKeySchema.multiSetQcMetaData(map);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 根据标准版本，删除其所有数据元的质控规则
     *
     * @param stdVersion 标准版本
     */
    public void delQcRuleByStdVersion(String stdVersion) {
        try {
            // 标准的版本信息
            List<StdDataSetModel> dataSetModelList = stdDatasetService.searchPageData(StdDataSetModel.class, StandardVersion.DataSetTable, stdVersion, null, null, null, null);
            for (StdDataSetModel dataSetModel : dataSetModelList) {
                List<StdMetaDataModel> metadataList = stdMetadataService.getMetadataList(StdMetaDataModel.class, stdVersion, dataSetModel.getId(), null, null, null, null);
                // 遍历标准版本的数据元集合
                for (StdMetaDataModel metadata : metadataList) {
                    qcMetaDataKeySchema.deleteQcMetaData(stdVersion, dataSetModel.getCode(), metadata.getInnerCode());
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 根据标准版本，删除其所有数据元的质控规则
     *
     * @param stdVersion 标准版本
     */
    public void deleteQcMetaData(String stdVersion) {
        try {
            // 根据版本号模糊删除
            qcMetaDataKeySchema.deleteQcMetaData(stdVersion);

        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    /**
     * 获取 数据元绑定质控规则方法（反射）
     *
     * @param version
     * @param dataSetCode
     * @param metadataInnerCode
     * @return
     * @throws IOException
     */
    public String getQcRule(String version, String dataSetCode, String metadataInnerCode) throws IOException {
        return qcMetaDataKeySchema.getQcMetaData(version, dataSetCode, metadataInnerCode);
    }

    /**
     * 模糊获取数据集下的有绑定规则的数据元
     */
    public List<String> getMetaDataByDataSet(String stdVersion, String stdDataSet) {

        return qcMetaDataKeySchema.getMetaDataByDataSet(stdVersion, stdDataSet);
    }


}
