package com.yihu.hos.redis.controller;

import com.yihu.hos.common.controller.EnvelopRestEndPoint;
import com.yihu.hos.core.constants.MicroServices;
import com.yihu.hos.core.constants.ServiceApi;
import com.yihu.hos.redis.service.AdapterRedisService;
import com.yihu.hos.standard.model.adapter.AdapterDatasetModel;
import com.yihu.hos.web.framework.constant.ContextAttributes;
import com.yihu.hos.web.framework.thread.LocalContext;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;


@RestController
@RequestMapping(value = MicroServices.ApiVersion.Version1_0)
@Api(value = "Redis", description = "Redis数据缓存服务")
public class AdapterRedisEndPoint extends EnvelopRestEndPoint {

    @Autowired
    private AdapterRedisService adapterRedisService;

    @ApiOperation("Redis缓存适配方案")
    @RequestMapping(value = ServiceApi.Redis.InitAdapterVersions, method = RequestMethod.POST)
    public String cacheVersions(
            @ApiParam(name = "versions", value = "适配方案版本,多个用逗号分隔")
            @RequestParam("versions") String versions,
            @ApiParam(name = "force", value = "强制清除再缓存", defaultValue = "true")
            @RequestParam("force") boolean force) throws Exception {
        String schema = LocalContext.getContext().getAttachment(ContextAttributes.SCHEMA);
        adapterRedisService.cacheVersions(versions, force,schema);
        return "Redis适配方案缓存完成！";
    }

    @ApiOperation("根据适配版本号和目标数据集code,获取的源数据集code")
    @RequestMapping(value = ServiceApi.Redis.AdapterDataSetCode, method = RequestMethod.GET)
    public String getAdapterDatasetCode(
            @ApiParam(name = "version", value = "适配方案版本")
            @RequestParam("version") String version,
            @ApiParam(name = "stdDatasetCode", value = "标准数据集code")
            @RequestParam("stdDatasetCode") String stdDatasetCode){
        return adapterRedisService.getAdapterDatasetCode(version,stdDatasetCode);
    }

    @ApiOperation("根据适配版本号和目标数据集code,目标数据元code,获取的源数据元code")
    @RequestMapping(value = ServiceApi.Redis.AdapterMetadataCode, method = RequestMethod.GET)
    public String getAdapterMetadataCode(
            @ApiParam(name = "version", value = "适配方案版本")
            @RequestParam("version") String version,
            @ApiParam(name = "stdDatasetCode", value = "标准数据集code")
            @RequestParam("stdDatasetCode") String stdDatasetCode,
            @ApiParam(name = "stdMetadataCode", value = "标准数据元code")
            @RequestParam("stdMetadataCode") String stdMetadataCode){
        return adapterRedisService.getAdapterMetadataCode(version,stdDatasetCode,stdMetadataCode);
    }

    @ApiOperation("根据适配版本号和目标字典code,获取的源字典code")
    @RequestMapping(value = ServiceApi.Redis.AdapterDictCode, method = RequestMethod.GET)
    public String getAdapterDictCode(@ApiParam(name = "version", value = "适配方案版本")
                                     @RequestParam("version") String version,
                                     @ApiParam(name = "stdDictCode", value = "标准字典code")
                                     @RequestParam("stdDictCode") String stdDictCode){
        return adapterRedisService.getAdapterDictCode(version,stdDictCode);
    }

    @ApiOperation("根据适配版本号和目标字典code,目标字典项code,获取的源字典项code")
    @RequestMapping(value = ServiceApi.Redis.AdapterDictEntryCode, method = RequestMethod.GET)
    public String getAdapterDictEntryCode(@ApiParam(name = "version", value = "适配方案版本")
                                     @RequestParam("version") String version,
                                     @ApiParam(name = "stdDictCode", value = "标准字典code")
                                     @RequestParam("stdDictCode") String stdDictCode,
                                     @ApiParam(name = "stdDictEntryCode", value = "标准字典项code")
                                     @RequestParam("stdDictEntryCode") String stdDictEntryCode){
        return adapterRedisService.getAdapterDictEntryCode(version,stdDictCode,stdDictEntryCode);
    }


}
