package com.yihu.hos.redis.controller;

import com.yihu.hos.common.controller.EnvelopRestEndPoint;
import com.yihu.hos.core.constants.MicroServices;
import com.yihu.hos.core.constants.ServiceApi;
import com.yihu.hos.redis.service.StdRedisService;
import com.yihu.hos.web.framework.constant.ContextAttributes;
import com.yihu.hos.web.framework.thread.LocalContext;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;


/**
 * @author hzp add at 20170425
 */
@RestController
@RequestMapping(value = MicroServices.ApiVersion.Version1_0)
@Api(value = "Redis", description = "Redis数据缓存服务")
public class StdRedisEndPoint extends EnvelopRestEndPoint {

    @Autowired
    private StdRedisService stdRedisService;

    @ApiOperation("通过version值获取版本名")
    @RequestMapping(value = ServiceApi.Redis.StdVersion, method = RequestMethod.GET)
    @ResponseBody
    public String getStdVersion(@ApiParam(value = "key", defaultValue = "59083976eebd")
                                    @RequestParam("key") String key) {
        return stdRedisService.getStdVersion(key);
    }

    @ApiOperation("通过StdDataSet版本和id获取标准数据集编码")
    @RequestMapping(value = ServiceApi.Redis.StdDataSetCode, method = RequestMethod.GET)
    @ResponseBody
    public String getDataSetCode(@ApiParam(value = "version", defaultValue = "")
                                 @RequestParam("version") String version,
                                 @ApiParam(value = "id", defaultValue = "")
                                 @RequestParam("id") String id){
        return stdRedisService.getDataSetCode(version, id);
    }

    @ApiOperation("通过StdDataSet版本和id获取标准数据集名称")
    @RequestMapping(value = ServiceApi.Redis.StdDataSetName, method = RequestMethod.GET)
    public String getDataSetName(@ApiParam(value = "version", defaultValue = "")
                                     @RequestParam("version") String version,
                                 @ApiParam(value = "id", defaultValue = "")
                                 @RequestParam("id") String id){
        return stdRedisService.getDataSetName(version, id);
    }


    @ApiOperation("通过StdDataSet版本和编码获取标准数据集名称")
    @RequestMapping(value = ServiceApi.Redis.StdDataSetNameByCode, method = RequestMethod.GET)
    public String getDataSetNameByCode(@ApiParam(value = "version", defaultValue = "")
                                           @RequestParam("version") String version,
                                       @ApiParam(value = "code", defaultValue = "")
                                           @RequestParam("code") String code){
        return stdRedisService.getDataSetNameByCode(version, code);
    }


    @ApiOperation("通过StdDataSet版本和编码获取标准数据集主从表信息")
    @RequestMapping(value = ServiceApi.Redis.StdDataSetMultiRecord, method = RequestMethod.GET)
    public Boolean getDataSetMultiRecord(@ApiParam(value = "version", defaultValue = "")
                                            @RequestParam("version") String version,
                                        @ApiParam(value = "code", defaultValue = "")
                                            @RequestParam("code") String code){
        return stdRedisService.getDataSetMultiRecord(version, code);
    }


    @ApiOperation("通过StdDataSet版本、编码以及标准数据元的内部编码获取标准数据元对应类型")
    @RequestMapping(value = ServiceApi.Redis.StdMetadataType, method = RequestMethod.GET)
    public String getMetaDataType(@ApiParam(value = "version", defaultValue = "")
                                      @RequestParam("version") String version,
                                  @ApiParam(value = "dataSetCode", defaultValue = "")
                                  @RequestParam("dataSetCode") String dataSetCode,
                                  @ApiParam(value = "innerCode", defaultValue = "")
                                  @RequestParam("innerCode") String innerCode) {

        return stdRedisService.getMetaDataType( version, dataSetCode , innerCode);
    }

    @ApiOperation("通过StdDataSet版本、编码以及标准数据元的内部编码获取标准数据元字典ID")
    @RequestMapping(value = ServiceApi.Redis.StdMetadataDict, method = RequestMethod.GET)
    public String getMetaDataDict(@ApiParam(value = "version", defaultValue = "")
                                      @RequestParam("version") String version,
                                  @ApiParam(value = "dataSetCode", defaultValue = "")
                                      @RequestParam("dataSetCode") String dataSetCode,
                                  @ApiParam(value = "innerCode", defaultValue = "")
                                      @RequestParam("innerCode") String innerCode) {
        return stdRedisService.getMetaDataDict(version, dataSetCode, innerCode);
    }


    @ApiOperation("通过StdDataSet版本、标准数据元字典ID以及标准数据元字典项编码获取标准数据元字典项值")
    @RequestMapping(value = ServiceApi.Redis.StdDictEntryValue, method = RequestMethod.GET)
    public String getDictEntryValue(@ApiParam(value = "version", defaultValue = "")
                                        @RequestParam("version") String version,
                                    @ApiParam(value = "dictId", defaultValue = "")
                                    @RequestParam("dictId") String dictId,
                                    @ApiParam(value = "entryCode", defaultValue = "")
                                    @RequestParam("entryCode") String entryCode) {

        return stdRedisService.getDictEntryValue(version, dictId , entryCode);
    }


    @ApiOperation("通过版本号,数据集code查找数据元,返回通过,分隔的数据元字符串")
    @RequestMapping(value = ServiceApi.Redis.StdMetadataCodes, method = RequestMethod.GET)
    public String getMetadataCodes(@ApiParam(value = "version", defaultValue = "")
                                   @RequestParam("version") String version,
                                   @ApiParam(value = "datasetCode", defaultValue = "")
                                   @RequestParam("datasetCode") String datasetCode){
        return stdRedisService.getMetadataCodes(version, datasetCode);
    }

    @ApiOperation("通过版本号、标准数据集code和数据元code 获取标准数据元名称")
    @RequestMapping(value = ServiceApi.Redis.StdMetadataName, method = RequestMethod.GET)
    public String getMetadataName(@ApiParam(value = "version", defaultValue = "")
                                   @RequestParam("version") String version,
                                   @ApiParam(value = "datasetCode", defaultValue = "")
                                   @RequestParam("datasetCode") String datasetCode,
                                  @ApiParam(value = "metadataCode", defaultValue = "")
                                  @RequestParam("metadataCode") String metadataCode){
        return stdRedisService.getMetadataName(version, datasetCode,metadataCode);
    }

    @ApiOperation("通过版本号、标准数据集code和数据元code 获取标准数据元是否为空")
    @RequestMapping(value = ServiceApi.Redis.StdMetadataNullable, method = RequestMethod.GET)
    public boolean isMetaDataNullable(@ApiParam(value = "version", defaultValue = "")
                                  @RequestParam("version") String version,
                                  @ApiParam(value = "datasetCode", defaultValue = "")
                                  @RequestParam("datasetCode") String datasetCode,
                                  @ApiParam(value = "metadataCode", defaultValue = "")
                                  @RequestParam("metadataCode") String metadataCode){
        return stdRedisService.isMetaDataNullable(version, datasetCode,metadataCode);
    }

    @ApiOperation("通过版本号、标准数据集code和数据元code 获取标准数据元是否为空")
    @RequestMapping(value = ServiceApi.Redis.StdMetadataFormat, method = RequestMethod.GET)
    public String getMetadataFormat(@ApiParam(value = "version", defaultValue = "")
                                      @RequestParam("version") String version,
                                      @ApiParam(value = "datasetCode", defaultValue = "")
                                      @RequestParam("datasetCode") String datasetCode,
                                      @ApiParam(value = "metadataCode", defaultValue = "")
                                      @RequestParam("metadataCode") String metadataCode){
        return stdRedisService.getMetadataFormat(version, datasetCode,metadataCode);
    }


    @ApiOperation("Redis缓存标准版本")
    @RequestMapping(value = ServiceApi.Redis.InitVersions, method = RequestMethod.POST)
    public String cacheVersions(
            @ApiParam(name = "versions", value = "版本列表，使用逗号分隔", defaultValue = "59083976eebd")
            @RequestParam("versions") String versions,
            @ApiParam(name = "force", value = "强制清除再缓存", defaultValue = "true")
            @RequestParam("force") boolean force) throws Exception {
        String schema = LocalContext.getContext().getAttachment(ContextAttributes.SCHEMA);
        stdRedisService.cacheVersions(versions, force,schema);
        return "Redis缓存标准版本完成！";
    }

    @ApiOperation("通过StdDataSet版本、标准数据元字典ID以及标准数据元字典项编码判断编码是否存在")
    @RequestMapping(value = ServiceApi.Redis.StdDictEntryCodeExist, method = RequestMethod.GET)
    public Boolean isDictCodeExist(
            @ApiParam(value = "version", defaultValue = "")
            @RequestParam("version") String version,
            @ApiParam(value = "dictId", defaultValue = "")
            @RequestParam("dictId") String dictId,
            @ApiParam(value = "entryCode", defaultValue = "")
            @RequestParam("entryCode") String entryCode) {
        return stdRedisService.isDictCodeExist(version, dictId, entryCode);
    }

}
