package com.yihu.hos.redis.schema.std;

import com.yihu.ehr.redis.schema.KeySchema;
import com.yihu.hos.web.framework.util.StringBuilderEx;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.io.Serializable;
import java.util.*;

/**
 * 标准数据元质控规则 redis辅助类
 * @author HZY
 * @vsrsion 1.0
 * Created at 2018/5/24.
 */
@Component
public class StdQcMetaDataKeySchema extends KeySchema {

    private final static String QC = "qc_";
    @Autowired
    private RedisTemplate<String, Serializable> redisTemplate;

    /**
     * 获取key
     */
    public String getKey(String version,String dataSet,String metaData){
        return new StringBuilderEx(keySchema)
                .arg(QC+version)
                .arg(dataSet)
                .arg(metaData)
                .toString();
    }

    public void setQcMetaData(String version, String dataSetCode, String innerCode, String value) {
        redisClient.set(makeKey(QC + version, dataSetCode , innerCode),value);
    }

    public void multiSetQcMetaData(Map<String, Serializable> data) {
        redisClient.multiSetData(data);
    }



    public void setQcMetaDataList(String version, String dataSetCode, String innerCode, String value) {
        redisClient.set(makeKey(QC + version, dataSetCode, innerCode), value);
    }

    public void deleteQcMetaData(String version, String dataSetCode, String innerCode) {
        redisClient.delete(makeKey(QC + version, dataSetCode ,innerCode));
    }

    /**
     * 通过版本号，模糊删除
     * @param version
     */
    public void deleteQcMetaData(String version ) {
        String replacePattern = new StringBuilderEx("%1%2:%3")
                .arg(QC)
                .arg(version)
                .arg("*")
                .toString();
        redisClient.delete(replacePattern);
    }


    public String getQcMetaData(String version, String dataSetCode, String innerCode) {
        assert version != null && version.length() != 0 &&
                dataSetCode != null && dataSetCode.length() != 0 &&
                innerCode != null && innerCode.length() != 0;
        String key = makeKey(QC + version, dataSetCode, innerCode);
        return redisClient.get(key)== null ? null : this.redisClient.get(key).toString();
//        return this.redisTemplate.opsForValue().get(key) == null ? null : this.redisTemplate.opsForValue().get(key).toString();
    }

    public Boolean isQcMetaDataExist(String version, String dataSetCode, String innerCode) {
        assert StringUtils.isNotBlank(version) && StringUtils.isNotBlank(innerCode);

        return redisClient.hasKey(makeKey( version, dataSetCode ,innerCode));
    }

    /**
     * 模糊获取数据集下的有绑定规则的数据元
     */
    public  List<String>  getMetaDataByDataSet(String stdVersion,String stdDataSet){
       List<String> metaDatas = new ArrayList<>();
       String replacePattern = new StringBuilderEx("%1%2:%3:")
               .arg(QC)
               .arg(stdVersion)
               .arg(stdDataSet)
               .toString();
        Set<String> keys = redisClient.keys(makeKey(QC + stdVersion,stdDataSet,"*"));
        keys.stream().forEach(key ->{
            key = key.replace(replacePattern,"");
            metaDatas.add(key);
        });
        return metaDatas;
    }

    //测试redisTemplate 直取
    public Serializable getValue(String key){
        return  this.redisTemplate.opsForValue().get(key);
    }

}
