package com.yihu.hos.redis.service;


import com.yihu.ehr.redis.client.RedisClient;
import com.yihu.hos.redis.schema.adapter.AdapterDataSetKeySchema;
import com.yihu.hos.redis.schema.adapter.AdapterDictEntryKeySchema;
import com.yihu.hos.redis.schema.adapter.AdapterDictKeySchema;
import com.yihu.hos.redis.schema.adapter.AdapterMetaDataKeySchema;
import com.yihu.hos.redis.task.AdapterRedisTask;
import com.yihu.hos.web.framework.constant.ContextAttributes;
import com.yihu.hos.web.framework.model.standard.AdapterVersion;
import com.yihu.hos.web.framework.thread.LocalContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.AsyncResult;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Future;

@Service
public class AdapterRedisService {

    @Resource(name = "jdbcTemplate")
    private JdbcTemplate jdbc;
    @Autowired
    private AdapterRedisTask adapterRedisTask;
    @Autowired
    private AdapterDataSetKeySchema adapterDataSetKeySchema;
    @Autowired
    private AdapterMetaDataKeySchema adapterMetaDataKeySchema;
    @Autowired
    private AdapterDictKeySchema adapterDictKeySchema;
    @Autowired
    private AdapterDictEntryKeySchema adapterDictEntryKeySchema;
    @Autowired
    private RedisClient redisClient;

    @Async
    public Future<Boolean> cacheVersions(String versions, boolean force,String schema) {
        LocalContext.getContext().setAttachment(ContextAttributes.SCHEMA,schema);
        for (String version : versions.split(",")) {
            List<Map<String, Object>> ver = jdbc.queryForList("select * from adapter_scheme_version where version='"+version+"'");
            if(!CollectionUtils.isEmpty(ver)){
                redisClient.set(version,ver.get(0).get("name")+"");
            }
            AdapterVersion adapterVersion = new AdapterVersion(version);
            String dataSetTable = adapterVersion.getDataSetTableName();
            String metaDataTable = adapterVersion.getMetaDataTableName();
            String dictTable = adapterVersion.getDictTableName();
            String dictEntryTable = adapterVersion.getDictEntryTableName();
            if(force) {
                adapterDataSetKeySchema.delete(dataSetTable);
                adapterMetaDataKeySchema.delete(metaDataTable);
                adapterDictKeySchema.delete(dictTable);
                adapterDictEntryKeySchema.delete(dictEntryTable);
            }
            String dataSetQuery = "SELECT std_dataset_id,std_dataset_code,std_dataset_name,adapter_dataset_id,adapter_dataset_code,adapter_dataset_name FROM data.set.table";

            List<Map<String,Object>> dataSetList = jdbc.queryForList(dataSetQuery.replace("data.set.table", dataSetTable));
            //缓存数据集和数据元
            Future<Boolean> datasetAndMetadataCache = adapterRedisTask.cacheDatasetAndMetadata(dataSetList, dataSetTable, metaDataTable,schema);

            //获取适配字典进行缓存
            List<Map<String,Object>> dictList = jdbc.queryForList("select * from "+dictTable);
            Map<String, String> stdDictMap = new HashMap<>();//目标字典map, key为目标字典id  value 为目标字典code
            Map<String, String> adapterDictMap = new HashMap<>();//源字典map, key为源字典id  value 为源字典code

            //缓存字典
            Future<Boolean> dictCache = adapterRedisTask.cacheDict(dictList, dictTable,schema);
            for (Map<String, Object> temDictMap: dictList) {
                String stdDictCode = (String) temDictMap.get("std_dict_code");
                String stdDictId = String.valueOf(temDictMap.get("std_dict_id"));
                String adapterDictId =  String.valueOf(temDictMap.get("adapter_dict_id"));
                String adapterDictCode = (String) temDictMap.get("adapter_dict_code");
                if(StringUtils.isEmpty(adapterDictCode)){
                    adapterDictCode="";
                }
                stdDictMap.put(stdDictId,stdDictCode);
                adapterDictMap.put(adapterDictId,adapterDictCode);
            }

            //获取字典项进行缓存
            List<Map<String,Object>> dictEntryList = jdbc.queryForList("select * from "+dictEntryTable);
            Future<Boolean> dictEntryCache = adapterRedisTask.cacheDictEntry(dictEntryList, dictEntryTable,stdDictMap,adapterDictMap,schema);

            //数据集,数据元,字典,字典项全部缓存好在进行下个适配版本的缓存
            for (;;) {
                if(datasetAndMetadataCache.isDone() && dictCache.isDone() && dictEntryCache.isDone()) {
                    // 三个任务都调用完成，退出循环等待
                    break;
                }
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }
        return new AsyncResult<Boolean>(true);
    }

    /**
     * 根据适配版本号和数据集code,获取源数据集code
     * @param version 适配版本号
     * @param stdDatasetCode 数据集code
     * @return
     */
    public String getAdapterDatasetCode(String version, String stdDatasetCode) {
        AdapterVersion adapterVersion = new AdapterVersion(version);
        return adapterDataSetKeySchema.getAdapterDatasetCode(adapterVersion.getDataSetTableName(),stdDatasetCode);
    }

    /**
     * 根据适配版本号和目标数据集code,目标数据元code,获取源数据元code
     * @param version 适配版本号
     * @param stdDatasetCode 数据集code
     * @return
     */
    public String getAdapterMetadataCode(String version, String stdDatasetCode, String stdMetadataCode) {
        AdapterVersion adapterVersion = new AdapterVersion(version);
        return adapterMetaDataKeySchema.getAdapterMetaDataCode(adapterVersion.getMetaDataTableName(),stdDatasetCode,stdMetadataCode);
    }

    /**
     * 根据适配版本号和目标字典code,获取的源字典code
     * @param version  适配版本号
     * @param stdDictCode  目标字典code
     * @return
     */
    public String getAdapterDictCode(String version, String stdDictCode) {
        AdapterVersion adapterVersion = new AdapterVersion(version);
        return adapterDictKeySchema.getAdapterDictCode(adapterVersion.getDictTableName(),stdDictCode);
    }

    public String getAdapterDictEntryCode(String version, String stdDictCode, String stdDictEntryCode) {
        AdapterVersion adapterVersion = new AdapterVersion(version);
        return adapterDictEntryKeySchema.getAdapterDictEntryCode(adapterVersion.getDictEntryTableName(),stdDictCode,stdDictEntryCode);
    }
}
