package com.yihu.hos.redis.service;

import com.yihu.ehr.redis.schema.StdCdaVersionKeySchema;
import com.yihu.ehr.redis.schema.StdDataSetKeySchema;
import com.yihu.ehr.redis.schema.StdMetaDataKeySchema;
import com.yihu.hos.redis.task.StdRedisTask;
import com.yihu.hos.web.framework.constant.ContextAttributes;
import com.yihu.hos.web.framework.thread.LocalContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.AsyncResult;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Future;


/**
 * Redis管理
 * @author hzp add at 20170425
 */
@Service
public class StdRedisService {

    @Autowired
    private StdCdaVersionKeySchema stdVersionKeySchema;
    @Autowired
    private StdDataSetKeySchema stdDataSetKeySchema;
    @Autowired
    private StdMetaDataKeySchema stdMetaDataKeySchema;
    @Resource(name = "jdbcTemplate")
    private JdbcTemplate jdbc;
    @Autowired
    private StdRedisTask stdRedisTask;

    /**
     *获取标准版本 redis
     */
    public String getStdVersion(String key)
    {
        return stdVersionKeySchema.get(key);
    }

    /**
     *获取标准数据集代码 redis
     */
    public String getDataSetCode(String version, String id){
        return stdDataSetKeySchema.dataSetCode(version, id);
    }

    /**
     *获取标准数据集名称 redis
     */
    public String getDataSetName(String version, String id){
        return stdDataSetKeySchema.dataSetName(version, id);
    }

    /**
     *获取标准数据集名称 redis
     */
    public String getDataSetNameByCode(String version, String code){
        return stdDataSetKeySchema.dataSetNameByCode(version, code);
    }

    /**
     *获取标准数据集--主从表 redis
     */
    public Boolean getDataSetMultiRecord(String version, String code){
        return stdDataSetKeySchema.dataSetMultiRecord(version, code);
    }

    /**
     * 获取标准数据元对应类型 redis
     */
    public String getMetaDataType(String version, String dataSetCode, String innerCode) {

        return stdMetaDataKeySchema.metaDataType(version, dataSetCode , innerCode);
    }

    /**
     * 获取标准数据元名称 redis
     */
    public String getMetadataName(String version, String datasetCode, String metadataCode) {
        return stdMetaDataKeySchema.metaName(version, datasetCode , metadataCode);
    }


    /**
     * 获取标准数据元是否为空 redis
     */
    public boolean isMetaDataNullable(String version, String datasetCode, String metadataCode) {
        return stdMetaDataKeySchema.metaDataNullable(version, datasetCode , metadataCode);
    }

    /**
     * 获取标准数据元格式 redis
     */
    public String getMetadataFormat(String version, String datasetCode, String metadataCode) {
        return stdMetaDataKeySchema.metaDataFormat(version, datasetCode , metadataCode);
    }


    /**
     * 获取标准数据元对应字典 redis
     */
    public String getMetaDataDict(String version, String dataSetCode, String innerCode) {
        return stdMetaDataKeySchema.metaDataDict(version, dataSetCode,innerCode);
    }

    /**
     * 获取标准数据字典对应值 redis
     */
    public String getDictEntryValue(String version, String dictId, String entryCode) {

        return stdMetaDataKeySchema.dictEntryValue(version, dictId , entryCode);
    }

    /**
     * 缓存版本
     * @param versions
     * @param force
     * @return
     */
    @Async
    public Future<Boolean> cacheVersions(String versions, boolean force,String schema) {
        String versionQuery = "SELECT version_name FROM std_version WHERE version = 'std.version.column'" ;
        String dataSetQuery = "SELECT id, code, name, multi_record FROM data.set.table";
        String metaDataQuery = "SELECT a.code AS data_set_code, b.inner_code, b.type, b.dict_id,b.name FROM data.set.table a, " +
                "meta.data.table b WHERE a.id = b.dataset_id";
        String dictEntryQuery = "SELECT t.dict_id, t.code, t.value FROM dict.entry.table t";
        LocalContext.getContext().setAttachment(ContextAttributes.SCHEMA,schema);
        for (String version : versions.split(",")) {
            if(force) {
                stdVersionKeySchema.delete(version);
                stdMetaDataKeySchema.deleteMetaDataDict(version, "*", "*");
                stdMetaDataKeySchema.deleteMetaDataType(version, "*", "*");
                stdMetaDataKeySchema.deleteDictEntryValue(version, "*", "*");
            }
            String dataSetTable = "std_dataset_" + version;
            String metaDataTable = "std_metadata_" + version;
            String dictEntryTable = "std_dictionary_entry_" + version;
            //数据集
            List<Map<String,Object>> dataSetList = jdbc.queryForList(dataSetQuery.replace("data.set.table", dataSetTable));
            Future<Boolean> cacheDataset = stdRedisTask.cacheDataset(version, dataSetList,schema);

            //数据元
            List<Map<String,Object>> metaDataList = jdbc.queryForList(metaDataQuery.replace("data.set.table", dataSetTable).replace("meta.data.table", metaDataTable));
            Future<Boolean> cacheMetadata = stdRedisTask.cacheMetadata(version, metaDataList,force,schema);

            //字典项
            List<Map<String,Object>> dictEntryList = jdbc.queryForList(dictEntryQuery.replace("dict.entry.table", dictEntryTable));
            Future<Boolean> cacheDict = stdRedisTask.cacheDict(version, dictEntryList);
            //该版本数据全部缓存好在进行下个版本的缓存
            for(;;){
                if(cacheDataset.isDone() && cacheMetadata.isDone() && cacheDict.isDone()){
                    break;
                }
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }

            }

            //版本名
            Map<String, Object> versionMap = jdbc.queryForMap(versionQuery.replace("std.version.column", version));
            String versionName = (String) versionMap.get("version_name");
            stdVersionKeySchema.set(version, versionName);
        }
        return new AsyncResult<Boolean>(true);
    }

    public String getMetadataCodes(String version, String datasetCode) {
        return stdDataSetKeySchema.metadataCodes(version, datasetCode);
    }

    /**
     * 判断值域编码是否存在
     *
     * @param version
     * @param dictId
     * @param entryCode
     * @return
     */
    public Boolean isDictCodeExist(String version, String dictId, String entryCode) {
        return stdMetaDataKeySchema.isDictCodeExist(version, dictId, entryCode);
    }

}
