package com.yihu.hos.redis.task;

import com.yihu.ehr.redis.schema.StdDataSetKeySchema;
import com.yihu.ehr.redis.schema.StdMetaDataKeySchema;
import com.yihu.hos.core.log.LogService;
import com.yihu.hos.web.framework.constant.ContextAttributes;
import com.yihu.hos.web.framework.model.standard.StandardVersion;
import com.yihu.hos.web.framework.thread.LocalContext;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.AsyncResult;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Future;

@Component
public class StdRedisTask {


    @Autowired
    private StdDataSetKeySchema stdDataSetKeySchema;
    @Autowired
    private StdMetaDataKeySchema stdMetaDataKeySchema;
    @Resource(name = "jdbcTemplate")
    private JdbcTemplate jdbc;

    /**
     * 缓存标准的数据集
     * @param version
     * @param dataSetList
     * @return
     */
    @Async
    public Future<Boolean> cacheDataset(String version, List<Map<String,Object>> dataSetList,String scheme){
        System.out.println("开始缓存标准数据集");
        LocalContext.getContext().setAttachment(ContextAttributes.SCHEMA,scheme);
        StandardVersion standardVersion = new StandardVersion(version);
        for (Map<String, Object> tempMap: dataSetList) {
            String id = tempMap.get("id").toString();
            String code = (String) tempMap.get("code");
            String name = (String) tempMap.get("name");
            boolean multiRecord = (boolean) tempMap.get("multi_record");
            stdDataSetKeySchema.setDataSetCode(version, id, code);
            stdDataSetKeySchema.setDataSetName(version, id, name);
            stdDataSetKeySchema.setDataSetNameByCode(version, code, name);
            stdDataSetKeySchema.setDataSetMultiRecord(version, code, multiRecord);
            //通过该数据集id,查找所有的数据元codes,多个codes用,分隔
            String metaCodes = "";

            String sql = "select t.inner_code from "+standardVersion.getMetaDataTableName()+" t where t.dataset_id=?";
            List<String> metadataCodeList = jdbc.queryForList(sql,String.class, id);
            if(CollectionUtils.isNotEmpty(metadataCodeList)){
                for(String c:metadataCodeList){
                    metaCodes+=","+c;
                }
            }
            if(!StringUtils.isEmpty(metaCodes)){
                metaCodes = metaCodes.substring(1,metaCodes.length());
            }
            stdDataSetKeySchema.setMetadataCode(version, code, metaCodes);

        }
        System.out.println("缓存标准的数据集完成");
        return new AsyncResult<Boolean>(true);
    }

    /**
     * 缓存标准的数据元
     * @param version
     * @param metaDataList
     * @param force
     * @return
     */
    @Async
    public Future<Boolean> cacheMetadata(String version, List<Map<String,Object>> metaDataList, boolean force,String schema){
        System.out.println("开始缓存标准数据元");
        LocalContext.getContext().setAttachment(ContextAttributes.SCHEMA,schema);
        for(Map<String, Object> tempMap : metaDataList) {
            String dataSetCode = (String) tempMap.get("data_set_code");
            String innerCode = (String) tempMap.get("inner_code");
            String name = (String) tempMap.get("name");
            String type = (String) tempMap.get("type");
            Integer dictId = (Integer) tempMap.get("dict_id");
            String nullable = (String) tempMap.get("nullable");
            if(StringUtils.isEmpty(nullable)){
                nullable="1";//1为可以为空
            }
            String formate =(String) tempMap.get("formate");
            if(StringUtils.isEmpty(formate)){
                formate="";
            }
            String metaDataTypeKey = stdMetaDataKeySchema.makeKey(version, dataSetCode, innerCode);
            if (!force && stdMetaDataKeySchema.hasKey(metaDataTypeKey)) {
                LogService.getLogger().warn("Meta data duplicated: " + metaDataTypeKey);
            }
            stdMetaDataKeySchema.setMetaDataName(version, dataSetCode,innerCode, name);
            stdMetaDataKeySchema.setMetaDataType(version, dataSetCode, innerCode, type);
            if(dictId != null && dictId != 0 ){
                stdMetaDataKeySchema.setMetaDataDict(version, dataSetCode, innerCode, String.valueOf(dictId));
            }
            stdMetaDataKeySchema.setMetaDataNullable(version, dataSetCode, innerCode, nullable);
            stdMetaDataKeySchema.setMetaDataFormat(version, dataSetCode, innerCode, formate);
        }
        System.out.println("缓存标准数据元完成");
        return new AsyncResult<Boolean>(true);
    }

    /**
     * 缓存标准的字典
     * @param version
     * @param dictEntryList
     * @return
     */
    @Async
    public Future<Boolean> cacheDict(String version,List<Map<String,Object>> dictEntryList){
        System.out.println("开始缓存标准字典");
        for (Map<String, Object> tempMap: dictEntryList) {
            String dictId = tempMap.get("dict_id").toString();
            String code = (String) tempMap.get("code");
            String value = (String) tempMap.get("value");
            stdMetaDataKeySchema.setDictEntryValue(version, dictId, code, value);
        }
        System.out.println("缓存标准字典完成");
        return new AsyncResult<Boolean>(true);
    }
}
