package com.yihu.hos.services;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.hos.core.log.Logger;
import com.yihu.hos.core.log.LoggerFactory;
import com.yihu.hos.web.framework.constant.MycatConstant;
import com.yihu.hos.web.framework.model.bo.ServiceMycat;
import io.zbus.mq.Broker;
import io.zbus.mq.Message;
import io.zbus.mq.Producer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;

/**
 * mycat配置文件操作 消息到ＭＱ
 *
 * @author HZY
 * @vsrsion 1.0
 * Created at 2017/1/5.
 */
@Component
public class ServiceMycatEventService {
    static final Logger logger = LoggerFactory.getLogger(ServiceMycatEventService.class);

    @Autowired
    private ObjectMapper objectMapper;
    private Broker broker;

    @Autowired
    public void setBroker(Broker broker) {
        this.broker = broker;
    }

    public void executeMycatConfig(ServiceMycat servviceMycat, String tenantAndHost) {
        this.sendMsg(MycatConstant.EXECUTE_MYCAT, servviceMycat, tenantAndHost);
    }


    private void sendMsg(String event, ServiceMycat servviceMycat, String tenantAndHost) {
        if (broker == null) {
            logger.error("broker is null.");
            return;
        }

        try {
            String msg = objectMapper.writeValueAsString(servviceMycat);

            String topic = MycatConstant.ZBUS_MQ + "@" + tenantAndHost;
            Producer producer = new Producer(broker);
            //确定为创建消息队列需要显示调用
            producer.declareTopic(topic);
            Message message = new Message();
            message.setTopic(topic);
            message.setHeader("event", event);
            message.setHeader("tenant", tenantAndHost);
            message.setMethod("POST");
            message.setBody(msg);
            producer.publish(message);

        } catch (JsonProcessingException e) {
            e.printStackTrace();
            logger.error(e.getMessage());
        } catch (InterruptedException | IOException e) {
            e.printStackTrace();
        }
    }


}
