package com.yihu.hos.services;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.hos.core.log.Logger;
import com.yihu.hos.core.log.LoggerFactory;
import com.yihu.hos.web.framework.constant.ContextAttributes;
import com.yihu.hos.web.framework.constant.SSHConstant;
import com.yihu.hos.web.framework.model.bo.ServiceShell;
import com.yihu.hos.web.framework.thread.LocalContext;
import io.zbus.mq.Broker;
import io.zbus.mq.Message;
import io.zbus.mq.Producer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;

//import org.zbus.broker.ZbusBroker;
//import org.zbus.mq.Producer;
//import org.zbus.net.http.Message;

/**
 * 发送shell命令消息到ＭＱ
 *
 * @author HZY
 * @vsrsion 1.0
 * Created at 2017/1/5.
 */
@Component
public class ServiceShellEventService {
    static final Logger logger = LoggerFactory.getLogger(ServiceShellEventService.class);

    @Autowired
    private ObjectMapper objectMapper;

    private Broker broker;

    @Autowired
    public void setBroker(Broker broker) {
        this.broker = broker;
    }

    public void serviceShellSend(ServiceShell serviceShell) {
        serviceShell.setType(SSHConstant.ARBITER_SHELL_SEND);
        this.sendMsg(SSHConstant.ARBITER_SHELL_SEND, serviceShell);
    }


    private void sendMsg(String event, ServiceShell serviceShell) {
        if (broker == null) {
            logger.error("broker is null.");
            return;
        }

        try {
            String msg = objectMapper.writeValueAsString(serviceShell);
            String tenant = LocalContext.getContext().getAttachment(ContextAttributes.TENANT_NAME);

            String topic = SSHConstant.SHELL_REQUEST + "@" + tenant;
            Producer producer = new Producer(broker);
            //确定为创建消息队列需要显示调用
            producer.declareTopic(topic);
            Message message = new Message();
            message.setTopic(topic);
            message.setHeader("event", event);
            message.setHeader("tenant", tenant);
            message.setMethod("POST");
            message.setBody(msg);
            producer.publish(message);
        } catch (IOException | InterruptedException e) {
            logger.error(e.getMessage());
            e.printStackTrace();
        }
    }


}
