package com.yihu.hos.specialdict.controller;


import com.yihu.hos.common.controller.EnvelopRestEndPoint;
import com.yihu.hos.core.constants.MicroServices;
import com.yihu.hos.core.constants.ServiceApi;
import com.yihu.hos.model.specialdict.MDrugDict;
import com.yihu.hos.model.specialdict.MHealthProblemDict;
import com.yihu.hos.model.specialdict.MIcd10Dict;
import com.yihu.hos.model.specialdict.MIndicatorsDict;
import com.yihu.hos.specialdict.model.*;
import com.yihu.hos.specialdict.service.HealthProblemDictService;
import com.yihu.hos.specialdict.service.Icd10HpRelationService;
import com.yihu.hos.web.framework.constant.ErrorCode;
import com.yihu.hos.web.framework.exception.ApiException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

@RestController
@RequestMapping(MicroServices.ApiVersion.Version1_0)
@Api(value = "HealthProblemDictEndPoint", description = "健康问题字典", tags = {"特殊字典-健康问题字典"})
public class HealthProblemDictEndPoint extends EnvelopRestEndPoint {

    @Autowired
    private HealthProblemDictService hpDictService;

    @Autowired
    private Icd10HpRelationService icd10HpRelationService;

    @RequestMapping(value = ServiceApi.Standards.Specialdict.HealthProblemDict.Create, method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ApiOperation(value = "创建新的健康问题字典" )
    public MHealthProblemDict createHpDict(
            @ApiParam(name = "dictionary", value = "字典JSON结构")
            @RequestBody String dictJson) throws Exception {

        HealthProblemDict dict = toEntity(dictJson, HealthProblemDict.class);
        dict.setCreateDate(new Date());
        HealthProblemDict healthProblemDict = hpDictService.createDict(dict);
        return convertToModel(healthProblemDict, MHealthProblemDict.class, null);
    }

     @RequestMapping(value = "dict/hp/{id}", method = RequestMethod.DELETE)
     @ApiOperation(value = "根据id删除健康问题字典（含健康问题字典及与ICD10的关联关系。）")
     public boolean deleteHpDict(
            @ApiParam(name = "id", value = "字典代码")
            @PathVariable( value = "id") long id) throws Exception {

        List<Icd10HpRelation> icd10HpRelationList = icd10HpRelationService.getHpIcd10RelationListByHpId(id);
        if (icd10HpRelationList != null) {
            for(Icd10HpRelation icd10HpRelation : icd10HpRelationList){
                icd10HpRelationService.deleteEntity(icd10HpRelation);
            }
        }
        hpDictService.deleteEntity(HealthProblemDict.class,id);

        return true;
    }

    @RequestMapping(value =ServiceApi.Standards.Specialdict.HealthProblemDict.Delete, method = RequestMethod.DELETE)
    @ApiOperation(value = "根据ids批量删除健康问题字典（含健康问题字典及与ICD10的关联关系。）")
    public boolean deleteHpDicts(
            @ApiParam(name = "ids", value = "字典代码,多个以逗号隔开")
            @RequestParam( value = "ids") String ids) throws ParseException {

        String[] hpStrIds = ids.split(",");
        Long[] hpIds = new Long[hpStrIds.length];
        List<Long> relationIds = new ArrayList<>();
        for(int i=0; i<hpStrIds.length;i++){
            long hpId = Long.parseLong(hpStrIds[i]);
            hpIds[i] = hpId;
            List<Icd10HpRelation> icd10HpRelationList = icd10HpRelationService.getHpIcd10RelationListByHpId(hpId);
            if (icd10HpRelationList != null) {
                for(Icd10HpRelation icd10HpRelation : icd10HpRelationList){
                    relationIds.add(icd10HpRelation.getId());
                }
            }
        }
        if (relationIds.size() != 0){
            icd10HpRelationService.delete(relationIds);
        }
        hpDictService.delete(hpIds);
        return true;
    }

    @RequestMapping(value = ServiceApi.Standards.Specialdict.HealthProblemDict.Update, method = RequestMethod.PUT, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ApiOperation(value = "更新健康问题字典" )
    public MHealthProblemDict updateHpDict(
            @ApiParam(name = "dictionary", value = "字典JSON结构")
            @RequestBody String dictJson)  throws Exception{

        HealthProblemDict dict = toEntity(dictJson, HealthProblemDict.class);
        if (null == hpDictService.retrieve(dict.getId())) throw new ApiException(ErrorCode.GetDictFaild, "字典不存在");
        dict.setUpdateDate(new Date());
        hpDictService.saveEntity(dict);
        return convertToModel(dict, MHealthProblemDict.class);
    }

    @RequestMapping(value = ServiceApi.Standards.Specialdict.HealthProblemDict.GetById, method = RequestMethod.GET)
    @ApiOperation(value = "根据ID获取相应的健康问题字典信息。" )
    public MHealthProblemDict getHpDict(
            @ApiParam(name = "id", value = "字典内码")
            @PathVariable(value = "id") long id) throws Exception {

        HealthProblemDict dict = hpDictService.retrieve(id);
        if (dict == null) throw new ApiException(ErrorCode.GetDictFaild, "字典不存在");
        return convertToModel(dict, MHealthProblemDict.class);
    }

    @RequestMapping(value = ServiceApi.Standards.Specialdict.HealthProblemDict.ListPage, method = RequestMethod.GET)
    @ApiOperation(value = "根据查询条件查询相应的疾病字典信息。" )
    public Collection<MHealthProblemDict> getHpDictList(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段", defaultValue = "id,code,name,description")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有信息", defaultValue = "")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档", defaultValue = "+code,+name")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "size", value = "分页大小", defaultValue = "15")
            @RequestParam(value = "size", required = false) int size,
            @ApiParam(name = "page", value = "页码", defaultValue = "1")
            @RequestParam(value = "page", required = false) int page,
            HttpServletRequest request,
            HttpServletResponse response) throws Exception{

        List<HealthProblemDict> hpDictList = hpDictService.search(fields, filters, sorts, page, size);
        pagedResponse(request, response, hpDictService.getCount(filters), page, size);
        return convertToModels(hpDictList, new ArrayList<>(hpDictList.size()), MHealthProblemDict.class, fields);
    }

    @RequestMapping(value = ServiceApi.Standards.Specialdict.HealthProblemDict.IsNameExist, method = RequestMethod.GET)
    @ApiOperation(value = "判断提交的字典名称是否已经存在")
    public boolean isNameExists(
            @ApiParam(name = "name", value = "name", defaultValue = "")
            @RequestParam(value = "name") String name) throws ParseException {
        return hpDictService.isNameExist(name);
    }

    @RequestMapping(value = ServiceApi.Standards.Specialdict.HealthProblemDict.IsCodeExist , method = RequestMethod.GET)
    @ApiOperation(value = "判断提交的字典代码是否已经存在")
    public boolean isCodeExists(
            @ApiParam(name = "code", value = "code", defaultValue = "")
            @PathVariable(value = "code") String code) throws ParseException {
        return hpDictService.isCodeExist(code);
    }

    @RequestMapping(value = ServiceApi.Standards.Specialdict.HealthProblemDict.GetHpDictByCode , method = RequestMethod.GET)
    @ApiOperation(value = "根据字典代码获取健康问题字典")
    public MHealthProblemDict getHpDictByCode(
            @ApiParam(name = "code", value = "code")
            @PathVariable(value = "code") String code) throws ParseException {
        HealthProblemDict healthProblemDict = hpDictService.findHpDictByCode(code);
        return convertToModel(healthProblemDict,MHealthProblemDict.class,null);
    }

    @RequestMapping(value =ServiceApi.Standards.Specialdict.HealthProblemDict.GetDrugDictListByHpCode , method = RequestMethod.GET)
    @ApiOperation(value = "根据健康问题代码获取药品信息")
    public Collection<MDrugDict> getDrugDictListByHpCode(
            @ApiParam(name = "code", value = "code")
            @PathVariable(value = "code") String code) throws Exception{
        List<DrugDict> list = hpDictService.getDrugDictListByHpCode(code);
        return convertToModels(list, new ArrayList<>(list.size()), MDrugDict.class,null);
    }

    @RequestMapping(value = ServiceApi.Standards.Specialdict.HealthProblemDict.GetIndicatorsByHpCode , method = RequestMethod.GET)
    @ApiOperation(value = "根据健康问题代码获取指标信息")
    public Collection<MIndicatorsDict> getIndicatorsByHpCode(
            @ApiParam(name = "code", value = "code")
            @PathVariable(value = "code") String code) throws Exception{
        List<IndicatorsDict> list = hpDictService.getIndicatorsByHpCode(code);
        return convertToModels(list, new ArrayList<>(list.size()), MIndicatorsDict.class,null);
    }

    @RequestMapping(value = ServiceApi.Standards.Specialdict.HealthProblemDict.GetIcd10ByHpCode , method = RequestMethod.GET)
    @ApiOperation(value = "根据健康问题代码获取ICD10代码列表")
    public Collection<MIcd10Dict> getIcd10ByHpCode(
            @ApiParam(name = "code", value = "code")
            @PathVariable(value = "code") String code) throws Exception{
        List<Icd10Dict> list = hpDictService.getIcd10ByHpCode(code);
        return convertToModels(list, new ArrayList<>(list.size()), MIcd10Dict.class,null);
    }


}