package com.yihu.hos.specialdict.controller;


import com.yihu.hos.common.controller.EnvelopRestEndPoint;
import com.yihu.hos.core.constants.MicroServices;
import com.yihu.hos.core.constants.ServiceApi;
import com.yihu.hos.model.specialdict.MIndicatorsDict;
import com.yihu.hos.specialdict.model.IndicatorsDict;
import com.yihu.hos.specialdict.service.Icd10IndicatorRelationService;
import com.yihu.hos.specialdict.service.IndicatorsDictService;
import com.yihu.hos.web.framework.constant.ErrorCode;
import com.yihu.hos.web.framework.exception.ApiException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

@RestController
@RequestMapping(MicroServices.ApiVersion.Version1_0)
@Api(value = "IndicatorsDictEndPoint", description = "指标字典", tags = {"特殊字典-指标字典"})
public class IndicatorsDictEndPoint extends EnvelopRestEndPoint {

    @Autowired
    private IndicatorsDictService indicatorsDictService;
    @Autowired
    private Icd10IndicatorRelationService icd10IndicatorRelationService;

    @RequestMapping(value = ServiceApi.Standards.Specialdict.IndicatorDict.Create, method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ApiOperation(value = "创建新的指标字典" )
    public MIndicatorsDict createIndicatorsDict(
            @ApiParam(name = "dictionary", value = "字典JSON结构")
            @RequestBody String dictJson) throws Exception {
        IndicatorsDict dict = toEntity(dictJson, IndicatorsDict.class);
        dict.setCreateDate(new Date());
        IndicatorsDict indicatorsDict = indicatorsDictService.createDict(dict);
        return convertToModel(indicatorsDict, MIndicatorsDict.class, null);
    }

    @RequestMapping(value = ServiceApi.Standards.Specialdict.IndicatorDict.DeleteById, method = RequestMethod.DELETE)
    @ApiOperation(value = "根据id删除指标字典")
    public boolean deleteIndicatorDict(
            @ApiParam(name = "id", value = "指标字典代码")
            @PathVariable( value = "id") long id) {
        indicatorsDictService.delete(id);
        return true;
    }

    @RequestMapping(value = ServiceApi.Standards.Specialdict.IndicatorDict.Delete, method = RequestMethod.DELETE)
    @ApiOperation(value = "根据ids批量删除指标字典")
    public boolean deleteIndicatorsDict(
            @ApiParam(name = "ids", value = "指标字典代码,多个以逗号分隔")
            @RequestParam( value = "ids") String ids) {

        String[] strIds = ids.split(",");
        Long[] longIds = new Long[strIds.length];
        for(int i=0; i<strIds.length;i++){
            longIds[i] = Long.parseLong(strIds[i]);
        }
        indicatorsDictService.delete(longIds);
        return true;
    }

    @RequestMapping(value = ServiceApi.Standards.Specialdict.IndicatorDict.Update, method = RequestMethod.PUT, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ApiOperation(value = "更新指标字典" )
    public MIndicatorsDict updateIndicatorsDict(
            @ApiParam(name = "dictionary", value = "字典JSON结构")
            @RequestBody String dictJson) throws Exception {

        IndicatorsDict dict = toEntity(dictJson, IndicatorsDict.class);
        dict.setUpdateDate(new Date());
        indicatorsDictService.updateEntity(dict);
        return convertToModel(dict, MIndicatorsDict.class);
    }

    @RequestMapping(value = ServiceApi.Standards.Specialdict.IndicatorDict.GetById, method = RequestMethod.GET)
    @ApiOperation(value = "根据ID获取相应的指标字典信息。" )
    public MIndicatorsDict getIndicatorsDict(
            @ApiParam(name = "id", value = "字典内码")
            @PathVariable(value = "id") long id) throws Exception {

        IndicatorsDict dict = indicatorsDictService.retrieve(id);
        if (dict == null) throw new ApiException(ErrorCode.GetDictFaild, "字典不存在");
        return convertToModel(dict, MIndicatorsDict.class);
    }

    @RequestMapping(value = ServiceApi.Standards.Specialdict.IndicatorDict.ListPage, method = RequestMethod.GET)
    @ApiOperation(value = "根据查询条件查询相应的指标字典信息。" )
    public Collection<MIndicatorsDict> getIndicatorsDictList(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段", defaultValue = "id,code,name,type,unit,upperLimit,lowerLimit,description")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有信息", defaultValue = "")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档", defaultValue = "+code,+name")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "size", value = "分页大小", defaultValue = "15")
            @RequestParam(value = "size", required = false) int size,
            @ApiParam(name = "page", value = "页码", defaultValue = "1")
            @RequestParam(value = "page", required = false) int page,
            HttpServletRequest request,
            HttpServletResponse response) throws Exception{

        List<IndicatorsDict> indicatorsDictList = indicatorsDictService.search(fields, filters, sorts, page, size);
        pagedResponse(request, response, indicatorsDictService.getCount(filters), page, size);
        return convertToModels(indicatorsDictList, new ArrayList<>(indicatorsDictList.size()), MIndicatorsDict.class, fields);
    }

    @RequestMapping(value =ServiceApi.Standards.Specialdict.IndicatorDict.IsUsage , method = RequestMethod.GET)
    @ApiOperation(value = "根据指标的ID判断是否与ICD10字典存在关联。")
    public boolean indicatorIsUsage(
            @ApiParam(name = "id", value = "指标字典代码")
            @PathVariable( value = "id") long id) throws ParseException {

        return icd10IndicatorRelationService.isUsage(id);
    }

    @RequestMapping(value = ServiceApi.Standards.Specialdict.IndicatorDict.IsNameExist , method = RequestMethod.GET)
    @ApiOperation(value = "判断提交的字典名称是否已经存在")
    public boolean isNameExists(
            @ApiParam(name = "name", value = "name", defaultValue = "")
            @RequestParam(value = "name") String name) throws ParseException {
        return indicatorsDictService.isNameExist(name);
    }

    @RequestMapping(value =ServiceApi.Standards.Specialdict.IndicatorDict.IsCodeExist , method = RequestMethod.GET)
    @ApiOperation(value = "判断提交的字典代码是否已经存在")
    public boolean isCodeExists(
            @ApiParam(name = "code", value = "code", defaultValue = "")
            @PathVariable(value = "code") String code) throws ParseException {
        return indicatorsDictService.isCodeExist(code);
    }

    @RequestMapping(value = ServiceApi.Standards.Specialdict.IndicatorDict.GetByCode, method = RequestMethod.GET)
    @ApiOperation(value = "根据code获取相应的指标字典信息" )
    public MIndicatorsDict getIndicatorsDictByCode(
            @ApiParam(name = "code", value = "指标代码")
            @RequestParam(value = "code") String code) throws Exception {
        IndicatorsDict dict = indicatorsDictService.findByCode(code);
        if (dict == null) throw new ApiException(ErrorCode.GetDictFaild, "字典不存在");
        return convertToModel(dict, MIndicatorsDict.class);
    }

    @RequestMapping(value = ServiceApi.Standards.Specialdict.IndicatorDict.GetByCodeList, method = RequestMethod.POST)
    @ApiOperation(value = "根据code获取相应的指标字典信息" )
    public List<MIndicatorsDict> getIndicatorsDictByCodeList(
            @ApiParam(name = "code", value = "指标代码List")
            @RequestParam(value = "codeList") List<String> codeList) throws Exception {
        List<MIndicatorsDict> result=new ArrayList<>();
        for(int i=0;i<codeList.size();i++){
            IndicatorsDict dict = indicatorsDictService.findByCode(codeList.get(i));
            if (dict == null) throw new ApiException(ErrorCode.GetDictFaild, codeList.get(i)+"的字典不存在");
            result.add(convertToModel(dict, MIndicatorsDict.class));
        }
      return result;
    }

    @RequestMapping(value = ServiceApi.Standards.Specialdict.IndicatorDict.GetByIds, method = RequestMethod.GET)
    @ApiOperation(value = "根据ids获取相应的指标字典信息" )
    public List<MIndicatorsDict> getIndicatorsDictByIds(
            @ApiParam(name = "ids", value = "指标代码")
            @RequestParam(value = "ids") String[] ids) throws Exception {
        long[] longIds = new long[ids.length];
        for(int i=0; i<ids.length;i++){
            longIds[i] = Long.parseLong(ids[i]);
        }
        List<IndicatorsDict> indicatorsDicts = indicatorsDictService.getIndicatorsDictByIds(longIds);
        return (List<MIndicatorsDict>)convertToModels(indicatorsDicts, new ArrayList<>(indicatorsDicts.size()), MIndicatorsDict.class, "");
    }

}