package com.yihu.hos.specialdict.service;


import com.yihu.hos.core.datatype.CollectionUtil;
import com.yihu.hos.specialdict.model.DrugDict;
import com.yihu.hos.specialdict.model.HealthProblemDict;
import com.yihu.hos.specialdict.model.Icd10Dict;
import com.yihu.hos.specialdict.model.IndicatorsDict;
import com.yihu.hos.web.framework.baseservice.BaseService;
import org.hibernate.SQLQuery;
import org.hibernate.Session;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.ParseException;
import java.util.List;

/**
 * 用户公私钥管理
 *
 * @author CWS
 * @version 1.0
 * @created 02-6月-2015 17:38:05
 */
@Transactional
@Service
public class HealthProblemDictService extends BaseService<HealthProblemDict>{

    @Autowired
    private Icd10HpRelationService icd10HpRelationService;

    public boolean isCodeExist(String code) throws ParseException {
        HealthProblemDict hpDict = findByCode(code);
        return hpDict != null;
    }

    public boolean isNameExist(String name) throws ParseException {
        HealthProblemDict hpDict = findByName(name);
        return hpDict != null;
    }

    public HealthProblemDict createDict(HealthProblemDict dict) throws Exception {
        dict.setName(dict.getName());
        saveEntity(dict);
        return dict;
    }

    public boolean isUsage(long id) throws ParseException {
        boolean result = (icd10HpRelationService.findByHpId(id) != null);
        return result;
    }

    public HealthProblemDict findHpDictByCode(String code) throws ParseException {
        return findByCode(code);
    }

    //根据健康问题代码获取药品信息
    public List<DrugDict> getDrugDictListByHpCode(String code) throws Exception{
        Session session = getCurrentSession();
        String sql = "select d.* from icd10_drug_relation idr,drug_dict d,\n" +
                "(select i.id as icd10_id\n" +
                "from hp_icd10_relation r,icd10_dict i,health_problem_dict h\n" +
                "where r.hp_id=h.id \n" +
                "and r.icd10_id=i.id\n" +
                "and h.code=:code) icd10\n" +
                "where idr.drug_id=d.id\n" +
                "and idr.icd10_id = icd10.icd10_id";
        SQLQuery query = session.createSQLQuery(sql);
        query.setParameter("code", code);
        query.addEntity(DrugDict.class);
        List<DrugDict> list = query.list();

        return list;
    }

    //根据健康问题代码获取指标信息
    public List<IndicatorsDict> getIndicatorsByHpCode(String code) throws Exception{
        Session session = getCurrentSession();
        String sql = "select d.* from icd10_indicator_relation iir,indicators_dict d,\n" +
                "(select i.id as icd10_id\n" +
                "from hp_icd10_relation r,icd10_dict i,health_problem_dict h\n" +
                "where r.hp_id=h.id \n" +
                "and r.icd10_id=i.id\n" +
                "and h.code=:code) icd10\n" +
                "where iir.indicator_id=d.id\n" +
                "and iir.icd10_id = icd10.icd10_id";
        SQLQuery query = session.createSQLQuery(sql);
        query.setParameter("code", code);
        query.addEntity(IndicatorsDict.class);
        List<IndicatorsDict> list = query.list();
        return list;
    }

    //根据健康问题代码获取ICD10代码列表
    public List<Icd10Dict> getIcd10ByHpCode(String code) throws Exception{
        Session session = getCurrentSession();
        String sql = "select i.* \n" +
                "from hp_icd10_relation r,icd10_dict i,health_problem_dict h\n" +
                "where r.hp_id=h.id \n" +
                "and r.icd10_id=i.id\n" +
                "and h.code=:code";

        SQLQuery query = session.createSQLQuery(sql);
        query.setParameter("code", code);
        query.addEntity(Icd10Dict.class);
        List<Icd10Dict> list = query.list();

        return list;
    }

    public HealthProblemDict findByCode(String code) throws ParseException {
        List search = search("code=" + code);
        if(CollectionUtil.isEmpty(search)){
            return null;
        }
        return (HealthProblemDict) search.get(0);
    }

    public HealthProblemDict findByName(String name) throws ParseException {
        List search = search("name=" + name);
        if(CollectionUtil.isEmpty(search)){
            return null;
        }
        return (HealthProblemDict) search.get(0);
    }





}