package com.yihu.hos.specialdict.service;


import com.yihu.hos.core.datatype.CollectionUtil;
import com.yihu.hos.specialdict.model.DrugDict;
import com.yihu.hos.specialdict.model.Icd10DrugRelation;
import com.yihu.hos.web.framework.baseservice.BaseService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;

/**
 * 用户公私钥管理
 *
 * @author CWS
 * @version 1.0
 * @created 02-6月-2015 17:38:05
 */
@Transactional
@Service
public class Icd10DrugRelationService extends BaseService<Icd10DrugRelation>{


    public boolean isExist(long icd10Id,long drugId) throws ParseException {
        Icd10DrugRelation icd10DrugRelation = findByIcd10IdAndDrugId(icd10Id, drugId);
        return icd10DrugRelation != null;
    }

    public List<Icd10DrugRelation> getIcd10DrugRelationListByIcd10Id(long icd10Id) throws ParseException {

        List<Icd10DrugRelation> icd10DrugRelations = findByIcd10Id(icd10Id);
        if(CollectionUtil.isEmpty(icd10DrugRelations)){
            return null;
        }
        return icd10DrugRelations;
    }

    public boolean isUsage(long drugId) throws ParseException {
        List<Icd10DrugRelation> icd10DrugRelations = findByDrugId(drugId);
        if(CollectionUtil.isEmpty(icd10DrugRelations)){
            return false;
        }
        return true;
    }

    public List<Icd10DrugRelation> getIcd10DrugRelationsByIcd10Ids(long[] icd10Ids) {
        return findByIcd10Ids(icd10Ids);
    }

    public List<Icd10DrugRelation> findByIcd10Id(long icd10Id) throws ParseException {
        List search = search("icd10Id=" + icd10Id);
        if(CollectionUtil.isEmpty(search)){
            return null;
        }
        return search;
    }
    public List<Icd10DrugRelation> findByDrugId(long drugId) throws ParseException {
        List search = search("drugId=" + drugId);
        if(CollectionUtil.isEmpty(search)){
            return null;
        }
        return search;
    }

    public Icd10DrugRelation findByIcd10IdAndDrugId(long icd10Id, long drugId) throws ParseException {
        List search = search("drugId=" + drugId+"&icd10Id="+icd10Id);
        if(CollectionUtil.isEmpty(search)){
            return null;
        }
        return (Icd10DrugRelation) search.get(0);
    }

    public List<Icd10DrugRelation> findByIcd10Ids(long[] icd10Ids){
        if(icd10Ids==null){
            return null;
        }
        List<Long> longs = new ArrayList<>();
        for(long id:icd10Ids){
            longs.add(id);
        }
        return getListByIdList(DrugDict.class,"drug_dict","id",longs);
    }
}