package com.yihu.hos.specialdict.service;


import com.yihu.hos.core.datatype.CollectionUtil;
import com.yihu.hos.specialdict.model.DrugDict;
import com.yihu.hos.specialdict.model.Icd10IndicatorRelation;
import com.yihu.hos.web.framework.baseservice.BaseService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;

/**
 * 用户公私钥管理
 *
 * @author CWS
 * @version 1.0
 * @created 02-6月-2015 17:38:05
 */
@Transactional
@Service
public class Icd10IndicatorRelationService extends BaseService<Icd10IndicatorRelation>{


    public boolean isExist(long icd10Id,long indicatorId) throws ParseException {
        Icd10IndicatorRelation icd10IndicatorRelation = findByIcd10IdAndIndicatorId(icd10Id, indicatorId);
        return  icd10IndicatorRelation != null;
    }

    public List<Icd10IndicatorRelation> getIcd10IndicatorRelationListByIcd10Id(long icd10Id) throws ParseException {
        List<Icd10IndicatorRelation> icd10IndicatorRelations = findByIcd10Id(icd10Id);
        if(CollectionUtil.isEmpty(icd10IndicatorRelations)){
            return null;
        }
        return icd10IndicatorRelations;
    }

    public boolean isUsage(long indicatorId) throws ParseException {
        List<Icd10IndicatorRelation> icd10IndicatorRelations = findByIndicatorId(indicatorId);
        if(CollectionUtil.isEmpty(icd10IndicatorRelations)){
            return false;
        }
        return true;
    }

    public List<Icd10IndicatorRelation> getIcd10IndicatorRelationsByIcd10Ids(long[] icd10Ids) {
        return findByIcd10Ids(icd10Ids);
    }

    public List<Icd10IndicatorRelation> findByIcd10Id(long icd10Id) throws ParseException {
        List search = search("icd10Id=" + icd10Id);
        if(CollectionUtil.isEmpty(search)){
            return null;
        }
        return search;
    }
    public List<Icd10IndicatorRelation> findByIndicatorId(long indicatorId) throws ParseException {
        List search = search("indicatorId=" + indicatorId);
        if(CollectionUtil.isEmpty(search)){
            return null;
        }
        return search;
    }

    public Icd10IndicatorRelation findByIcd10IdAndIndicatorId(long icd10Id, long indicatorId) throws ParseException {
        List search = search("indicatorId=" + indicatorId+"&icd10Id="+icd10Id);
        if(CollectionUtil.isEmpty(search)){
            return null;
        }
        return (Icd10IndicatorRelation) search.get(0);
    }

    public List<Icd10IndicatorRelation> findByIcd10Ids(long[] icd10Ids){
        if(icd10Ids==null){
            return null;
        }
        List<Long> longs = new ArrayList<>();
        for(long id:icd10Ids){
            longs.add(id);
        }
        return getListByIdList(DrugDict.class,"drug_dict","id",longs);
    }
}