package com.yihu.hos.standard.controller;

import com.netflix.hystrix.contrib.javanica.annotation.HystrixCommand;
import com.netflix.hystrix.contrib.javanica.annotation.HystrixProperty;
import com.yihu.hos.standard.service.adapter.AdapterDatasetService;
import com.yihu.hos.web.framework.model.DetailModelResult;
import com.yihu.hos.web.framework.model.Envelop;
import com.yihu.hos.web.framework.model.Result;
import com.yihu.hos.web.framework.util.controller.BaseController;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.json.JSONObject;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.io.IOException;

/**
 * Created by LLH on 2016/1/6.
 */
@RestController("AdapterDatasetController")
@RequestMapping("/adapterDatasetEndpoint")
//@Api(protocols = "https", value = "AdapterDatasetController", description = "适配数据集管理", tags = {"适配数据集"})
public class AdapterDatasetController extends BaseController {

    @Resource(name = AdapterDatasetService.BEAN_ID)
    private AdapterDatasetService datasetService;
    /**
     * 查询数据集列表信息（get）
     * @param adapterVersion
     * @param condition
     * @param order
     * @param rows
     * @param page
     * @return
     * @throws Exception
     */
    @RequestMapping(value = "/searchDatasetList")
    @ApiOperation(value = "获取数据集列表", response = DetailModelResult.class, produces = "application/json", notes = "获取数据集列表")
    public DetailModelResult getDatasetList(
            @ApiParam(name = "adapterVersion", value = "适配方案版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "condition", value = "查询条件：编码或名称")
            @RequestParam(value = "condition", required = false) String condition,
            @ApiParam(name = "order", value = "Order param,Must be json.ascending:asc,descending:desc")
            @RequestParam(value = "order", required = false) String order,
            @ApiParam(name = "rows", value = "Limit the size of result set. Must be an integer")
            @RequestParam(value = "rows", required = false) Integer rows,
            @ApiParam(name = "page", value = "Start position of result set. Must be an integer")
            @RequestParam(value = "page", required = false) Integer page) {

        return datasetService.getDetailModelResult(adapterVersion, condition, order, rows, page);
    }

    @RequestMapping("/getForDatasetId")
    @ApiOperation(value = "获取适配数据集", response = Result.class, produces = "application/json", notes = "获取适配数据集")
    public Result getForDatasetId(
            @ApiParam(name = "adapterVersion", value = "适配版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "datasetId", value = "适配数据集ID")
            @RequestParam(value = "datasetId") Integer datasetId) {

        return datasetService.get(adapterVersion, datasetId);
    }

    @RequestMapping(value = "/delDataset")
    @ApiOperation(value = "删除数据集", response = Result.class, produces = "application/json", notes = "删除适配数据集")
    public Result delDataset(
            @ApiParam(name = "adapterVersion", value = "适配版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "datasetId", value = "适配数据集ID")
            @RequestParam(value = "datasetId") Integer datasetId) {
       return datasetService.delete(adapterVersion, datasetId);
    }

    @RequestMapping(value = "/updateDataset")
    @ApiOperation(value = "修改适配数据集", response = Result.class, produces = "application/json", notes = "修改适配数据集")
    public Result modify(
            @ApiParam(name = "adapterVersion", value = "适配版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "dataset", value = "数据集,JSON")
            @RequestParam(value = "dataset") String dataset) {
        return datasetService.modify(adapterVersion, dataset);
    }

    @RequestMapping(value = "/addDataset")
    @ApiOperation(value = "保存适配数据集", response = Result.class, produces = "application/json", notes = "保存数据集")
    public Result add(
            @ApiParam(name = "adapterVersion", value = "适配版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "dataset", value = "数据集信息")
            @RequestParam(value = "dataset") String dataset) {
        return datasetService.add(adapterVersion, dataset);
    }

    @RequestMapping(value = "/getAdapterDataset")
    @ApiOperation(value = "获取定制数据集列表", response = DetailModelResult.class, responseContainer = "List",produces = "application/json", notes = "获取数据集列表")
    public DetailModelResult getDatasetCustomize(
            @ApiParam(name = "adapterVersion", value = "适配版本号")
            @RequestParam(value = "adapterVersion") String adapterVersion,
            @ApiParam(name = "stdVersion", value = "标准版本号")
            @RequestParam(value = "stdVersion") String stdVersion) {
        return datasetService.getDatasetCustomize(adapterVersion, stdVersion);
    }

    @RequestMapping(value = "/saveAdapterDataset", method = RequestMethod.POST)
    @ApiOperation(value = "定制适配数据集", response = Result.class, responseContainer = "List",produces = "application/json", notes = "获取数据集列表")
    public Result adapterDataSet(
            @RequestBody String data) {
        JSONObject jsonObject = new JSONObject(data);
        int schemeId = jsonObject.getInt("schemeId");
        String adapterVersion = jsonObject.getString("adapterVersion");
        String stdVersion = jsonObject.getString("stdVersion");
        String datasetIdStr = jsonObject.getString("datasetIdStr");
        String metaDataIdStr = jsonObject.getString("metaDataIdStr");
        return datasetService.updateCustomize(schemeId, adapterVersion, stdVersion, datasetIdStr,metaDataIdStr);//这边的datasetIdStr是belongDatasetId
    }

    @ResponseBody
    @RequestMapping(value = "/cloneDataSet", method = RequestMethod.GET)
    @ApiOperation(value = "克隆一个数据集", response = Envelop.class,produces = "application/json")
    public Envelop cloneDataSet(@ApiParam(name = "adapterVersion", value = "适配版本号")
                                @RequestParam(value = "adapterVersion") String adapterVersion,
                                @ApiParam(name = "code", value = "adapter_dataset_* 的std_dataset_code")
                                @RequestParam(value = "code") String code){
        try {
            return datasetService.cloneDataSet(adapterVersion, code);
        } catch (IOException e) {
            e.printStackTrace();
            Envelop envelop = new Envelop();
            envelop.setSuccessFlg(false);
            envelop.setErrorMsg("克隆数据集失败");
            return envelop;
        }
    }

    @ResponseBody
    @RequestMapping(value = "/deleteCloneDataSet", method = RequestMethod.DELETE)
    @ApiOperation(value = "删除克隆的数据集", response = Envelop.class,produces = "application/json")
    public Envelop deleteCloneDataSet(@ApiParam(name = "adapterVersion", value = "适配版本号")
                                      @RequestParam(value = "adapterVersion") String adapterVersion,
                                      @ApiParam(name = "id", value = "adapter_dataset_* 的id")
                                      @RequestParam(value = "id") Long id){
        return datasetService.deleteCloneDataSet(adapterVersion, id);
    }

    @RequestMapping(value = "/isNeedRelation", method = RequestMethod.GET,produces="application/json;charset=UTF-8")
    @ApiOperation(value = "查询是否需要关联关系", response = Envelop.class,produces = "application/json", notes = "查询是否需要关联关系")
    public Envelop isNeedRelation(@ApiParam(name = "adapterVersion", value = "适配版本号")
                                  @RequestParam(value = "adapterVersion") String adapterVersion,
                                  @ApiParam(name = "id", value = "adapter_dataset_* 的id")
                                  @RequestParam(value = "id") Long id){
        return datasetService.isNeedRelation(adapterVersion, id);
    }

    @RequestMapping(value = "/getDataSets", method = RequestMethod.GET,produces="application/json;charset=UTF-8")
    @ApiOperation(value = "查询关联关系的dataset列表", response = Envelop.class,produces = "application/json", notes = "查询是否需要关联关系")
    public Envelop getDataSets(@ApiParam(name = "adapterVersion", value = "适配版本号")
                               @RequestParam(value = "adapterVersion") String adapterVersion,
                               @ApiParam(name = "id", value = "adapter_dataset_* 的id")
                               @RequestParam(value = "id") Long id,
                               @ApiParam(name = "sourceVersion", value = "源标准version")
                               @RequestParam(value = "sourceVersion") String sourceVersion,
                               @ApiParam(name = "page", value = "页码")
                               @RequestParam(value = "page",required = false) Integer page,
                               @ApiParam(name = "rows", value = "每页条数")
                               @RequestParam(value = "rows",required = false) Integer rows,
                               @ApiParam(name = "filterCodeOrName", value = "根据code或者name条件查询")
                               @RequestParam(value = "filterCodeOrName",required = false) String filterCodeOrName) throws IOException {
        return datasetService.getDataSets(adapterVersion, id,sourceVersion,page,rows,filterCodeOrName);
    }

    /**
     * 单个数据集智能匹配
     */
    @ResponseBody
    @RequestMapping(value = "/smartMatchMedata", method = RequestMethod.GET,produces = "application/json")
    @ApiOperation(value = "单个字典智能匹配", response = Envelop.class, produces = "application/json", notes = "单个字典智能匹配")
    @HystrixCommand(commandProperties = {
            @HystrixProperty(name = "execution.isolation.thread.timeoutInMilliseconds", value = "-1"),//超时时间
            @HystrixProperty(name = "execution.timeout.enabled", value = "false") })
    public Envelop smartMatchMedata(
            @ApiParam(value = "源version", required = true) @RequestParam("std_version") String std_version,
            @ApiParam(value = "适配方案版本号", required = true) @RequestParam("adapter_version") String adapter_version,
            @ApiParam(value = "适配表的id(目标的id)", required = true) @RequestParam("id") Long id,
            @ApiParam(value = "适配表的adapterDataSetId(源的stdDatasetId)", required = true) @RequestParam("adapterDataSetId") Long adapterDataSetId) throws Exception {
        return datasetService.smartMatchMedata(std_version,adapter_version,id,adapterDataSetId);
    }


}
