package com.yihu.hos.standard.controller;

import com.yihu.ehr.util.rest.Envelop;
import com.yihu.hos.model.standard.MSTDVersion;
import com.yihu.hos.standard.service.standard.*;
import com.yihu.hos.web.framework.constant.ContextAttributes;
import com.yihu.hos.web.framework.model.ActionResult;
import com.yihu.hos.web.framework.model.DictionaryResult;
import com.yihu.hos.web.framework.model.Result;
import com.yihu.hos.web.framework.thread.LocalContext;
import com.yihu.hos.web.framework.util.controller.BaseController;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;

/**
 * Created by LLH on 2016/1/6.
 */
@RestController("StandardVersionController")
@RequestMapping("/standardVersionEndpoint")
//@Api(protocols = "https", value = "StandardVersionController", description = "标准版本管理", tags = {"标准版本"})
public class StandardVersionController extends BaseController {
    @Resource(name = StandardVersionService.BEAN_ID)
    private StandardVersionService standardVersionService;

    @Autowired
    private StdDatasetService stdDatasetService;
    @Autowired
    private StdDictService stdDictService;
    @Autowired
    private StdMetadataService stdMetadataService;
    @Autowired
    StdDictEntryService stdDictEntryService;



    @RequestMapping("/versions")
    @ApiOperation(value = "获取标准版本列表", response = DictionaryResult.class, produces = "application/json", notes = "获取标准版本列表")
    public Envelop versions(
            @ApiParam(name = "standardId", value = "标准ID")
            @RequestParam(value = "standardId") Long standardId,
            @ApiParam(name = "condition", value = "Must be Json")
            @RequestParam(value = "condition", required = false) String condition,
            @ApiParam(name = "order", value = "Must be Json")
            @RequestParam(value = "order", required = false) String order,
            @ApiParam(name = "rows", value = "Limit the size of result set. Must be an integer")
            @RequestParam(value = "rows", required = false) Integer rows,
            @ApiParam(name = "page", value = "Start position of result set. Must be an integer")
            @RequestParam(value = "page", required = false) Integer page) {
        return standardVersionService.getResultlList(standardId, condition, order, rows, page);
    }


    /**
     * 根据版本id获取标准版本信息（get）
     * @param versionId
     * @return
     * @throws Exception
     */
    @RequestMapping("/getForVersionId")
    @ApiOperation(value = "获取标准版本", response = Result.class, produces = "application/json", notes = "获取标准版本")
    public Result getForVersionId(
            @ApiParam(name = "versionId", value = "标准版本ID")
            @RequestParam(value = "versionId") Long versionId) {
        return standardVersionService.get(versionId);
    }

    /**
     * 新增版本（post）
     * @param version
     * @return
     * @throws Exception
     */
    @RequestMapping("/addVersion")
    @ApiOperation(value = "保存标准版本", response = Result.class, produces = "application/json", notes = "保存标准版本")
    public Result addVersion(
            @ApiParam(name = "version", value = "标准版本信息")
            @RequestParam(value = "version") String version) throws Exception {
        return standardVersionService.add(version);
    }

    /**
     * 修改版本（put）
     * @param version
     * @return
     * @throws Exception
     */
    @RequestMapping("/updateVersion")
    @ApiOperation(value = "修改版本", response = Result.class, produces = "application/json", notes = "修改版本")
    public Result updateVersion(
            @ApiParam(name = "version", value = "标准版本")
            @RequestParam(value = "version") String version) throws Exception {
        return standardVersionService.update(version);
    }

    /**
     * 发布版本（post）
     * @param versionId
     * @param publisher
     * @return
     * @throws Exception
     */
    @RequestMapping("/publishVersion")
    @ApiOperation(value = "发布版本", response = Result.class, produces = "application/json", notes = "发布版本")
    public MSTDVersion publish(
            @ApiParam(name = "versionId", value = "标准版本ID")
            @RequestParam(value = "versionId") Long versionId,
            @ApiParam(name = "publisher", value = "发布者")
            @RequestParam(value = "publisher") String publisher) throws Exception {
        return convertToModel(standardVersionService.publish(versionId, publisher), MSTDVersion.class);
    }

    /**
     * 删除版本（put）
     * @param versionId
     * @return
     * @throws Exception
     */
    @RequestMapping("/deleteVersion")
    @ApiOperation(value = "删除版本", response = Result.class, produces = "application/json", notes = "删除版本")
    public Result delete(
            @ApiParam(name = "versionId", value = "标准版本ID")
            @RequestParam(value = "versionId") Long versionId,
            @ApiParam(name = "version", value = "标准版本值")
            @RequestParam(value = "version") String version) throws Exception {
        return standardVersionService.delete(versionId,version);
    }

    /**
     * 获取基础版本名称列表
     * @param standardId
     * @return
     * @throws Exception
     */
    @RequestMapping("/getVersions")
    @ApiOperation(value = "获取标准版本列表", response = DictionaryResult.class, produces = "application/json", notes = "获取标准版本列表")
    public DictionaryResult getVersions(
            @ApiParam(name = "standardId", value = "标准ID")
            @RequestParam(value = "standardId") Long standardId,
            @ApiParam(name = "condition", value = "Must be Json")
            @RequestParam(value = "condition", required = false) String condition,
            @ApiParam(name = "order", value = "Must be Json")
            @RequestParam(value = "order", required = false) String order,
            @ApiParam(name = "rows", value = "Limit the size of result set. Must be an integer")
            @RequestParam(value = "rows", required = false) Integer rows,
            @ApiParam(name = "page", value = "Start position of result set. Must be an integer")
            @RequestParam(value = "page", required = false) Integer page) {
        return standardVersionService.getDetailResultModelList(standardId, condition, order, rows, page);
    }
    /**
     * 根据类型文件上传保存对应的数据
     *
     * @param type
     * @param file
     * @return
     * @throws Exception
     */
    @ResponseBody
    @RequestMapping(value = "/importFile", method = RequestMethod.POST,consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    @ApiOperation(value = "根据type导入数据接口", response = Object.class, produces = "application/json", notes = "导入数据接口")
    public Result importFile(@ApiParam(value = "类型（1:数据集 2:字典）", required = true) @RequestParam("type") String type,
                             @ApiParam(value = "项目版本", required = true) @RequestParam("version") String version,
                             @ApiParam(value = "项目版本", required = true) @RequestParam("standardId") Long standardId,
                             @ApiParam(value = "文件", required = true, allowMultiple = true) @RequestPart("file")  MultipartFile file) throws Exception {

        ActionResult actionResult = new ActionResult();
        //每次调用微服务时,schema都可能会被重新赋值....
        //故这边得自己手动设置schema....
        //获取schema
        String schema = LocalContext.getContext().getAttachment(ContextAttributes.SCHEMA);
        try {
            //根據类别判断上传的文件1:数据集 2:字典
            if("1".equals(type)){
                synchronized (DataSetExcelUtil.class) {
                    Integer index = DataSetExcelUtil.size.get(version);//当前已经存入数据库的条数
                    Integer total = DataSetExcelUtil.totle.get(version);//总条数
                    //判断什么时候执行
                    if (index==null || index==-1 || index.intValue()==total.intValue()){
                        DataSetExcelUtil.size.put(version,0);//未解析之前,将其置为0,防止前台请求时,误认为该excel格式有问题
                        File excel = File.createTempFile("tmp", null);
                        file.transferTo(excel);
                        //导入的时候,先清除原有数据(可以节省做数据是否重复的判断,数据量太大,这部分会影响效率,不如直接重复导入.....)
                        standardVersionService.deleteDataset(version);
                        DataSetExcelUtil dataSetExcelUtil =new DataSetExcelUtil(stdDatasetService,stdDictService,stdMetadataService,version,excel,standardId,schema);
                        new Thread(dataSetExcelUtil).start();
                    }
                }
                //version = standardVersionService.saveDatasetExcel(file, version,standardId);
            }else {
                synchronized (DictExcelUtil.class) {
                    Integer index = DictExcelUtil.size.get(version);//当前已经存入数据库的条数
                    Integer total = DictExcelUtil.totle.get(version);//总条数
                    //判断什么时候执行
                    if (index==null || index==-1 || index.intValue()==total.intValue()){
                        DictExcelUtil.size.put(version,0);//未解析之前,将其置为0,防止前台请求时,误认为该excel格式有问题
                        File excel = File.createTempFile("tmp", null);
                        file.transferTo(excel);
                        //导入的时候,先清除原有数据(可以节省做数据是否重复的判断,数据量太大,这部分会影响效率,不如直接重复导入.....)
                        standardVersionService.deleteDict(version);
                        DictExcelUtil dictExcelUtil =new DictExcelUtil(stdDictService, stdDictEntryService,version,excel,standardId,schema);
                        new Thread(dictExcelUtil).start();
                    }
                }
            }
            //解析excel文件 保存
            actionResult.setData("{\"version\":\"" + version + "\"}");
        } catch (Exception e) {
            e.printStackTrace();
            return Result.error(e.getMessage());
        }
        return actionResult;
    }

    @ResponseBody
    @RequestMapping(value = "/isUpdateDatasetAndDict", method = RequestMethod.POST)
    @ApiOperation(value = "项目列表", response = Object.class, produces = "application/json", notes = "项目列表")
    public Result isUpdateDatasetAndDict(
            @RequestParam(required = true) String version) throws Exception {
        ActionResult actionResult = new ActionResult();
        try {
            actionResult.setData(standardVersionService.isUpdateDatasetAndDict(version));
        } catch (Exception e) {
            e.printStackTrace();
            return Result.error(e.getMessage());
        }
        return actionResult;
    }

    /**
     * 根据类型和主键删除数据
     *
     * @param type
     * @return
     * @throws Exception
     */
    @ResponseBody
    @RequestMapping(value = "/deleteByType", method = RequestMethod.POST)
    @ApiOperation(value = "文件上传成功后的删除", response = Object.class, produces = "application/json", notes = "文件上传成功后的删除")
    public Result deleteByType(
            @ApiParam(value = "类型（1:数据集 2:字典 ）", required = true) @RequestParam("type") String type,
            @ApiParam(value = "主键(上传成功后返回的ID)", required = true) @RequestParam("version") String version) throws Exception {
        ActionResult actionResult = new ActionResult();
        try {
            //根據类别判断上传的文件1:数据集 2:字典 删除数据
            switch (type) {
                case "1": {
                    standardVersionService.deleteDataset(version);
                    break;
                }
                case "2": {
                    standardVersionService.deleteDict(version);
                    break;
                }
            }
        } catch (Exception e) {
            return Result.error(e.getMessage());
        }
        return actionResult;
    }

    @RequestMapping(value = "/getSchedule", method = RequestMethod.GET)
    public String getSchedule(String type, String version){
        JSONObject json = new JSONObject();
        //1:数据集 2:字典
        switch (type) {
            case "1": {
                Integer index = DataSetExcelUtil.size.get(version);//获取当前导入的条数
                Integer total = DataSetExcelUtil.totle.get(version);//获取总条数
                json.put("index",index);
                json.put("total",total);
                break;
            }
            case "2": {
                Integer index = DictExcelUtil.size.get(version);//获取当前导入的条数
                Integer total = DictExcelUtil.totle.get(version);//获取总条数
                json.put("index",index);
                json.put("total",total);
                break;
            }
        }
        return json.toString();
    }
}

