package com.yihu.hos.standard.controller;


import com.yihu.hos.core.constants.MicroServices;
import com.yihu.hos.core.constants.ServiceApi;
import com.yihu.hos.model.standard.MStdIndex;
import com.yihu.hos.standard.model.standard.StdIndex;
import com.yihu.hos.standard.service.standard.StdIndexService;
import com.yihu.hos.web.framework.model.Envelop;
import com.yihu.hos.web.framework.model.standard.StandardVersion;
import com.yihu.hos.web.framework.util.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

@Api(value = "stdIndex", description = "标准索引相关")
@RestController("StdIndexController")
@RequestMapping(value = MicroServices.ApiVersion.Version1_0)
public class StdIndexController extends BaseController{

    @Autowired
    private StdIndexService stdIndexService;

    @ApiOperation("获取列表数据(分页)")
    @RequestMapping(value = ServiceApi.StdIndex.List, method = RequestMethod.GET)
    @ResponseBody
    public Envelop getList(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段", defaultValue = "")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件", defaultValue = "")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档", defaultValue = "")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "rows", value = "分页大小", defaultValue = "15")
            @RequestParam(value = "rows", required = false) int rows,
            @ApiParam(name = "page", value = "页码", defaultValue = "1")
            @RequestParam(value = "page", required = false) int page,
            @ApiParam(name = "version", value = "版本", defaultValue = "")
            @RequestParam(value = "version") String version) throws IOException {
        List<StdIndex> responseEntity = stdIndexService.searchPage(StdIndex.class, StandardVersion.StdIndexTable,version, filters, sorts, page, rows,true);
        List<StdIndex> all = stdIndexService.searchPage(StdIndex.class, StandardVersion.StdIndexTable,version, filters, sorts, null, null,true);
        Long count = Long.valueOf(all.size());
        List<MStdIndex> list = (List<MStdIndex>) convertToModels(responseEntity, new ArrayList<MStdIndex>(responseEntity.size()), MStdIndex.class, null);
        Envelop envelop = getResult(list,count.intValue(), page, rows);

        return envelop;
    }

    @ApiOperation("根据id获取索引")
    @RequestMapping(value = ServiceApi.StdIndex.GetById, method = RequestMethod.GET)
    @ResponseBody
    public MStdIndex getById(@ApiParam(name = "version", value = "版本", defaultValue = "")
                             @RequestParam(value = "version") String version,
                             @ApiParam(name = "id", value = "id", defaultValue = "")
                             @PathVariable(value = "id") int id) throws IOException {
        String filters = "id="+id;
        List<StdIndex> list = stdIndexService.searchPage(StdIndex.class, StandardVersion.StdIndexTable,version, filters, null, null, null,true);
        if(CollectionUtils.isEmpty(list)){
            return null;
        }
        return convertToModel(list.get(0),MStdIndex.class);
    }

    @ApiOperation("保存标准索引")
    @RequestMapping(value = ServiceApi.StdIndex.Create, method = RequestMethod.POST)
    @ResponseBody
    public Envelop save(@ApiParam(name = "indexJson", value = "标准索引json字符串")
                          @RequestParam(value = "indexJson") String indexJson){
        Envelop envelop = new Envelop();
        try {
            stdIndexService.save(indexJson);
            envelop.setSuccessFlg(true);
        } catch (Exception e) {
            e.printStackTrace();
            envelop.setSuccessFlg(false);
            envelop.setErrorMsg(e.getMessage());
        }
        return envelop;
    }

    @ApiOperation("保存标准索引")
    @RequestMapping(value = ServiceApi.StdIndex.Update, method = RequestMethod.PUT)
    @ResponseBody
    public Envelop update(@ApiParam(name = "indexJson", value = "标准索引json字符串")
                        @RequestParam(value = "indexJson") String indexJson){
        Envelop envelop = new Envelop();
        try {
            stdIndexService.update(indexJson);
            envelop.setSuccessFlg(true);
        } catch (IOException e) {
            e.printStackTrace();
            envelop.setSuccessFlg(false);
            envelop.setErrorMsg(e.getMessage());
        }
        return envelop;
    }

    @ApiOperation("根据传过来的条件判断是否是唯一的,唯一返回true")
    @RequestMapping(value = ServiceApi.StdIndex.IsUnique, method = RequestMethod.POST)
    @ResponseBody
    public boolean isUniqueName(@ApiParam(name = "filters", value = "数据集id")
                                @RequestParam(value = "filters") String filters,
                                @ApiParam(name = "version", value = "版本号")
                                @RequestParam(value = "version") String version) throws IOException {
        return stdIndexService.isUnique(filters,version);
    }


    @ApiOperation("删除索引,多个用,隔开")
    @RequestMapping(value = ServiceApi.StdIndex.DeleteById, method = RequestMethod.DELETE)
    @ResponseBody
    public boolean delete(@ApiParam(name = "ids", value = "ids")
                           @PathVariable(value = "ids") String ids,
                          @ApiParam(name = "version", value = "version")
                           @RequestParam(value = "version") String version) {
        String sql = "delete from std_index_"+version+" where id in ('"+ ids + "')";
        try {
            stdIndexService.execute(sql);
            return true;
        } catch (Exception e){
            e.getMessage();
            return false;
        }
    }

}
