package com.yihu.hos.standard.controller;


import com.yihu.hos.core.constants.MicroServices;
import com.yihu.hos.core.constants.ServiceApi;
import com.yihu.hos.model.standard.MStdDatasetRelation;
import com.yihu.hos.standard.model.standard.StdDatasetRelation;
import com.yihu.hos.standard.model.standard.StdIndex;
import com.yihu.hos.standard.service.standard.StdRelationService;
import com.yihu.hos.web.framework.model.Envelop;
import com.yihu.hos.web.framework.model.standard.StandardVersion;
import com.yihu.hos.web.framework.util.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api(value = "stdRelation", description = "标准关联关系相关")
@RestController("StdRelationController")
@RequestMapping(value = MicroServices.ApiVersion.Version1_0)
public class StdRelationController extends BaseController{

    @Autowired
    private StdRelationService stdRelationService;

    @ApiOperation("获取列表数据(分页)")
    @RequestMapping(value = ServiceApi.StdRelation.List, method = RequestMethod.GET)
    @ResponseBody
    public Envelop getList(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段", defaultValue = "")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件", defaultValue = "")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档", defaultValue = "")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "rows", value = "分页大小", defaultValue = "15")
            @RequestParam(value = "rows", required = false) int rows,
            @ApiParam(name = "page", value = "页码", defaultValue = "1")
            @RequestParam(value = "page", required = false) int page,
            @ApiParam(name = "version", value = "版本", defaultValue = "")
            @RequestParam(value = "version") String version) throws IOException {
        List<StdDatasetRelation> responseEntity = stdRelationService.searchPage(StdIndex.class, StandardVersion.StdIndexTable,version, filters, sorts, page, rows,true);
        List<StdDatasetRelation> all = stdRelationService.searchPage(StdIndex.class, StandardVersion.StdIndexTable,version, filters, sorts, null, null,true);
        Long count = Long.valueOf(all.size());
        List<MStdDatasetRelation> list = (List<MStdDatasetRelation>) convertToModels(responseEntity, new ArrayList<MStdDatasetRelation>(responseEntity.size()), MStdDatasetRelation.class, null);
        Envelop envelop = getResult(list,count.intValue(), page, rows);

        return envelop;
    }

    @ApiOperation("获取列表数据(不分页)")
    @RequestMapping(value = ServiceApi.StdRelation.ListNoPage, method = RequestMethod.GET)
    @ResponseBody
    public Envelop getListNoPage(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段", defaultValue = "")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件", defaultValue = "")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档", defaultValue = "")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "version", value = "版本", defaultValue = "")
            @RequestParam(value = "version") String version) throws IOException {
        List<StdDatasetRelation> responseEntity = stdRelationService.searchPage(StdDatasetRelation.class, StandardVersion.StdRelationTable,version, filters, sorts, null, null,true);
        List<MStdDatasetRelation> list = (List<MStdDatasetRelation>) convertToModels(responseEntity, new ArrayList<MStdDatasetRelation>(responseEntity.size()), MStdDatasetRelation.class, null);
        Envelop envelop = new Envelop();
        envelop.setSuccessFlg(true);
        envelop.setDetailModelList(list);
        return envelop;
    }

    @ApiOperation("保存/修改标准关联关系")
    @RequestMapping(value = ServiceApi.StdRelation.Modify, method = RequestMethod.POST)
    @ResponseBody
    public Envelop modify(@ApiParam(name = "list", value = "json字符串")
                          @RequestParam(value = "list") String list,
                          @ApiParam(name = "version", value = "版本号")
                          @RequestParam(value = "version") String version){
        Envelop envelop = new Envelop();
        try {
            List<StdDatasetRelation> stdDatasetRelations = Arrays.asList(objectMapper.readValue(list, StdDatasetRelation[].class));
            stdRelationService.modifyRelation(version,stdDatasetRelations);
            envelop.setSuccessFlg(true);
        } catch (Exception e) {
            e.printStackTrace();
            envelop.setSuccessFlg(false);
            envelop.setErrorMsg(e.getMessage());
        }
        return envelop;
    }

}
