package com.yihu.hos.standard.service.adapter;


import com.fasterxml.jackson.databind.JsonNode;
import com.yihu.hos.core.datatype.CollectionUtil;
import com.yihu.hos.core.datatype.StringUtil;
import com.yihu.hos.standard.dao.IStdDataSetModelDao;
import com.yihu.hos.standard.dao.IStdMetaDataModelDao;
import com.yihu.hos.standard.match.matchModel.MetadataMatchVO;
import com.yihu.hos.standard.match.metadata.MetadataStrategyExecute;
import com.yihu.hos.standard.match.metadata.MetatdataStandardNameStrategy;
import com.yihu.hos.standard.model.adapter.AdapterDatasetModel;
import com.yihu.hos.standard.model.adapter.AdapterMetadataModel;
import com.yihu.hos.standard.model.adapter.AdapterSchemeVersionModel;
import com.yihu.hos.standard.model.standard.StdDataSetModel;
import com.yihu.hos.standard.model.standard.StdMetaDataModel;
import com.yihu.hos.standard.service.standard.StdMetadataService;
import com.yihu.hos.standard.util.GetChineseFirst;
import com.yihu.hos.standard.util.LikeHashMap;
import com.yihu.hos.web.framework.constant.ContextAttributes;
import com.yihu.hos.web.framework.constant.ErrorCode;
import com.yihu.hos.web.framework.constant.SqlConstants;
import com.yihu.hos.web.framework.dao.SQLGeneralDAO;
import com.yihu.hos.web.framework.exception.ApiException;
import com.yihu.hos.web.framework.model.DetailModelResult;
import com.yihu.hos.web.framework.model.Envelop;
import com.yihu.hos.web.framework.model.Result;
import com.yihu.hos.web.framework.model.TreeView;
import com.yihu.hos.web.framework.model.standard.AdapterVersion;
import com.yihu.hos.web.framework.model.standard.StandardVersion;
import com.yihu.hos.web.framework.thread.LocalContext;
import com.yihu.hos.web.framework.util.springutil.SpringBeanUtil;
import com.yihu.hos.web.framework.util.sql.BeanTransformer;
import com.yihu.hos.web.framework.util.sql.SqlCreator;
import org.apache.commons.collections.CollectionUtils;
import org.hibernate.Query;
import org.hibernate.Session;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.io.IOException;
import java.util.*;

@Transactional
@Service("AdapterDatasetService")
public class AdapterDatasetService extends SQLGeneralDAO {

    public static final String BEAN_ID = "AdapterDatasetService";

    @Autowired
    private AdapterMetadataService adapterMetadataService;

    @Autowired
    private AdapterSchemeVersionService adapterSchemeVersionService;

    @Autowired
    private ThreadPoolTaskExecutor threadPoolTaskExecutor;

    @Autowired
    private IStdMetaDataModelDao stdMetaDataModelDao;

    @Autowired
    private IStdDataSetModelDao stdDataSetModelDao;

    @Autowired
    private StdMetadataService stdMetadataService;

    private AdapterDatasetService adapterDatasetService;

    public AdapterDatasetService() {
    }

    public void finalize() throws Throwable {
        super.finalize();
    }

    public Result add(String adapterVersion, String dataset) {
        try {
            AdapterVersion version = new AdapterVersion(adapterVersion);
            AdapterDatasetModel adapterDatasetModel = objectMapper.readValue(dataset, AdapterDatasetModel.class);
            int maxId = getMaxId(version.getDataSetTableName());
            adapterDatasetModel.setId( Long.valueOf(maxId));
            JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(adapterDatasetModel));
            SqlCreator sqlCreator = new SqlCreator(AdapterDatasetModel.class);
            String sql = sqlCreator.insertData(version.getDataSetTableName(), jsonNode);
            Query query = getExeuteQuery(sqlCreator, sql);
            query.executeUpdate();
            adapterDatasetModel.setSuccessFlg(true);
            adapterDatasetModel.setMessage("保存适配数据集成功");
            return adapterDatasetModel;
        } catch (Exception e) {
            return Result.error("保存适配数据集失败");
        }
    }

    public Result add(String adapterVersion, AdapterDatasetModel adapterDatasetModel) {
        try {
            AdapterVersion version = new AdapterVersion(adapterVersion);
            int maxId = getMaxId(version.getDataSetTableName());
            adapterDatasetModel.setId( Long.valueOf(maxId));
            JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(adapterDatasetModel));
            SqlCreator sqlCreator = new SqlCreator(AdapterDatasetModel.class);
            String sql = sqlCreator.insertData(version.getDataSetTableName(), jsonNode);
            Query query = getExeuteQuery(sqlCreator, sql);
            query.executeUpdate();
            adapterDatasetModel.setSuccessFlg(true);
            adapterDatasetModel.setMessage("保存适配数据集成功");
            return adapterDatasetModel;
        } catch (Exception e) {
            return Result.error("保存适配数据集失败");
        }
    }

    public Result delete(String adapterVersion, Integer datasetId) {
        try {
            if (adapterVersion == null || adapterVersion.length() == 0) {
                return Result.success("无效适配版本");
            }
            AdapterVersion version = new AdapterVersion(adapterVersion);
            Session session = getCurrentSession();
            String sql = "delete from " + version.getDataSetTableName() + " where id = :id";
            Query query = session.createSQLQuery(sql);
            query.setInteger("id", datasetId);
            query.executeUpdate();
            return Result.success("删除适配数据集成功");
        } catch (Exception e) {
            return Result.error("删除适配数据集失败");
        }
    }

    public Result modify(String adapterVersion, String dataset) {
        try {
            AdapterVersion version = new AdapterVersion(adapterVersion);
            AdapterDatasetModel adapterDatasetModel = objectMapper.readValue(dataset, AdapterDatasetModel.class);
            JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(adapterDatasetModel));
            SqlCreator sqlCreator = new SqlCreator(AdapterDatasetModel.class);
            String sql = sqlCreator.updateDataByTableKey(version.getDataSetTableName(), jsonNode);
            Query query = getExeuteQuery(sqlCreator, sql);
            query.executeUpdate();
            adapterDatasetModel.setSuccessFlg(true);
            adapterDatasetModel.setMessage("修改适配数据集成功");
            return adapterDatasetModel;
        } catch (Exception e) {
            return Result.error("修改适配数据集失败");
        }
    }

    public Result get(String version, Integer datasetId) {
        try {
            AdapterVersion adapterVersion = new AdapterVersion(version);
            AdapterDatasetModel adapterDatasetModel = (AdapterDatasetModel) get(AdapterDatasetModel.class, adapterVersion.getDataSetTableName(), datasetId);
            adapterDatasetModel.setSuccessFlg(true);
            adapterDatasetModel.setMessage("获取适配数据集成功");
            return adapterDatasetModel;
        } catch (Exception e) {
            return Result.error("获取适配数据集失败");
        }
    }

    public DetailModelResult getDetailModelResult(String version, String condition, String order, Integer rows, Integer page) {
        try {
            AdapterVersion adapterVersion = new AdapterVersion(version);
            List<AdapterDatasetModel> dataSetModelList = getDatasetList(AdapterDatasetModel.class, version, condition, order, rows, page);
            Integer count = getDatasetInt(AdapterDatasetModel.class, adapterVersion, condition);
            DetailModelResult detailModelResult = DetailModelResult.success("获取适配数据集成功");
            detailModelResult.setDetailModelList(dataSetModelList);
            detailModelResult.setTotalCount(count);
            return detailModelResult;
        } catch (Exception e) {
            return DetailModelResult.error("获取适配数据集失败");
        }
    }

    public List getDatasetList(Class tClass, String version, String condition, String order, Integer limit, Integer offset) throws Exception {
        AdapterVersion adapterVersion = new AdapterVersion(version);
        String tableName = adapterVersion.getDataSetTableName();
        SqlCreator sqlCreator = new SqlCreator(tClass);
        if (!StringUtil.isEmpty(condition)) {
            JsonNode jsonNode = objectMapper.readTree(condition);
            String name = jsonNode.get("name").asText();
            sqlCreator.likeOrCondition("stdDatasetName", "stdDatasetCode", name);
            if(jsonNode.has("needCrawer")){
                sqlCreator.equalCondition("needCrawer",jsonNode.get("needCrawer").asInt());
            }
        }
        if (!StringUtil.isEmpty(order)) {
            JsonNode jsonNode = objectMapper.readTree(order);
            Iterator<String> fieldNames = jsonNode.fieldNames();
            while (fieldNames.hasNext()) {
                String fieldName = fieldNames.next();
                String value = jsonNode.get(fieldName).asText().toUpperCase();
                sqlCreator.order(fieldName, value);
            }
        }
        String sql = sqlCreator.selectData(tableName);
        Query query = getQuery(sqlCreator, sql);
        if (limit != null) {
            query.setMaxResults(limit);
            if (offset != null) {
                query.setFirstResult((offset - 1) * limit);
            }
        }
        return query.list();
    }

    public Integer getDatasetInt(Class tClass, AdapterVersion adapterVersion, String condition) {
        try {
            String tableName = adapterVersion.getDataSetTableName();
            SqlCreator sqlCreator = new SqlCreator(tClass);
            if (!StringUtil.isEmpty(condition)) {
                JsonNode jsonNode = objectMapper.readTree(condition);
                String name = jsonNode.get("name").asText();
                sqlCreator.likeOrCondition("stdDatasetName", "stdDatasetCode", name);
                if(jsonNode.has("needCrawer")){
                    sqlCreator.equalCondition("needCrawer",jsonNode.get("needCrawer").asInt());
                }
            }
            String sql = sqlCreator.countData(tableName);
            Query query = getCurrentSession().createSQLQuery(sql);
            for (String key : sqlCreator.getKeyValueMap().keySet()) {
                query.setParameter(key, sqlCreator.getKeyValueMap().get(key));
            }
            Integer count = Integer.parseInt(StringUtil.toString(query.list().get(0)));
            return count;
        } catch (Exception e) {
            e.printStackTrace();
            throw new ApiException(ErrorCode.GetDataSetListFailed);
        }
    }

    public List<TreeView> getAdapterCustomize(String adapterVersion) throws IOException {

        List<AdapterDatasetModel> adapterDataSetList = searchPage(AdapterDatasetModel.class, AdapterVersion.DataSetTablePrefix,adapterVersion, "", null, null, null,false);
        List<TreeView> adapterCustomizeList = new ArrayList<>();
        int i = 0;//用于判断左侧根节点是否需要全选

        for (AdapterDatasetModel adapterDataset : adapterDataSetList) {
            TreeView parent = new TreeView();
            parent.setId("SD" + adapterDataset.getId());
            parent.setPid("S0");
            parent.setText(adapterDataset.getStdDatasetName());
            if(adapterDataset.getNeedCrawer()!=null && adapterDataset.getNeedCrawer()==1){
                i++;
                parent.setIschecked(true);
                //添加到右侧树形列表
                TreeView rightParent = new TreeView();
                rightParent.setId("AD" + adapterDataset.getId());
                rightParent.setPid("A0");
                rightParent.setText(adapterDataset.getStdDatasetName());
                rightParent.setIschecked(true);
                adapterCustomizeList.add(rightParent);
            }else{
                parent.setIschecked(false);
            }
            adapterCustomizeList.add(parent);
            Long belongDatasetId = adapterDataset.getId();
            List<AdapterMetadataModel> adapterMetaDataList = adapterMetadataService.searchPage(AdapterMetadataModel.class, AdapterVersion.MetaDataTablePrefix,adapterVersion, "belongAdapterId="+belongDatasetId, null, null, null,false) ;
            //数据元
            for (AdapterMetadataModel adapterMetadat : adapterMetaDataList) {
                TreeView child = new TreeView();
                child.setId("SM" + adapterMetadat.getId());
                child.setPid("SD" + belongDatasetId);
                child.setText(adapterMetadat.getStdMetadataName());
                if(adapterMetadat.getNeedCrawer()!= null && adapterMetadat.getNeedCrawer()==1){
                    child.setIschecked(true);

                    //添加到右侧树形列表
                    TreeView rightChild = new TreeView();
                    rightChild.setId("AM" + adapterMetadat.getId());
                    rightChild.setPid("AD" + adapterDataset.getId());
                    rightChild.setText(adapterMetadat.getStdMetadataName());
                    rightChild.setIschecked(true);
                    adapterCustomizeList.add(rightChild);
                } else {
                    child.setIschecked(false);
                }
                adapterCustomizeList.add(child);
            }
        }

        //添加左侧根节点
        TreeView leftParent = new TreeView();
        leftParent.setId("S0");
        leftParent.setPid("-1");
        leftParent.setText("数据集");
        if(i >= 1){
            //首先添加右侧根节点
            TreeView adapterRoot = new TreeView();
            adapterRoot.setId("A0");
            adapterRoot.setPid("-1");
            adapterRoot.setText("数据集");
            adapterRoot.setIschecked(true);
            adapterCustomizeList.add(adapterRoot);
        }
        if (i == adapterDataSetList.size()) {
            leftParent.setIschecked(true);
        } else {
            leftParent.setIschecked(false);
        }

        adapterCustomizeList.add(leftParent);
        
        return adapterCustomizeList;
    }


    public DetailModelResult getDatasetCustomize(String adapterVersion, String stdVersion) {

        try {
            List<TreeView> adapterCustomize = getAdapterCustomize(adapterVersion);
            DetailModelResult detailModelResult = DetailModelResult.success("获取定制数据集列表成功");
            detailModelResult.setDetailModelList(adapterCustomize);
            return detailModelResult;
        } catch (Exception e) {
            e.printStackTrace();
            return DetailModelResult.error("获取定制数据集列表失败");
        }
    }

    public Result updateCustomize(Integer schemeId, String adapterVersion, String stdVersion, String datasetIdStr, String metaDataIdStr) {
        AdapterVersion aVersion = new AdapterVersion(adapterVersion);
        StandardVersion sVersion = new StandardVersion(stdVersion);
        try {
            String[] datasetIdList = datasetIdStr.split(",");
            String[] metadataIdList = metaDataIdStr.split(",");
            List<Long> newDatasetIdList = new ArrayList<>();
            List<Long> newMetadataIdList = new ArrayList<>();

            for (String datasetId : datasetIdList) {
                if (!StringUtil.isStrEmpty(datasetId)) {
                    Long newDatasetId = Long.parseLong(datasetId);
                    newDatasetIdList.add(newDatasetId);
                }
            }

            for (String metadataId : metadataIdList) {
                if (!StringUtil.isStrEmpty(metadataId)) {
                    Long newMetadataId = Long.parseLong(metadataId);
                    newMetadataIdList.add(newMetadataId);
                }
            }

            if (newDatasetIdList.size() == 0) {
                return Result.error("定制内容为空");
            }

            updateDatasetCustomizeData(schemeId, aVersion, sVersion, newDatasetIdList, newMetadataIdList);
             return Result.success("定制数据集成功");
        } catch (Exception e) {
            e.printStackTrace();
            return Result.error("定制数据集失败");
        }
    }

    public void updateDatasetCustomizeData(Integer schemeId, AdapterVersion aVersion, StandardVersion sVersion, List<Long> newDatasetIdList, List<Long> newMetadataIdList)  {
        //首先把适配数据集和适配的数据元,字典 全部设置成不需要采集
        String dataSetTableName = aVersion.getDataSetTableName();
        String metaDataTableName = aVersion.getMetaDataTableName();
        String dictName = aVersion.getDictTableName();
        jdbcTemplate.execute("update "+dataSetTableName +" set need_crawer=0");
        jdbcTemplate.execute("update "+metaDataTableName +" set need_crawer=0");
        jdbcTemplate.execute("update "+dictName +" set need_crawer=0");

        //设置需要采集的数据集
        if(CollectionUtils.isNotEmpty(newDatasetIdList)){
            String ids="";
            for(Long id:newDatasetIdList){
                ids+=","+id;
            }
            jdbcTemplate.execute("update "+dataSetTableName +" set need_crawer=1 where id in ("+ids.substring(1,ids.length())+")");
        }

        String metadaIds = "";
        //设置需要采集的数据元
        if(CollectionUtils.isNotEmpty(newMetadataIdList)){
            for(Long id:newMetadataIdList){
                metadaIds+=","+id;
            }
            metadaIds = metadaIds.substring(1, metadaIds.length());
            jdbcTemplate.execute("update "+metaDataTableName +" set need_crawer=1 where id in ("+metadaIds+")");
        }

        if(!StringUtils.isEmpty(metadaIds)){
            //设置需要采集的字典
            List<Long> dictIds = jdbcTemplate.queryForList("select DISTINCT  std_dict_id from " + metaDataTableName + " where std_dict_id is not null and std_dict_id!='' and id in (" + metadaIds + ")",Long.class);
            if(CollectionUtils.isNotEmpty(dictIds)){
                String ids = "";
                for(Long id:dictIds){
                    ids+=","+id;
                }
                jdbcTemplate.execute("update "+dictName +" set need_crawer=1 where id in ("+ids.substring(1,ids.length())+")");
            }
        }
    }

    public List getAdapterDatasetNotNullList(AdapterVersion adapterVersion, String condition) {
        try {
            StringBuffer sql = new StringBuffer();
            sql.append("SELECT * FROM ").append(adapterVersion.getDataSetTableName());
            if (!StringUtil.isEmpty(condition)) {
                JsonNode jsonNode = objectMapper.readTree(condition);
                String column = jsonNode.get("column").asText();
                sql.append(" WHERE ").append(column).append(" IS NOT NULL");
            }
            Query query = getCurrentSession().createSQLQuery(sql.toString());
            query.setResultTransformer(new BeanTransformer(AdapterDatasetModel.class));
            return query.list();
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public List getListByAdapterDatasetIdList(AdapterVersion adapterVersion, List<Integer> datasetIdList) {
        try {
            if (CollectionUtil.isEmpty(datasetIdList)) {
                return null;
            }
            SqlCreator sqlCreator = new SqlCreator(AdapterDatasetModel.class);
            sqlCreator.inCondition("stdDatasetId", datasetIdList);
            String sql = sqlCreator.selectData(adapterVersion.getDataSetTableName());
            Query query = getQuery(sqlCreator, sql);
            return query.list();
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public List<AdapterDatasetModel> getAdapterDatasetByCode(String version, String datasetCode) {
        if (StringUtil.isEmpty(datasetCode)) {
            return null;
        }
        SqlCreator sqlCreator = new SqlCreator(AdapterDatasetModel.class);
        sqlCreator.equalCondition("stdDatasetCode", datasetCode);
        String sql = sqlCreator.selectData("adapter_dataset_" + version);
        Query query = getQuery(sqlCreator, sql);
        return query.list();
    }

    public AdapterDatasetModel getAdapterDatasetByStdId(String version, Long datasetId) {
        try {
            if (StringUtil.isEmpty(datasetId)) {
                return null;
            }
            SqlCreator sqlCreator = new SqlCreator(AdapterDatasetModel.class);
            sqlCreator.equalCondition("stdDatasetId", datasetId);
            String sql = sqlCreator.selectData("adapter_dataset_" + version);
            Query query = getQuery(sqlCreator, sql);
            return (AdapterDatasetModel) query.uniqueResult();
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }


    public AdapterDatasetModel getAdapterDatasetByCode(String schemaName, String version, String datasetCode) {
        try {
            if (StringUtil.isEmpty(datasetCode)) {
                return null;
            }
            SqlCreator sqlCreator = new SqlCreator(AdapterDatasetModel.class);
            sqlCreator.equalCondition("stdDatasetCode", datasetCode);
            String sql = sqlCreator.selectData("adapter_dataset_" + version);

            String myCatAnnotation = "/*#mycat:schema=" + schemaName + "*/ ";
            String completeSql = myCatAnnotation + sql;

            Query query = getQuery(sqlCreator, completeSql);
            return (AdapterDatasetModel) query.uniqueResult();
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public void strategy(String schemaName,String version,String std_version,String adapter_std_version) throws Exception {
        List<AdapterMetadataModel> unAdapterMetadataModels =  adapterMetadataService.getAllUnAdaptMetadata(schemaName,version);//等待适配的标准数据元
        List<AdapterMetadataModel> adapterMetadataModel = new ArrayList<AdapterMetadataModel>();//之前已经适配好的适配方案
        //获取所有未匹配的
        //查找出  之前已经适配好的适配方案
        List<AdapterSchemeVersionModel> all = adapterSchemeVersionService.getAll(schemaName);
        if(all!=null){
            for(AdapterSchemeVersionModel adapterSchemeVersionModel:all){
                String vers = adapterSchemeVersionModel.getVersion();
                if(org.apache.commons.lang3.StringUtils.isNotBlank(vers)){
                    List<AdapterMetadataModel> adaptMetadata = adapterMetadataService.getAllAdaptMetadata(schemaName,adapterSchemeVersionModel.getVersion());
                    adapterMetadataModel.addAll(adaptMetadata);
                }
            }
        }
        MetadataMatchVO matchVO = new MetadataMatchVO();
        matchVO.setCodeAdapter(matchVO.geNameGroup(adapterMetadataModel));

        //設置匹配的机构数据緩存
        List<StdMetaDataModel> orgMetadatas = stdMetadataService.getList(schemaName,adapter_std_version, null, null, null, null, null);//查找出机构所有的metadatas
        for (StdMetaDataModel stdMetadata:orgMetadatas){
            StdDataSetModel stdDataset = stdDataSetModelDao.getDatasetName(schemaName,adapter_std_version, stdMetadata.getDataSetId());
            if(null != stdDataset){
                MetatdataStandardNameStrategy.getNameMap().put(stdMetadata.getName() + "_" + stdDataset.getName(),stdMetadata);
                //MetatdataStandardNameStrategy.getRemarkMap().put(stdMetadata.getDefinition() + "_" + stdDataset.getName(),stdMetadata);  //根据数据元定义去匹配,匹配度低,浪费性能, 注释掉
            }
        }

        //执行匹配
        MetadataStrategyExecute.setFirst(version, unAdapterMetadataModels.size());//设置第一次匹配
        Set<String> sqlList = new HashSet<String>();

        for (AdapterMetadataModel unAdapterMetadataModel : unAdapterMetadataModels) {
            threadPoolTaskExecutor.execute(new MetadataStrategyExecute(schemaName,unAdapterMetadataModel, matchVO, std_version,adapter_std_version,version, stdMetaDataModelDao,stdDataSetModelDao,adapterMetadataService));

            String metadataNameFirstCode = GetChineseFirst.cn2py(unAdapterMetadataModel.getStdMetadataName());//待匹配
            List<AdapterMetadataModel> adapterMetadatas = matchVO.getCodeAdapter().get(metadataNameFirstCode);//已存在
            if (adapterMetadatas != null && adapterMetadatas.size() > 0) {
                //获取待适配的数据集名
                Long unAdaptStdId = unAdapterMetadataModel.getStdDatasetId();
                StdDataSetModel adapterDataset = stdDataSetModelDao.getDatasetName(schemaName,adapter_std_version, unAdaptStdId);
                if (adapterDataset != null) {
                    adapterDatasetService = SpringBeanUtil.getService(AdapterDatasetService.BEAN_ID);
                    AdapterDatasetModel entity = adapterDatasetService.getAdapterDatasetByCode(schemaName,version, adapterDataset.getCode());
                    String sqrSql = null;
                    if(null != entity){
                        sqrSql = adapterDatasetService.saveAdaptDatasetSql(schemaName,entity, adapterDataset, version);
                    }
                    if(null != sqrSql && !"".equals(sqrSql)){
                        sqlList.add(sqrSql);
                    }
                }
            }
        }

        if(CollectionUtils.isNotEmpty(sqlList)){
            List list=new ArrayList();
            Iterator it=sqlList.iterator();
            while(it.hasNext()){
                list.add((String) it.next());
            }
            super.insertBatch(list);
        }
    }



    public String saveAdaptDatasetSql(AdapterDatasetModel strategyDataset, StdDataSetModel orgDataset, String version) throws Exception {
        StringBuilder sqlBuffer = new StringBuilder();
        sqlBuffer.append(SqlConstants.UPDATE + new AdapterVersion(version).getDataSetTableName() + SqlConstants.SET);
        sqlBuffer.append(" adapter_dataset_id = " + orgDataset.getId());
        sqlBuffer.append(",adapter_dataset_code = '" + orgDataset.getCode() + "'");
        sqlBuffer.append(",adapter_dataset_name = '" + orgDataset.getName() + "'");
        sqlBuffer.append(" where id=" + strategyDataset.getId());
        return sqlBuffer.toString();
    }

    @Transactional
    public String saveAdaptDatasetSql(String schemaName, AdapterDatasetModel strategyDataset, StdDataSetModel orgDataset, String version) throws Exception {
        StringBuilder sqlBuffer = new StringBuilder();
        sqlBuffer.append(SqlConstants.UPDATE + new AdapterVersion(version).getDataSetTableName() + SqlConstants.SET);
        sqlBuffer.append(" adapter_dataset_id = " + orgDataset.getId());
        sqlBuffer.append(",adapter_dataset_code = '" + orgDataset.getCode() + "'");
        sqlBuffer.append(",adapter_dataset_name = '" + orgDataset.getName() + "'");
        sqlBuffer.append(" where id=" + strategyDataset.getId());

        String myCatAnnotation = "/*#mycat:schema=" + schemaName + "*/ ";
        String completeSql = myCatAnnotation + sqlBuffer;

        return completeSql;
    }

    public Envelop cloneDataSet(String adapterVersion,String code) throws IOException {
        Envelop envelop = new Envelop();
        AdapterDatasetModel adapterDataset = null;
        List<AdapterDatasetModel> adapterDatasets = getAdapterDatasetByCode(adapterVersion, code);
        if (adapterDatasets != null && adapterDatasets.size()>0) {
            adapterDataset = adapterDatasets.get(0);
        }else{
            envelop.setSuccessFlg(false);
            envelop.setErrorMsg("数据集不存在");
            return envelop;
        }
        if(adapterDataset==null){
            envelop.setSuccessFlg(false);
            envelop.setErrorMsg("数据集不存在");
            return envelop;
        }
        AdapterDatasetModel clone_adapter_dataset = new AdapterDatasetModel();
        clone_adapter_dataset.setSchemeId(adapterDataset.getSchemeId());
        clone_adapter_dataset.setStdDatasetId(adapterDataset.getStdDatasetId());
        clone_adapter_dataset.setStdDatasetCode(adapterDataset.getStdDatasetCode());
        clone_adapter_dataset.setStdDatasetName(adapterDataset.getStdDatasetName());
        clone_adapter_dataset.setIsClone(1);
        clone_adapter_dataset.setNeedCrawer(1);
        Result add = add(adapterVersion, clone_adapter_dataset);
        if(add==null || !add.isSuccessFlg()){
            envelop.setSuccessFlg(false);
            envelop.setErrorMsg("数据集克隆失败");
            return envelop;
        }
        List<AdapterMetadataModel> adapterMetadatas = adapterMetadataService.searchPage(AdapterMetadataModel.class,"adapter_metadata",adapterVersion,"belongAdapterId="+adapterDataset.getStdDatasetId(),null,1,9999,false);
        if(adapterMetadatas==null){
            envelop.setSuccessFlg(false);
            envelop.setErrorMsg("数据元不存在");
            return envelop;
        }
        AdapterDatasetModel clone_dataset = (AdapterDatasetModel)add;
        List<String> sqls = new ArrayList<>();
        AdapterVersion version = new AdapterVersion(adapterVersion);
        String dictIds = "";
        for(AdapterMetadataModel medata:adapterMetadatas){
            AdapterMetadataModel adapterMetadataModel = new AdapterMetadataModel();
            adapterMetadataModel.setSchemeId(medata.getSchemeId());
            adapterMetadataModel.setStdDatasetId(medata.getStdDatasetId());
            adapterMetadataModel.setStdMetadataId(medata.getStdMetadataId());
            adapterMetadataModel.setStdMetadataCode(medata.getStdMetadataCode());
            adapterMetadataModel.setStdMetadataName(medata.getStdMetadataName());
            Long stdDictId = medata.getStdDictId();
            adapterMetadataModel.setStdDictId(stdDictId);
            if(stdDictId != null && stdDictId!=0){
                dictIds+=","+stdDictId;
            }
            adapterMetadataModel.setBelongAdapterId(clone_dataset.getId());
            adapterMetadataModel.setNeedCrawer(1);
            String sql = getInsertSql(version, adapterMetadataModel);
            sqls.add(sql);
        }
        if (sqls.size() > 0) {
            adapterMetadataService.insertBatch(sqls);
        }
        if(!StringUtils.isEmpty(dictIds)){
            jdbcTemplate.execute("update "+new AdapterVersion(adapterVersion).getDictTableName() +" set need_crawer=1 where id in ("+dictIds.substring(1,dictIds.length())+")");
        }
        envelop.setSuccessFlg(true);
        return envelop;
    }

    public String getInsertSql(AdapterVersion version, AdapterMetadataModel medata) throws IOException {
        SqlCreator sqlCreator = new SqlCreator(AdapterMetadataModel.class);
        JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(medata));
        String sql = sqlCreator.insertDataToSQL(version.getMetaDataTableName(), jsonNode);
        return sql;
    }

    public Envelop isNeedRelation(String adapterVersion, Long id) {
        Envelop envelop = new Envelop();
        AdapterVersion version = new AdapterVersion(adapterVersion);
        String sql = "select DISTINCT adapter_dataset_id from "+ version.getMetaDataTableName() +" where adapter_metadata_code is not null and adapter_dataset_id is not null and belong_adapter_id = ?";
        List<Long> adapterIds = jdbcTemplate.queryForList(sql,Long.class, id);
        if(adapterIds!=null&&adapterIds.size()>=2){
            envelop.setSuccessFlg(true);
        }else{
            envelop.setSuccessFlg(false);
        }
        return envelop;

    }

    public Envelop smartMatchMedata(String std_version, String adapter_version, Long id, Long adapterDataSetId) throws Exception {
        Envelop envelop = new Envelop();
        //获取等待适配的数据元
        List<AdapterMetadataModel> unAdapterMedataModels= adapterMetadataService.getUnAdaptMetadataByid(adapter_version,id);


        //获取源数据元
        Map<String, String> order = new HashMap<>();
        Map<String, Object> query = new HashMap<>();
        query.put("dataSetId",adapterDataSetId);
        StandardVersion standardVersion = new StandardVersion(std_version);
        List<StdMetaDataModel> medataList = stdMetadataService.getList(standardVersion, null, query, order, null, null);

        if(medataList!=null && medataList.size()>0){
            LikeHashMap<String, StdMetaDataModel> nameMap = new LikeHashMap<String, StdMetaDataModel>();
            for(StdMetaDataModel medata:medataList){
                String code = medata.getCode();
                if(!StringUtil.isEmpty(code)){
                    code = code.trim();
                }
                String name = medata.getName();
                if(!StringUtil.isEmpty(name)){
                    name = name.trim();
                }
                nameMap.put(code+"@"+name,medata);
            }
            if(unAdapterMedataModels!=null && unAdapterMedataModels.size()>0){
                String schemaName = LocalContext.getContext().getAttachment(ContextAttributes.SCHEMA);
                //查询到的匹配字典项
                StdMetaDataModel stdMetadaModel = null;
                for(AdapterMetadataModel model:unAdapterMedataModels){
                    //查询到的匹配字典项
                    StdMetaDataModel stdMetaDataModel = null;

                    String code = model.getStdMetadataCode();
                    if(!StringUtil.isEmpty(code)){
                        code = code.trim();
                    }

                    String name = model.getStdMetadataName();
                    if(!StringUtil.isEmpty(name)){
                        name = name.trim();
                    }

                    //通过name做适配
                    stdMetadaModel = nameMap.get("@" + name, true);
                    if (stdMetadaModel != null) {
                        adapterMetadataService.saveAdapt(schemaName,model, stdMetadaModel, adapter_version,"1");//只匹配字典项code ,为疑似匹配
                    }else{ //通过code做匹配
                        stdMetadaModel = nameMap.get(code+"@", true);
                        if(stdMetadaModel!=null){
                            adapterMetadataService.saveAdapt(schemaName,model, stdMetadaModel, adapter_version,"1");//只匹配字典项code ,为疑似匹配
                        }
                    }
                }
            }
        }
        AdapterVersion adapterVersion = new AdapterVersion(adapter_version);
        adapterMetadataService.changeDatasetRelation(adapterVersion,id,std_version);
        envelop.setSuccessFlg(true);
        return envelop;

    }

    public Envelop getDataSets(String adapterVersion, Long id,String sourceVersion,Integer page,Integer rows ,String filterCodeOrName)throws IOException {
        Envelop envelop = new Envelop();
        AdapterVersion version = new AdapterVersion(adapterVersion);
        String sql = "select DISTINCT adapter_dataset_id from "+ version.getMetaDataTableName() +" where adapter_metadata_code is not null and adapter_dataset_id is not null and belong_adapter_id = ?";
        List<Long> adapterIds = jdbcTemplate.queryForList(sql,Long.class, id);
        String ids="";
        if(!CollectionUtil.isEmpty(adapterIds)){
            for(Long adapterId:adapterIds){
                ids+=","+adapterId;
            }
        }
        if(!StringUtil.isEmpty(ids)){
            ids=ids.substring(1,ids.length());
        }else{
            envelop.setSuccessFlg(true);
            envelop.setDetailModelList(null);
            return envelop;
        }
        String condition = "id=" + ids+";";
        if(!StringUtils.isEmpty(filterCodeOrName)){
            condition+="code?"+filterCodeOrName+" g1;name?"+filterCodeOrName+" g1";
        }
        List std_datasets = searchPage(StdDataSetModel.class, "std_dataset", sourceVersion, condition, null, page, rows,true);
        envelop.setDetailModelList(std_datasets);
        envelop.setSuccessFlg(true);
        return envelop;
    }

    public AdapterDatasetModel getById(String version, Long id) {
        try {
            if (id==null) {
                return null;
            }
            SqlCreator sqlCreator = new SqlCreator(AdapterDatasetModel.class);
            sqlCreator.equalCondition("id", id);
            String sql = sqlCreator.selectData("adapter_dataset_" + version);
            Query query = getQuery(sqlCreator, sql);
            return (AdapterDatasetModel) query.uniqueResult();
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public Envelop deleteCloneDataSet(String adapterVersion, Long id) {
        Envelop envelop = new Envelop();
        AdapterVersion version = new AdapterVersion(adapterVersion);
        String dataSetTableName = version.getDataSetTableName();
        String metaDataTableName = version.getMetaDataTableName();
        String dictTableName = version.getDictTableName();
        //删除适配数据集
        jdbcTemplate.execute("delete from "+dataSetTableName +" where id="+id);

        //判断适配字典是否需要转成不需要采集状态
        List<Long> needCrawerIds = jdbcTemplate.queryForList("select DISTINCT  std_dict_id from " + metaDataTableName + " where std_dict_id is not null and std_dict_id!='' and need_crawer=1 and belong_adapter_id!="+id,Long.class);
        jdbcTemplate.execute("update "+ dictTableName +" set need_crawer=0");
        if(CollectionUtils.isNotEmpty(needCrawerIds)){
            String ids = "";
            for(Long _id:needCrawerIds){
                ids+=","+_id;
            }
            jdbcTemplate.execute("update "+dictTableName +" set need_crawer=1 where id in ("+ids.substring(1,ids.length())+")");
        }

        //删除适配的数据元
        jdbcTemplate.execute("delete from "+metaDataTableName +" where belong_adapter_id="+id);
        envelop.setSuccessFlg(true);
        return envelop;
    }
}