package com.yihu.hos.standard.service.document;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.yihu.ehr.fastdfs.FastDFSUtil;
import com.yihu.hos.model.standard.MCdaDataSet;
import com.yihu.hos.standard.model.BaseCDADataSetRelationship;
import com.yihu.hos.standard.model.BaseCDADocument;
import com.yihu.hos.standard.model.CDADataSetRelationship;
import com.yihu.hos.standard.model.CDADocument;
import com.yihu.hos.standard.model.standard.StdDataSetModel;
import com.yihu.hos.standard.model.standard.StdMetaDataModel;
import com.yihu.hos.standard.service.standard.StdDatasetService;
import com.yihu.hos.standard.service.standard.StdMetadataService;
import com.yihu.hos.standard.util.CDAVersionUtil;
import com.yihu.hos.web.framework.constant.ErrorCode;
import com.yihu.hos.web.framework.dao.SQLGeneralDAO;
import com.yihu.hos.web.framework.exception.EsbApiException;
import com.yihu.hos.web.framework.model.Result;
import com.yihu.hos.web.framework.model.standard.StandardVersion;
import org.hibernate.Query;
import org.hibernate.Session;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

/**
 * @author AndyCai
 * @version 1.0
 * @created 02-9月-2015 14:00:55
 */
@Service
public class CDADataSetRelationshipManager extends SQLGeneralDAO {

    @Autowired
    private FastDFSUtil fastDFSUtil;

    @Autowired
    private CDADocumentService cdaDocumentService;

    @Autowired
    private StdMetadataService metaDataService;

    @Autowired
    private StdDatasetService dataSetService;


    /**
     * 根据关系ID删除关联关系
     *
     * @param ids
     */
    public boolean deleteRelationshipById(String version, String[] ids) {
        delete(BaseCDADataSetRelationship.class,ids,StandardVersion.CDADatasetTable, version);
        return true;
    }

    /**
     * 同时删除多个时 CDA ID用逗号隔开
     *
     * @param versionCode
     * @param ids
     */
    public boolean deleteDocument(String[] ids,String dataType,String versionCode) throws IOException {
        deleteRelationshipByCdaIds(versionCode,ids);
        delete(CDADocument.class,ids, dataType,versionCode);
        return true;
    }

    /**
     * 根据CDAID删除CDA数据集关联关系
     *
     * @param version
     * @param cdaIds
     */
    public void deleteRelationshipByCdaIds(String version, String[] cdaIds) throws IOException {
        List<BaseCDADataSetRelationship> ls = cdaDocumentService.searchPageData(BaseCDADataSetRelationship.class,StandardVersion.CDADatasetTable,version,"cdaId=" + String.join(",", cdaIds),null,null,null);

        String ids = "";
        for (int i = 0; i < ls.size(); i++) {
            BaseCDADataSetRelationship cdaDataSetRelationship = ls.get(i);
            if (i == 0) {
                ids += cdaDataSetRelationship.getId();
            } else {
                ids += "," + cdaDataSetRelationship.getId();
            }
            //ids += cdaDataSetRelationship.getId();
        }

        deleteByField("id",ids.split(","),StandardVersion.CDADatasetTable,version);
    }

    /**
     * 根据CDAID获取关联关系
     *
     * @param entityClass
     * @param filters
     * @param sorts
     * @param size
     * @param page
     * @return
     */
    public List getCDADataSetRelationships(Class entityClass,String version, String filters, String sorts, Integer size, Integer page) {

        List ls = null;
        try {
            ls = searchVersionData(entityClass,version,StandardVersion.CDADatasetTable, filters, sorts, page, size);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return ls;
    }

    public List getCDADataSetRelationshipByCDAId(Class entityClass,String version, String cdaId) {

        List ls = null;
        try {
            ls = searchVersionData(entityClass, version,StandardVersion.DataSetTable,"cdaId=" + cdaId);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return ls;
    }

    public List getCDADataSetRelationshipByCDAId( String version,String dataType, String cdaId) throws IOException {
        List ls = searchVersionData(CDADataSetRelationship.class,version,StandardVersion.CDADatasetTable, "cdaId=" + cdaId);
        return ls;
    }


    /**
     * 获取关联关系
     */
    public List<CDADataSetRelationship> getCDADataSetRelationship(String versionCode) {
        Session session = currentSession();
        String strTableName = CDAVersionUtil.getCDADatasetRelationshipTableName(versionCode);
        String strSql = "SELECT t.id," +
                "t.cda_id," +
                "t.dataSet_id" +
                " from " + strTableName + " t ";
        List<Object> records = session.createSQLQuery(strSql).list();
        List<CDADataSetRelationship> infos = new ArrayList<>();
        for (int i = 0; i < records.size(); ++i) {
            Object[] record = (Object[]) records.get(i);
            CDADataSetRelationship info = new CDADataSetRelationship();
            info.setId(record[0].toString());
            info.setCdaId(record[1].toString());
            info.setDataSetId(record[2].toString());
            infos.add(info);
        }
        return infos;
    }


    /**
     * 保存CDA信息
     * 1.先删除CDA数据集关联关系信息与cda文档XML文件，在新增信息
     *
     * @param strDataSetIds  关联的数据集
     * @param cdaId       cda文档 ID
     * @param versionCode 版本号
     * @param xmlInfo     xml 文件内容
     * @return 操作结果
     */
    public boolean SaveDataSetRelationship(String strDataSetIds, String cdaId, String versionCode, String xmlInfo) throws Exception {
        deleteRelationshipByCdaIds(versionCode, new String[]{cdaId});

        List<CDADocument> cdaDocuments = cdaDocumentService.searchPageData(BaseCDADocument.class,StandardVersion.CDADocumentTable,versionCode,"id="+cdaId,null,null,null);

        if (cdaDocuments.size() <= 0) return false;

        BaseCDADocument cdaDocument = cdaDocuments.get(0);
        if (cdaDocument.getFileGroup() != null
                && !cdaDocument.getFileGroup().equals("")
                && cdaDocument.getSchema() != null
                && !cdaDocument.getSchema().equals("")) {
            fastDFSUtil.delete(cdaDocument.getFileGroup(), cdaDocument.getSchema());
        }
        String[] dataSetIds = strDataSetIds.split(",");
        if (dataSetIds == null || dataSetIds.length == 0) return true;

        List<CDADataSetRelationship> infos = new ArrayList<>();
        for (int i = 0; i < dataSetIds.length; i++) {
            String dataSetId = dataSetIds[i];
            CDADataSetRelationship info = new CDADataSetRelationship();
            info.setCdaId(cdaId);
            info.setDataSetId(dataSetId);
            infos.add(info);
        }
        //cdaDataSetRelationshipManager.addRelationship(infos,versionCode);
        if (infos.size() > 0) {
            addRelationship(infos, versionCode);
        } else {
            throw new EsbApiException(ErrorCode.GetStdVersionFailed);
        }
        String strFilePath = SaveCdaFile(xmlInfo, versionCode, cdaId);

        //将文件上传到服务器中
        ObjectNode msg = fastDFSUtil.upload(strFilePath, "");
        String strFileGroup = msg.get(FastDFSUtil.GROUP_NAME).asText();
        String strSchemePath = msg.get(FastDFSUtil.REMOTE_FILE_NAME).asText();
        File file = new File(strFilePath);

        // 路径为文件且不为空则进行删除
        if (file.isFile() && file.exists()) {
            file.delete();
        }

        boolean bRes = saveXmlFilePath(new String[]{cdaId}, versionCode, strFileGroup, strSchemePath);
        return bRes;
    }

    /**
     * 将String 保存为XML文件
     *
     * @param fileInfo 文件信息
     * @return 返回 文件路径
     */
    public String SaveCdaFile(String fileInfo, String versionCode, String cdaId) throws IOException {
        fileInfo = fileInfo.replaceAll("&lt;", "<").replaceAll("&gt;", ">");
        String strPath = System.getProperty("java.io.tmpdir");
        String splitMark = System.getProperty("file.separator");
        strPath += "StandardFiles";

        //文件路径
        String strXMLFilePath = strPath + splitMark + "xml" + splitMark + versionCode + splitMark + "createfile" + splitMark + cdaId + ".xml";

        File file = new File(strXMLFilePath);
        if (!file.getParentFile().exists()) {
            file.getParentFile().mkdirs();
        }
        file.createNewFile();

        FileWriter fw = new FileWriter(file, true);
        BufferedWriter bw = new BufferedWriter(fw);
        bw.write(fileInfo);
        bw.flush();
        bw.close();
        fw.close();

        return strXMLFilePath;
    }

    public boolean saveXmlFilePath(String[] cdaIds, String versionCode, String fileGroup, String filePath) throws IOException {


        List<CDADocument> cdaDocuments = cdaDocumentService.searchVersionData(BaseCDADocument.class,versionCode,StandardVersion.CDADocumentTable, "id=" + String.join(",", cdaIds));
        if (cdaDocuments.size() <= 0) {
            return false;
        }

        BaseCDADocument cdaDocument = cdaDocuments.get(0);
        cdaDocument.setFileGroup(fileGroup);
        cdaDocument.setSchema(filePath);
        cdaDocument.setVersionCode(versionCode);
        Result result = cdaDocumentService.modify(versionCode, cdaDocument);
        if(result.isSuccessFlg()){
            return true;
        }
        return false;
    }

    public CDADocument saveCdaInfo(String cdaDocumentJsonData) throws Exception {
        ObjectMapper objectMapper = new ObjectMapper();
        CDADocument cdaInfo = objectMapper.readValue(cdaDocumentJsonData, CDADocument.class);
        if (cdaDocumentService.isDocumentExist(cdaInfo.getVersionCode(), cdaInfo.getCode(), cdaInfo.getId())) {
            throw new Exception("已存在");
        }

        cdaDocumentService.saveEntity(cdaInfo);
        return cdaInfo;
    }


    /**
     * 新增关联关系
     *
     * @param cdaDatasetRelationships
     */
    public boolean addRelationship(List<CDADataSetRelationship> cdaDatasetRelationships, String versionCode) {
        Session session = currentSession();

        String strTableName = CDAVersionUtil.getCDADatasetRelationshipTableName(versionCode);
        String sql;
        Query query;

        String strValues = "";
        for (int i = 0; i < cdaDatasetRelationships.size(); i++) {
            CDADataSetRelationship info = (CDADataSetRelationship) cdaDatasetRelationships.get(i);
            if (org.springframework.util.StringUtils.isEmpty(info.getId())) {
                String strId = UUID.randomUUID().toString();
                info.setId(strId);
            }
            strValues += "('" + info.getId() + "','" + info.getCdaId() + "','" + info.getDataSetId() + "'),";
        }
        strValues = strValues.substring(0, strValues.length() - 1);

        sql = "insert into " + strTableName +
                " (id,cda_id,dataset_id) " +
                "values " + strValues;
        query = session.createSQLQuery(sql);
        query.executeUpdate();
        return true;
    }


    public List<MCdaDataSet> getCDADataSetByCDAId(String version, String cdaId) {
        String sql = "SELECT " +
                " scdsr.id id,scdsr.cda_id cdaId,scdsr.dataset_id dataSetId, sds.CODE dataSetCode,sds.multi_record multiRecord " +
                "FROM " +
                " std_cda_data_set_relationship_" + version + " scdsr, " +
                " std_dataset_" + version + " sds " +
                "WHERE " +
                " 1 = 1 " +
                "AND scdsr.cda_id = '" + cdaId + "' " +
                "AND scdsr.dataset_id = sds.id";
        RowMapper rowMapper = (RowMapper) BeanPropertyRowMapper.newInstance(MCdaDataSet.class);
        return this.jdbcTemplate.query(sql, rowMapper);

    }

    /**
     * 根据CDA Id 和 版本编码 生成CDA文档
     * @param cdaId
     * @param versionCode
     * @return
     * @throws TransformerException
     * @throws ParserConfigurationException
     * @throws FileNotFoundException
     * @throws UnsupportedEncodingException
     */
    public boolean createCDASchemaFile(String cdaId, String versionCode) throws Exception {
        List<CDADataSetRelationship> relationshipsList = getCDADataSetRelationshipByCDAId(CDADataSetRelationship.class,versionCode,cdaId);

        String strPath = System.getProperty("java.io.tmpdir");
        strPath += "StandardFiles";
        String splitMark = System.getProperty("file.separator");
        String strXMLFilePath = strPath + splitMark + "xml" + splitMark + versionCode + splitMark;
        File file = new File(strXMLFilePath);
        if (!file.exists()) {
            file.mkdirs();
        }
        String strFilepath = strXMLFilePath + cdaId + ".xml";
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = factory.newDocumentBuilder();
        Document doc = builder.newDocument();
        Element root = doc.createElement("xs:schema");
        root.setAttribute("xmlns:xs", "http://www.w3.org/2001/XMLSchema");
        root.setAttribute("targetNamespace", "http://www.w3schools.com");
        root.setAttribute("xmlns", "http://www.w3schools.com");
        root.setAttribute("elementFormDefault", "qualified");
        doc.appendChild(root);
        for (int i = 0; i < relationshipsList.size(); i++) {
            //获取数据集
            BaseCDADataSetRelationship dataSetRelationship = relationshipsList.get(i);
            StandardVersion ver = new StandardVersion(versionCode);
            StdDataSetModel dataSet = dataSetService.retrieve(StdDataSetModel.class,ver.getDataSetTableName(),Long.parseLong(dataSetRelationship.getDataSetId()));
            Element rowSet = doc.createElement("xs:table");
            rowSet.setAttribute("code", dataSet.getCode());
            root.appendChild(rowSet);

            //获取数据元
            List<StdMetaDataModel> metaDatas = metaDataService.searchVersionData(StdMetaDataModel.class,StandardVersion.MetaDataTable,versionCode,"dataSetId="+dataSet.getId());

            for (int j = 0; j < metaDatas.size(); j++) {
                Element rowEle = doc.createElement("xs:element");
                rowEle.setAttribute("code", metaDatas.get(j).getCode());

                String strColumnType = StringUtils.isEmpty(metaDatas.get(j).getColumnType()) ? "VARCHAR" : metaDatas.get(j).getColumnType();

                rowEle.setAttribute("type", "xs:" + strColumnType);
                rowSet.appendChild(rowEle);
            }
        }
        //生成XML文件
        outputXml(doc, strFilepath);
        return true;
    }

    private void outputXml(Document doc, String fileName) throws TransformerException, FileNotFoundException, UnsupportedEncodingException {
        TransformerFactory tf = TransformerFactory.newInstance();
        Transformer transformer = tf.newTransformer();

        DOMSource source = new DOMSource(doc);
        transformer.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
        transformer.setOutputProperty(OutputKeys.INDENT, "yes");//设置文档的换行与缩进

        OutputStreamWriter pw = new OutputStreamWriter(new FileOutputStream(fileName), "UTF-8");
        StreamResult result = new StreamResult(pw);
        transformer.transform(source, result);
    }


}