package com.yihu.hos.standard.service.document;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.hos.standard.model.BaseCDADocument;
import com.yihu.hos.standard.model.CDADocument;
import com.yihu.hos.web.framework.dao.SQLGeneralDAO;
import com.yihu.hos.web.framework.model.Result;
import com.yihu.hos.web.framework.model.standard.StandardVersion;
import com.yihu.hos.web.framework.util.DateUtil;
import com.yihu.hos.web.framework.util.sql.SqlCreator;
import org.hibernate.Query;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.List;
import java.util.UUID;

/**
 * @author AndyCai
 * @version 1.0
 * @created 01-9月-2015 17:17:50
 */
@Service
public class CDADocumentService extends SQLGeneralDAO {

    public static final String BEAN_ID = "CDADocumentService";


    public void saveCdaDocument(BaseCDADocument cdaDocument) {
        try {
            saveEntity(cdaDocument);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public Result add(String adapterVersion, BaseCDADocument obj) throws IOException {
        StandardVersion version = new StandardVersion(adapterVersion);
        obj.setId(UUID.randomUUID().toString().replaceAll("-", ""));
        JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(obj));
        SqlCreator sqlCreator = new SqlCreator(BaseCDADocument.class);
        String sql = sqlCreator.insertData(version.getCDATableName(), jsonNode);
        Query query = getExeuteQuery(sqlCreator, sql);
        query.executeUpdate();
        obj.setSuccessFlg(true);
        obj.setMessage("保存成功");
        return obj;
    }

    public Result modify(String adapterVersion, BaseCDADocument obj) {
        try {
            StandardVersion version = new StandardVersion(adapterVersion);
            ObjectMapper mapper = new ObjectMapper();
            mapper.setDateFormat(new SimpleDateFormat(DateUtil.DEFAULT_YMDHMSDATE_FORMAT));
            JsonNode jsonNode = mapper.readTree(mapper.writeValueAsString(obj));
            SqlCreator sqlCreator = new SqlCreator(BaseCDADocument.class);
            String sql = sqlCreator.updateDataByTableKey(version.getCDATableName(), jsonNode);
            Query query = getExeuteQuery(sqlCreator, sql);
            query.executeUpdate();
            obj.setSuccessFlg(true);
            obj.setMessage("修改成功");
            return obj;
        } catch (Exception e) {
            return Result.error("修改失败");
        }
    }

    public boolean isDocumentExist(String version, String documentCode,String documentId) throws IOException {
        List list;
        if(StringUtils.isEmpty(documentId)){
            list = searchVersionData(CDADocument.class,version,StandardVersion.CDADocumentTable, "code="+documentCode);
        }else {
            list = searchVersionData(CDADocument.class,version, StandardVersion.CDADocumentTable, "code="+documentCode+",id="+documentId);
        }
        return list.size() > 0;
    }

    public boolean isFileExists(String cdaId, String strVersionCode) {
        String strFilePath = FilePath(strVersionCode) + cdaId + ".xml";
        File file = new File(strFilePath);
        return file.exists();
    }

    public String FilePath(String strVersionCode) {
        String strPath = System.getProperty("java.io.tmpdir");
        strPath += "StandardFiles";

        String splitMark = System.getProperty("file.separator");
        String strXMLFilePath = strPath + splitMark + "xml" + splitMark + strVersionCode + splitMark;
        return strXMLFilePath;
    }


    public BaseCDADocument retrieve(String version,String id){
        StandardVersion ver = new StandardVersion(version);
        BaseCDADocument baseCDADocument = (BaseCDADocument) get(BaseCDADocument.class, ver.getCDATableName(), id);
        return baseCDADocument;
    }

}