package com.yihu.hos.standard.service.standard;

import com.yihu.hos.standard.model.standard.StdDataSetModel;
import com.yihu.hos.standard.model.standard.StdMetaDataModel;
import com.yihu.hos.standard.util.ExcelUtilBigData;
import com.yihu.hos.web.framework.dao.SQLGeneralDAO;
import com.yihu.hos.web.framework.model.standard.StandardVersion;
import org.springframework.util.StringUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class DataSetExcelUtil extends SQLGeneralDAO implements Runnable {
    private StdDatasetService stdDatasetService;
    private StdDictService stdDictService;
    private StdMetadataService stdMetadataService;
    private String version;
    private File file;
    private Long standardId;
    private String schema;
    public static Map<String, Integer> size = new HashMap<String, Integer>();//记录当前条数 key为version, value为已经导入的条数  当value=-1时,则程序运行时有异常
    public static Map<String, Integer> totle = new HashMap<String, Integer>();//记录总条数
    public static Map<String,String> error = new HashMap<String,String>();//记录错误信息 key为version, value为错误信息


    public DataSetExcelUtil(StdDatasetService stdDatasetService, StdDictService stdDictService, StdMetadataService stdMetadataService, String version, File file, Long standardId, String schema) {
        this.stdDatasetService = stdDatasetService;
        this.stdDictService = stdDictService;
        this.stdMetadataService = stdMetadataService;
        this.version = version;
        this.file = file;
        this.standardId = standardId;
        this.schema = schema;
    }

    @Override
    public void run() {
        try {
            List<List<String[]>> list = null;//String[] 存放一行,List<String[]> 代表一个excel表格
            try {
                list = ExcelUtilBigData.readerExcel(new FileInputStream(file), 13, 0);
                System.out.println("共有"+list.size()+"个数据集sheet");
                DataSetExcelUtil.totle.put(version,list.size());
                DataSetExcelUtil.size.put(version,0);
            } catch (Exception e) {
                e.printStackTrace();
                DataSetExcelUtil.size.put(version,-1);
                DataSetExcelUtil.error.put(version,e.getMessage());
                return;
            }
            for (int i = 0; i < list.size(); i++) {
                List<String[]> l = list.get(i);
                //遍历sheet
                //新增数据集
                StandardVersion standardVersion = new StandardVersion(version);
                StdDataSetModel stdeDataset = new StdDataSetModel();
                stdeDataset.setName(getNotNull(l.get(0)[1]));
                stdeDataset.setCode(getNotNull(l.get(1)[1]));
                stdeDataset.setReference((getNotNull(l.get(2)[1])));
                stdeDataset.setSummary(getNotNull(l.get(3)[1]));
                stdeDataset.setStandardId(standardId);
                String insertSql = null;
                try {
                    insertSql = stdDatasetService.getInsertSql(standardVersion,stdeDataset,schema);
                } catch (IOException e) {
                    DataSetExcelUtil.size.put(version,-1);
                    DataSetExcelUtil.error.put(version,e.getMessage());
                    e.printStackTrace();
                    return;
                }
                stdDatasetService.getJdbcTemplate().execute(insertSql);
                Long id = stdDatasetService.getIdByCode(version,stdeDataset.getCode(),schema);
                stdeDataset.setId(id);

                //用于批量保存
                List<String> sqls = new ArrayList<>();
                for (int row = 5; row < l.size(); row++) {
                    //保存数据元
                    StdMetaDataModel stdeMetadata = new StdMetaDataModel();
                    stdeMetadata.setDataSetId(stdeDataset.getId());//肯定要设置
                    stdeMetadata.setStandardId(standardId);
                    stdeMetadata.setInnerCode(getNotNull(l.get(row)[1]));
                    stdeMetadata.setCode(getNotNull(l.get(row)[2]));
                    stdeMetadata.setName(getNotNull(l.get(row)[3]));
                    stdeMetadata.setDefinition(getNotNull(l.get(row)[4]));
                    stdeMetadata.setType(getNotNull(l.get(row)[5]));
                    stdeMetadata.setFormat(getNotNull(l.get(row)[6]));
                    String dicCode = getNotNull(l.get(row)[7]);
                    if(org.apache.commons.lang3.StringUtils.isNotBlank(dicCode)){
                        //通过dicCode查找dicId
                        Long dicId = stdDictService.getIdByCode(version, dicCode, schema);
                        stdeMetadata.setDictId(dicId);
                    }
                    stdeMetadata.setColumnName(getNotNull(l.get(row)[8]));
                    stdeMetadata.setColumnType(getNotNull(l.get(row)[9]));
                    stdeMetadata.setColumnLength(getNotNull(l.get(row)[10]));
                    stdeMetadata.setStdVersion(version);
                    String isPrimaryKey = getNotNull(l.get(row)[11]);
                    stdeMetadata.setPrimaryKey(false);
                    if(org.apache.commons.lang3.StringUtils.isNotBlank(isPrimaryKey)){
                        if("1".equals(isPrimaryKey)){
                            stdeMetadata.setPrimaryKey(true);
                        }
                    }
                    String isNullable = getNotNull(l.get(row)[12]);
                    stdeMetadata.setNullable(true);
                    if(org.apache.commons.lang3.StringUtils.isNotBlank(isNullable)){
                        if("1".equals(isPrimaryKey)){
                            stdeMetadata.setNullable(false);
                        }
                    }
                    if(org.apache.commons.lang3.StringUtils.isBlank(stdeMetadata.getInnerCode())){
                        continue;
                    }
                    //单条插入数据效率太低,更改成批量保存
                    //stdMetadataService.add(standardVersion, stdeMetadata);
                    String sql = null;
                    try {
                        sql = stdMetadataService.getInsertSql(standardVersion,stdeMetadata,schema);
                    } catch (IOException e) {
                        e.printStackTrace();
                        DataSetExcelUtil.size.put(version,-1);
                        DataSetExcelUtil.error.put(version,e.getMessage());
                        return;
                    }
                    sqls.add(sql);
                }
                if(sqls.size()>0){
                    stdDatasetService.insertBatch(sqls);
                }
                System.out.println("插入了第" + (i + 1) + "个数据集sheet");
                DataSetExcelUtil.size.put(version,i+1);
            }

        }catch (Exception e){
            e.printStackTrace();
            DataSetExcelUtil.size.put(version,-1);
            DataSetExcelUtil.error.put(version,e.getMessage());
            return;
        }


    }

    private String getNotNull(Object obj) {
        if (!StringUtils.isEmpty(obj)) {
            return obj.toString().replace("\"", "");
        }
        return "";
    }
}