package com.yihu.hos.standard.service.standard;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.ehr.util.rest.Envelop;
import com.yihu.hos.core.datatype.CollectionUtil;
import com.yihu.hos.core.datatype.StringUtil;
import com.yihu.hos.standard.model.adapter.AdapterSchemeModel;
import com.yihu.hos.standard.model.standard.*;
import com.yihu.hos.standard.util.ExcelUtilBigData;
import com.yihu.hos.web.framework.baseservice.BaseJpaService;
import com.yihu.hos.web.framework.constant.ContextAttributes;
import com.yihu.hos.web.framework.constant.ErrorCode;
import com.yihu.hos.web.framework.constant.SqlConstants;
import com.yihu.hos.web.framework.exception.ApiException;
import com.yihu.hos.web.framework.model.DictItem;
import com.yihu.hos.web.framework.model.DictionaryResult;
import com.yihu.hos.web.framework.model.Result;
import com.yihu.hos.web.framework.model.standard.StandardVersion;
import com.yihu.hos.web.framework.thread.LocalContext;
import com.yihu.hos.web.framework.util.sql.RequestParamTransformer;
import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.*;

/**
 * @created Airhead 2015/12/25.
 */
@Transactional
@Service("StandardVersionService")
public class StandardVersionService extends BaseJpaService {

    public static final String BEAN_ID = "StandardVersionService";
    @Autowired
    private ObjectMapper objectMapper;

    @Autowired
    private StdDatasetService stdDatasetService;

    @Autowired
    private StdMetadataService stdMetadataService;

    @Autowired
    private StdDictService stdDictService;

    @Autowired
    private StdDictEntryService stdDictEntryService;

    public DictionaryResult getDictionaryResult(Long standardId, String condition, String order, Integer rows, Integer page) {
        try {
            DictionaryResult dictionaryResult = DictionaryResult.success("获取标准版本成功");
            List<DictItem> detailModelList = new ArrayList<>();
            List<STDVersion> standardVersionModelList = getVersionList(standardId, condition, order, rows, page);

            for(STDVersion standardVersionModel : standardVersionModelList){
                DictItem items = new DictItem();
                String id = standardVersionModel.getVersion();
                String name = standardVersionModel.getVersionName();
                items.setCode(id.toString());
                items.setValue(name);
                detailModelList.add(items);
            }
            dictionaryResult.setDetailModelList(detailModelList);
            return dictionaryResult;
        } catch (Exception e) {
            return DictionaryResult.error("获取标准版本失败");
        }
    }

    public DictionaryResult getDetailResultModelList(Long standardId, String condition, String order, Integer rows, Integer page) {
        try {
            List<STDVersion> standardVersionModelList = getVersionList(standardId, condition, order, rows, page);
            DictionaryResult dictionaryResult = new DictionaryResult();
            List<DictItem> detailModelList = new ArrayList<DictItem>();
            DictItem item = new DictItem();
            item.setCode("");
            item.setValue("");
            detailModelList.add(item);
            for (STDVersion standardVersionModel : standardVersionModelList) {
                DictItem items = new DictItem();
                String  id = standardVersionModel.getVersion();
                String name = standardVersionModel.getVersionName();
                items.setCode(id.toString());
                items.setValue(name);
                detailModelList.add(items);
            }
            dictionaryResult.setDetailModelList(detailModelList);
            dictionaryResult.setSuccessFlg(true);
            dictionaryResult.setMessage("获取标准版本列表成功");
            return dictionaryResult;
        } catch (Exception e) {
            return DictionaryResult.error("获取标准版本列表失败");
        }
    }


    public Envelop getResultlList(Long standardId, String condition, String order, Integer rows, Integer page) {
            List<STDVersion> standardVersionModelList = getVersionList(standardId, condition, order, rows, page);
            Envelop result = new Envelop();
            result.setSuccessFlg(true);
            result.setDetailModelList(standardVersionModelList);
            result.setTotalCount(standardVersionModelList.size());
            result.setCurrPage(page);
            result.setPageSize(rows);
            if (result.getTotalCount() % result.getPageSize() > 0) {
                result.setTotalPage((result.getTotalCount() / result.getPageSize()) + 1);
            } else {
                result.setTotalPage(result.getTotalCount() / result.getPageSize());
            }
            return result;
    }

    public List<STDVersion> getVersionList(Long standardId, String condition, String order, Integer limit, Integer offset) {
        try {
            Map queryMap = RequestParamTransformer.parseJsonToMap(condition);
            Map orderMap = RequestParamTransformer.parseJsonToMap(order);
            return this.getList(standardId, queryMap, orderMap, limit, offset);
        } catch (Exception e) {
            e.printStackTrace();
        }

        throw new ApiException(ErrorCode.StandardVersionGetListFailed);
    }

    /**
     * 通过版本号获取standardId
     * @param version
     * @return
     */
    public Long getByVersion(String version){
        String sql = "FROM STDVersion WHERE version = :version";
        Query query = getCurrentSession().createQuery(sql);
        query.setParameter("version", version);
        List<STDVersion> list = query.list();
        if(list==null || list.size()==0){
            return null;
        }
        return list.get(0).getStandardId();
    }

    public Result add(String version) throws IOException {
        STDVersion versionModel = objectMapper.readValue(version, STDVersion.class);
        Boolean existFlg = versionNameValidate(versionModel.getStandardId(), versionModel.getVersionName());
        if(existFlg){
            return Result.error("已存在标准版本名称");
        }else{
            boolean add = add(versionModel);
            versionModel.setSuccessFlg(add);
            if(add){
                versionModel.setMessage("保存标准版本成功");
            }else{
                versionModel.setMessage("保存标准版本失败");
            }
            return versionModel;
        }
    }

    public Boolean versionNameValidate(Long standardId, String versionName) {
            Map<String, Object> query = new HashMap<>();
            Map<String, String> order = new HashMap<>();
            query.put("versionName", versionName);
            List<STDVersion> list = getList(standardId, query, order, 1, 0);
            if (!CollectionUtil.isEmpty(list)) {
                return true;
            }
            return false;
    }

    public Boolean versionNameValidate(Long standardId, String version, String versionName) throws Exception {
        String sql = "FROM STDVersion WHERE versionName = :versionName and standardId = :standardId and version != :version";
        Query query = getCurrentSession().createQuery(sql);
        query.setParameter("standardId", standardId);
        query.setParameter("versionName", versionName);
        query.setParameter("version", version);
        List<STDVersion> list = query.list();
        if (!CollectionUtil.isEmpty(list)) {
            return true;
        }
        return false;
    }

    public Result delete(Long versionId,String version) throws Exception {
        Boolean existFlg = false;
        Boolean useredFlg = false;
        try {
            STDVersion versionModel = (STDVersion) get(versionId);
            if (!versionModel.isInStage()) {
                existFlg = true;
            } else {
                String sql = "FROM AdapterSchemeModel WHERE stdVersion = :stdVersion or adapterStdVersion =  :adapterStdVersion";
                Query query = getCurrentSession().createQuery(sql);
                query.setParameter("stdVersion", version);
                query.setParameter("adapterStdVersion", version);
                List<AdapterSchemeModel> list = query.list();
                if (!CollectionUtil.isEmpty(list)) {
                    useredFlg = true;
                }else{
                    this.delete(versionModel);
                    deleteVersionTables(new StandardVersion(versionModel.getVersion()));
                    return Result.success("删除标准版本成功");
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (existFlg) {
            return Result.error("该标准版本已发布");
        } else if(useredFlg){
            return Result.error("该标准版本已被适配使用");
        } else {
            return Result.error("删除标准版本失败");
        }
    }

    public Result update(String version) throws Exception {
        Boolean existFlg = false;
        try {
            STDVersion versionModel = objectMapper.readValue(version, STDVersion.class);
            existFlg = versionNameValidate(versionModel.getStandardId(), versionModel.getVersion(), versionModel.getVersionName());
            if (!existFlg) {
                this.update(versionModel);
                versionModel.setSuccessFlg(true);
                versionModel.setMessage("修改标准版本成功");
                return versionModel;
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        if (existFlg) {
            return Result.error("已存在标准版本名称");
        } else {
            return Result.error("修改标准版本失败");
        }
    }

    public Result get(Long versionId) {
        try {
            STDVersion versionModel = this.myGet(versionId);
            versionModel.setSuccessFlg(true);
            versionModel.setMessage("获取标准版本信息成功");
            return versionModel;
        } catch (DataAccessException e) {
            return Result.error("获取标准版本信息失败");
        }
    }

    public Result publish(Long versionId, String publisher) throws Exception {
        STDVersion standardVersionModel = this.myPublish(versionId, publisher);

        standardVersionModel.setSuccessFlg(true);
        standardVersionModel.setMessage("发布标准版本成功");
        return standardVersionModel;
    }

    public STDVersion getLastestVersion(Long standardId) {
        Map<String, Object> query = new HashMap<>();
        Map<String, String> order = new HashMap<>();
        order.put("publisher", SqlConstants.DESC);

        List<STDVersion> list = getList(standardId, query, order, 1, 0);
        if (list != null && list.size() != 0) {
            return list.get(0);
        }

        return null;
    }

    public STDVersion myPublish(Long versionId, String publisher) throws Exception {
        STDVersion standardVersionModel = myGet(versionId);
        String path = "";
        standardVersionModel.setPath(path);
        standardVersionModel.setCommitTime(new Date());
        standardVersionModel.setPublishUser(publisher);
        standardVersionModel.setInStage(false);

        this.update(standardVersionModel);

        return standardVersionModel;
    }

    private STDVersion myGet(Long versionId) {
        try {
            return (STDVersion) getEntity(STDVersion.class, versionId);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    private void delete(STDVersion standardVersionModel) throws Exception {
        deleteEntity(standardVersionModel);
    }

    private void update(STDVersion standardVersionModel) throws Exception {
        try {
            updateEntity(standardVersionModel);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private boolean add(STDVersion standardVersionModel){
        boolean b = false;
        StandardVersion newVersion = null;
        try {
            StandardVersion standardVersion = StandardVersion.generateVersion();
            standardVersionModel.setVersion(standardVersion.getVersion());
            standardVersionModel.setCommitTime(null);
            standardVersionModel.setInStage(true);
            String baseVer = standardVersionModel.getBaseVersion();
            StandardVersion baseVersion = StringUtil.isEmpty(baseVer) ? null : new StandardVersion(baseVer);
            newVersion = new StandardVersion(standardVersionModel.getVersion());
            buildVersionTables(baseVersion, newVersion, standardVersionModel);
            saveEntity(standardVersionModel);
            b = true;
        }catch (Exception e){
            //保存失败后,尝试删除新建的表
            if(newVersion!=null){
                try {
                    deleteVersionTables(newVersion);
                }catch (Exception e1){
                }
                e.printStackTrace();
            }
        }
        return b;
    }

    private List<STDVersion> getList(Long standardId, Map<String, Object> query, Map<String, String> order, Integer limit, Integer offset) {
        Session session = getCurrentSession();
        Criteria criteria = session.createCriteria(STDVersion.class);
        criteria.add(Restrictions.eq("standardId", standardId));
        if (query != null) {
            for (String key : query.keySet()) {
                criteria.add(Restrictions.eq(key, query.get(key)));
            }
        }
        if (limit != null) {
            criteria.setMaxResults(limit);
            if (offset != null) {
                criteria.setFirstResult((offset - 1) * limit);
            }
        }
        if (order != null) {
            for (String key : order.keySet()) {
                if (order.get(key).equals(SqlConstants.ASC)) {
                    criteria.addOrder(Order.asc(key));
                } else if (order.get(key).equals(SqlConstants.DESC)) {
                    criteria.addOrder(Order.desc(key));
                }
            }
        }
        return (List<STDVersion>) criteria.list();
    }

    private void deleteVersionTables(StandardVersion version) {
        String[] tables = {version.getDataSetTableName(),
                version.getMetaDataTableName(),
                version.getDictTableName(),
                version.getDictEntryTableName(),
                version.getCDATableName(),
                version.getCDADatasetTableName()};
        List<String> deleteSql = new ArrayList<>();
        for (int i = 0; i < tables.length; ++i) {
            String table = tables[i];
            deleteSql.add(SqlConstants.DROP_TABLE + table);
        }
        insertBatch(deleteSql);
    }

    private void buildVersionTables(StandardVersion baseVersion, StandardVersion newVersion, STDVersion stdVersion) {

        String[] newTables = {
                newVersion.getDataSetTableName(),//数据集
                newVersion.getMetaDataTableName(),//数据元
                newVersion.getDictTableName(),//字典
                newVersion.getDictEntryTableName(),//字典项
                newVersion.getCDATableName(),//cda
                newVersion.getCDADatasetTableName(),//cda关联关系
                newVersion.getStdIndexTableName(),//索引
                newVersion.getStdRelationTableName()//关联关系
        };

        String[] baseTables;
        if (baseVersion == null) {
            String[] templateTables = {
                    StandardVersion.DataSetTable,
                    StandardVersion.MetaDataTable,
                    StandardVersion.DictTable,
                    StandardVersion.DictEntryTable,
                    StandardVersion.CDADocumentTable,
                    StandardVersion.CDADatasetTable,
                    StandardVersion.StdIndexTable,
                    StandardVersion.StdRelationTable
            };
            baseTables = templateTables;
        } else {
            String[] templateTables = {
                    baseVersion.getDataSetTableName(),
                    baseVersion.getMetaDataTableName(),
                    baseVersion.getDictTableName(),
                    baseVersion.getDictEntryTableName(),
                    baseVersion.getCDATableName(),
                    baseVersion.getCDADatasetTableName(),
                    baseVersion.getStdIndexTableName(),
                    baseVersion.getStdRelationTableName()
            };
            baseTables = templateTables;
        }

        List<String> createSql = new ArrayList<>();
        List<String> insertSql = new ArrayList<>();
        List<String> updateSql = new ArrayList<>();
        for (int i = 0; i < baseTables.length; ++i) {
            String baseTable = baseTables[i];
            String newTable = newTables[i];
            createSql.add(SqlConstants.CREATE_TABLE + newTable + SqlConstants.LIKE + baseTable);
        }

        for (int i = 0; i < baseTables.length; ++i) {
            String baseTable = baseTables[i];
            String newTable = newTables[i];
            insertSql.add(SqlConstants.INSERT_INTO + newTable + SqlConstants.SELECT + SqlConstants.ASTERISK + SqlConstants.FROM + baseTable);
        }

        for (int i = 0; i < baseTables.length; ++i) {
            String newTable = newTables[i];
            if(!newTable.toLowerCase().contains("std_dataset_relation") && !newTable.toLowerCase().contains("std_index")){
                updateSql.add(SqlConstants.UPDATE + newTable + SqlConstants.SET + " STD_ID = " + stdVersion.getStandardId());
                updateSql.add(SqlConstants.UPDATE + newTable + SqlConstants.SET + " STD_VERSION = '" + stdVersion.getVersion()+"'");
            }
            if(newTable.toLowerCase().contains("std_index")){
                updateSql.add(SqlConstants.UPDATE + newTable + SqlConstants.SET + " VERSION = '" + stdVersion.getVersion()+"'");
            }
        }
        insertBatch(createSql);
        insertBatch(insertSql);
        insertBatch(updateSql);
    }

    public String saveDatasetExcel(File file, String version, Long standardId) throws Exception {
        //每次调用微服务时,schema都会被重新赋值....
        //故这边得自己手动设置schema....

        //获取schema
        String schema = LocalContext.getContext().getAttachment(ContextAttributes.SCHEMA);



        List<List<String[]>> list = ExcelUtilBigData.readerExcel(new FileInputStream(file), 13, 0);//String[] 存放一行,List<String[]> 代表一个excel表格
        for (int i = 0; i < list.size(); i++) {
            List<String[]> l = list.get(i);
            //遍历sheet
            //新增数据集
            StandardVersion standardVersion = new StandardVersion(version);
            StdDataSetModel stdeDataset = new StdDataSetModel();
            stdeDataset.setName(getNotNull(l.get(0)[1]));
            stdeDataset.setCode(getNotNull(l.get(1)[1]));
            stdeDataset.setSummary(getNotNull(l.get(3)[1]));
            stdeDataset.setLanguage(getNotNull(l.get(4)[1]));
            stdeDataset.setClassModel(getNotNull(l.get(5)[1]));
            stdeDataset.setStandardId(standardId);
            stdeDataset.setCatalog(0);
            if(org.apache.commons.lang3.StringUtils.isNotBlank(getNotNull(l.get(2)[1]))){
                stdeDataset.setCatalog(Integer.valueOf(getNotNull(l.get(2)[1])));
            }
            stdDatasetService.add(standardVersion.getDataSetTableName(), stdeDataset);

            //用于批量保存
            List<String> sqls = new ArrayList<>();
            for (int row = 5; row < l.size(); row++) {
                //保存数据元
                StdMetaDataModel stdeMetadata = new StdMetaDataModel();
                stdeMetadata.setDataSetId(stdeDataset.getId());//肯定要设置
                stdeMetadata.setStandardId(standardId);
                stdeMetadata.setInnerCode(getNotNull(l.get(row)[1]));
                stdeMetadata.setCode(getNotNull(l.get(row)[2]));
                stdeMetadata.setName(getNotNull(l.get(row)[3]));
                stdeMetadata.setDefinition(getNotNull(l.get(row)[4]));
                stdeMetadata.setType(getNotNull(l.get(row)[5]));
                stdeMetadata.setFormat(getNotNull(l.get(row)[6]));
                String dicCode = getNotNull(l.get(row)[7]);
                if(org.apache.commons.lang3.StringUtils.isNotBlank(dicCode)){
                    //通过dicCode查找dicId
                    Long dicId = null;
                    JSONObject condition =new JSONObject();
                    condition.put("dicCode",dicCode);
                    List<StdDictionaryModel> dictionaryModels = stdDictService.getList(version, condition.toString(), new JSONObject().toString(), null, null);
                    if(dictionaryModels!=null&&dictionaryModels.size()>0){
                        dicId = dictionaryModels.get(0).getId();
                    }
                    stdeMetadata.setDictId(dicId);
                }
                stdeMetadata.setColumnName(getNotNull(l.get(row)[8]));
                stdeMetadata.setColumnType(getNotNull(l.get(row)[9]));
                stdeMetadata.setColumnLength(getNotNull(l.get(row)[10]));
                stdeMetadata.setStdVersion(version);
                String isPrimaryKey = getNotNull(l.get(row)[11]);
                stdeMetadata.setPrimaryKey(false);
                if(org.apache.commons.lang3.StringUtils.isNotBlank(isPrimaryKey)){
                    if("1".equals(isPrimaryKey)){
                        stdeMetadata.setPrimaryKey(true);
                    }
                }
                String isNullable = getNotNull(l.get(row)[12]);
                stdeMetadata.setNullable(false);
                if(org.apache.commons.lang3.StringUtils.isNotBlank(isNullable)){
                    if("1".equals(isPrimaryKey)){
                        stdeMetadata.setNullable(true);
                    }
                }
                if(org.apache.commons.lang3.StringUtils.isBlank(stdeMetadata.getInnerCode())){
                    continue;
                }
                //单条插入数据效率太低,更改成批量保存
                //stdMetadataService.add(standardVersion, stdeMetadata);
                String sql = stdMetadataService.getInsertSql(standardVersion,stdeMetadata,schema);
                sqls.add(sql);
            }
            if(sqls.size()>0){
                super.insertBatch(sqls);
            }
        }
        return version;
    }

    public String saveDictExcel(File file, String version,Long standardId) throws Exception {

        //每次调用微服务时,schema都会被重新赋值....
        //故这边得自己手动设置schema....

        //获取schema
        String schema = LocalContext.getContext().getAttachment(ContextAttributes.SCHEMA);

        List<List<String[]>> list = ExcelUtilBigData.readerExcel(new FileInputStream(file), 6, 3);
        System.out.println("共有"+list.size()+"个sheet");
        StandardVersion standardVersion = new StandardVersion(version);
        for (int i = 0; i < list.size(); i++) {
            List<String[]> l = list.get(i);
            //遍历sheet
            StdDictionaryModel stdeDict = new StdDictionaryModel();
            stdeDict.setStandardId(standardId);
            stdeDict.setCode(getNotNull(l.get(0)[1]));
            stdeDict.setName(getNotNull(l.get(1)[1]));
//            stdeDict.setDefine(getNotNull(l.get(2)[1]));
//            stdeDict.setDesc(getNotNull(l.get(3)[1]));
//            String parentId = getNotNull(l.get(4)[1]);
//            if(!StringUtils.isEmpty(parentId)){
//                stdeDict.setBaseDict(Integer.valueOf(parentId));
//            }
            stdDictService.add(standardVersion,stdeDict);
            List<String> sqls = new ArrayList<>();
            for (int row = 0; row < l.size(); row++) {
                if (StringUtils.isEmpty(l.get(row)[4])) {
                    continue;
                }
                StdDictionaryEntryModel stdeDictItem = new StdDictionaryEntryModel();
                stdeDictItem.setStandardId(standardId);
                stdeDictItem.setDictId(stdeDict.getId());
                stdeDictItem.setValue(getNotNull(l.get(row)[4]));
                stdeDictItem.setCode(getNotNull(l.get(row)[3]));
                stdeDictItem.setDesc(getNotNull(l.get(row)[5]));
                //单条插入太慢了.....改成批量插入
                //stdDictEntryService.add(standardVersion,stdeDictItem);
                String sql = stdDictEntryService.getInsertSql(standardVersion,stdeDictItem,schema);
                sqls.add(sql);
            }
            if(sqls.size()>0){
                super.insertBatch(sqls);
            }
            System.out.println("插入了第"+(i+1)+"个sheet");
        }
        return version;
    }

    private String getNotNull(Object obj) {
        if (!StringUtils.isEmpty(obj)) {
            return obj.toString().replace("\"", "");
        }
        return "";
    }

    public String isUpdateDatasetAndDict(String version) {
        JSONObject jo=new JSONObject();
        List<StdDataSetModel> stdDataSetModels = stdDatasetService.getList(version, null, null, null, null);
        if(stdDataSetModels!=null&&stdDataSetModels.size()>0){
            jo.put("datasetUpdate","true");
        }else{
            jo.put("datasetUpdate","false");
        }
        List<StdDictionaryModel> stdDictionaryModels = stdDictService.getList(version, null, null, null, null);
        if(stdDictionaryModels!=null&&stdDictionaryModels.size()>0){
            jo.put("dictUpdate","true");
        }else{
            jo.put("dictUpdate","false");
        }
        return jo.toString();
    }

    @Transactional
    public void deleteDataset(String version) throws Exception {
        Session session = getCurrentSession();
        //删除数据集
        String sql1 = "truncate table std_dataset_" + version;
        session.createSQLQuery(sql1).executeUpdate();
        //删除数据元
        String sql = "truncate table std_metadata_" + version;
        session.createSQLQuery(sql).executeUpdate();
    }

    public void deleteDict(String version) {
        Session session = getCurrentSession();
        //删除字典
        String sql1 = "truncate table std_dictionary_" + version;
        session.createSQLQuery(sql1).executeUpdate();
        //删除字典项
        String sql = "truncate table std_dictionary_entry_" + version;
        session.createSQLQuery(sql).executeUpdate();
    }
}

