package com.yihu.hos.standard.service.standard;

import com.fasterxml.jackson.databind.JsonNode;
import com.yihu.hos.core.datatype.CollectionUtil;
import com.yihu.hos.core.datatype.StringUtil;
import com.yihu.hos.standard.model.Select2;
import com.yihu.hos.standard.model.standard.STDVersion;
import com.yihu.hos.standard.model.standard.StandardModel;
import com.yihu.hos.standard.model.standard.StdDataSetModel;
import com.yihu.hos.standard.model.standard.resultModel.StandardParamResultModel;
import com.yihu.hos.standard.util.CDAVersionUtil;
import com.yihu.hos.web.framework.constant.ErrorCode;
import com.yihu.hos.web.framework.constant.SqlConstants;
import com.yihu.hos.web.framework.dao.SQLGeneralDAO;
import com.yihu.hos.web.framework.exception.ApiException;
import com.yihu.hos.web.framework.model.DetailModelResult;
import com.yihu.hos.web.framework.model.Result;
import com.yihu.hos.web.framework.model.standard.StandardVersion;
import com.yihu.hos.web.framework.util.sql.BeanTransformer;
import com.yihu.hos.web.framework.util.sql.RequestParamTransformer;
import com.yihu.hos.web.framework.util.sql.SqlCreator;
import org.apache.commons.lang.StringUtils;
import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.SQLQuery;
import org.hibernate.Session;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.sql.PreparedStatement;
import java.util.*;
import java.util.stream.Collectors;


/**
 * 数据集管理器.
 *
 * @author Sand
 * @version 1.0
 * @created 30-6月-2015 16:19:05
 */
@Transactional
@Service("StdDatasetService")
public class StdDatasetService extends SQLGeneralDAO {

    public static final String BEAN_ID = "StdDatasetService";

    @Autowired
    StandardService standardService;

    @Autowired
    StdMetadataService metaDataService;

    @Autowired
    JdbcTemplate jdbcTemplate;

    public StdDatasetService() {
    }

    public void finalize() throws Throwable {
        super.finalize();
    }

    public List<StdDataSetModel> getList(String stdVersion, String condition, String order, Integer limit, Integer offset) {
        try {
            Map queryMap = RequestParamTransformer.parseJsonToMap(condition);
            Map orderMap = RequestParamTransformer.parseJsonToMap(order);
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            return this.getList(standardVersion, queryMap, orderMap, limit, offset);
        } catch (Exception e) {
            e.printStackTrace();
        }
        throw new ApiException(ErrorCode.StandardDatasetGetListFailed);
    }

    public Result add(String stdVersion, String dataset) {
        if (StringUtil.isEmpty(stdVersion)) {
            return Result.error("并无该标准版本");
        }
        Boolean existFlg = false;
        try {
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            StdDataSetModel stdDataSetModel = objectMapper.readValue(dataset, StdDataSetModel.class);
            existFlg = datasetCodeValidate(standardVersion, stdDataSetModel.getCode());
            if (!existFlg) {
                stdDataSetModel.setStdVersion(stdVersion);
                Long standardId = getByVersion(stdVersion);
                stdDataSetModel.setStandardId(standardId);
                this.add(standardVersion.getDataSetTableName(), stdDataSetModel);
                stdDataSetModel.setSuccessFlg(true);
                stdDataSetModel.setMessage("保存数据集成功");
                return stdDataSetModel;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (existFlg) {
            return Result.error("已存在标准数据集编码");
        } else {
            return Result.error("保存数据集失败");
        }
    }

    public Boolean datasetCodeValidate(StandardVersion standardVersion, String code) {
        Map<String, Object> query = new HashMap<>();
        Map<String, String> order = new HashMap<>();
        query.put("code", code);

        List<StdDataSetModel> list = this.getList(standardVersion, query, order, 1, 1);
        if (!CollectionUtil.isEmpty(list)) {
            return true;
        }
        return false;
    }

    public Result delete(String stdVersion, Long datasetId) {
        if (StringUtil.isEmpty(stdVersion)) {
            return Result.error("并无该标准版本");
        }
        try {
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            if (datasetId != null) {
                this.delete(standardVersion, datasetId);
            }
            return Result.success("删除数据集成功");
        } catch (Exception e) {
            return Result.success("删除数据集失败");
        }
    }

    public Boolean datasetCodeValidate(StandardVersion standardVersion, Long datasetId,  String code) {
        SqlCreator sqlCreator = new SqlCreator(StdDataSetModel.class);
        sqlCreator.equalCondition("code", code);
        sqlCreator.neCondition("id", datasetId);
        String sql = sqlCreator.selectData(standardVersion.getDataSetTableName());
        Query sessionQuery = getQuery(sqlCreator, sql);
        List<StdDataSetModel> list = sessionQuery.list();
        if (!CollectionUtil.isEmpty(list)) {
            return true;
        }
        return false;
    }

    public Result update(String stdVersion, String dataset) {
        if (StringUtil.isEmpty(stdVersion)) {
            return Result.error("并无该标准版本");
        }
        Boolean existFlg = false;
        try {
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            StdDataSetModel stdDataSetModel = objectMapper.readValue(dataset, StdDataSetModel.class);
            existFlg = datasetCodeValidate(standardVersion, stdDataSetModel.getId(), stdDataSetModel.getCode());
            if (!existFlg) {
                this.update(standardVersion, stdDataSetModel);
                stdDataSetModel.setSuccessFlg(true);
                stdDataSetModel.setMessage("修改数据集成功");
                return stdDataSetModel;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (existFlg) {
            return Result.error("已存在标准数据集编码");
        } else {
            return Result.error("修改数据集失败");
        }
    }

    public Result get(String stdVersion, Long datasetId) {
        if (StringUtil.isEmpty(stdVersion)) {
            return Result.error("并无该数据集信息");
        }

        try {
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            if (datasetId != null) {
                StdDataSetModel stdDataSetModel = this.get(standardVersion, datasetId);
                stdDataSetModel.setSuccessFlg(true);
                stdDataSetModel.setMessage("获取标准数据集成功");
                return stdDataSetModel;
            }

        } catch (Exception e) {
            e.printStackTrace();
        }

        return Result.error("获取标准数据集失败");
    }

    public StdDataSetModel get(StandardVersion standardVersion, Long datasetId) {
        String dataSetTable = standardVersion.getDataSetTableName();
        SqlCreator sqlCreator = new SqlCreator(StdDataSetModel.class);
        sqlCreator.equalCondition("id", datasetId);
        String sql = sqlCreator.selectData(dataSetTable);
        Query query = getQuery(sqlCreator, sql);
        return (StdDataSetModel) query.uniqueResult();
    }

    private void update(StandardVersion standardVersion, StdDataSetModel stdDataSetModel) throws IOException {
        JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(stdDataSetModel));
        SqlCreator sqlCreator = new SqlCreator(StdDataSetModel.class);
        String dataSetTable = standardVersion.getDataSetTableName();
        String sql = sqlCreator.updateDataByTableKey(dataSetTable, jsonNode);
        Query query = getExeuteQuery(sqlCreator, sql);
        query.executeUpdate();
    }

    private void delete(StandardVersion standardVersion, Long datasetId) {
        Session session = getCurrentSession();
        String sql = "delete from " + standardVersion.getDataSetTableName() + " where id = :id";
        Query query = session.createSQLQuery(sql);
        query.setLong("id", datasetId);
        query.executeUpdate();
        sql = "delete from " + standardVersion.getMetaDataTableName() + " where dataset_id = :datasetId";
        query = session.createSQLQuery(sql);
        query.setLong("datasetId", datasetId);
        query.executeUpdate();
    }

    public void add(String dataSetTable, StdDataSetModel stdDataSetModel) throws IOException {
        int maxId = getMaxId(dataSetTable);
        stdDataSetModel.setId(Long.valueOf(maxId));

        JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(stdDataSetModel));
        SqlCreator sqlCreator = new SqlCreator(StdDataSetModel.class);
        String sql = sqlCreator.insertData(dataSetTable, jsonNode);
        Query query = getExeuteQuery(sqlCreator, sql);
        query.executeUpdate();
    }

    private List<StdDataSetModel> getList(StandardVersion standardVersion, Map<String, Object> query, Map<String, String> order, Integer limit, Integer offset) {
        SqlCreator sqlCreator = new SqlCreator(StdDataSetModel.class);
        for (String key : query.keySet()) {
            sqlCreator.equalCondition(key, query.get(key));
        }
        for (String key : order.keySet()) {
            sqlCreator.order(key, order.get(key));
        }

        String sql = sqlCreator.selectData(standardVersion.getDataSetTableName());
        Query sessionQuery = getQuery(sqlCreator, sql);
        if (limit != null) {
            sessionQuery.setMaxResults(limit);
            if (offset != null) {
                sessionQuery.setFirstResult((offset - 1) * limit);
            }
        }
        return (List<StdDataSetModel>)sessionQuery.list() ;
    }

    public DetailModelResult getDetailModelList(String stdVersion, String condition, String order, Integer rows, Integer page) {
        try {
            List<StdDataSetModel> dataSetModelList = getDatasetList(StdDataSetModel.class, stdVersion, condition, order, rows, page);
            List<StdDataSetModel> totalList = getDatasetList(StdDataSetModel.class, stdVersion, condition, order, null, null);
            Integer count = totalList.size();
//            Integer count = getDatasetInt(StdDataSetModel.class, stdVersion, condition);
            DetailModelResult result = DetailModelResult.success("获取标准数据集列表成功");
            result.setDetailModelList(dataSetModelList);
            result.setTotalCount(count);
            return result;
        } catch (Exception e) {
            return DetailModelResult.error("获取标准数据集列表失败");
        }
    }

    public List getDatasetList(Class tClass, String stdVersion, String condition, String order, Integer limit, Integer offset) throws IOException {
        StandardVersion standardVersion = new StandardVersion(stdVersion);
        String tableName = standardVersion.getDataSetTableName();
        SqlCreator sqlCreator = new SqlCreator(tClass);
        if (!StringUtil.isEmpty(condition)) {
            JsonNode jsonNode = objectMapper.readTree(condition);
            if(jsonNode.get("initVal")!=null && !jsonNode.get("initVal").asText().isEmpty()){
                String initName = jsonNode.get("initVal").asText();
                sqlCreator.neCondition("code",initName);
            }else if(jsonNode.get("name")!=null && !jsonNode.get("name").asText().isEmpty()){
                String name = jsonNode.get("name").asText();
                sqlCreator.likeOrCondition("code", "name", name);
            } else if (jsonNode.get("stdId") != null && !jsonNode.get("stdId").asText().isEmpty()) {
                String stdIds = jsonNode.get("stdId").asText();
                sqlCreator.inCondition("id", stdIds);
            }
        }
        if (!StringUtil.isEmpty(order)) {
            JsonNode jsonNode = objectMapper.readTree(order);
            Iterator<String> fieldNames = jsonNode.fieldNames();
            while (fieldNames.hasNext()) {
                String fieldName = fieldNames.next();
                String value = jsonNode.get(fieldName).asText().toUpperCase();
                sqlCreator.order(fieldName, value);
            }
        }
        String sql = sqlCreator.selectData(tableName);
        Query query = getQuery(sqlCreator, sql);
        if (limit != null) {
            query.setMaxResults(limit);
            if (offset != null) {
                query.setFirstResult((offset - 1) * limit);
            }
        }
        return query.list();
    }

    public Integer getDatasetInt(Class tClass, String stdVersion, String condition) {
        try {
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            String tableName = standardVersion.getDataSetTableName();
            SqlCreator sqlCreator = new SqlCreator(tClass);
            if (!StringUtil.isEmpty(condition)) {
                JsonNode jsonNode = objectMapper.readTree(condition);
                String name = jsonNode.get("name").asText();
                sqlCreator.likeOrCondition("code", "name", name);
            }
            String sql = sqlCreator.countData(tableName);
            Query query = getCurrentSession().createSQLQuery(sql);
            for (String key : sqlCreator.getKeyValueMap().keySet()) {
                query.setParameter(key, sqlCreator.getKeyValueMap().get(key));
            }
            Integer count = Integer.parseInt(StringUtil.toString(query.list().get(0)));
            return count;
        } catch (Exception e) {
            e.printStackTrace();
            throw new ApiException(ErrorCode.GetDataSetListFailed);
        }
    }

    public DetailModelResult getDatasetSelectList(String stdVersion, String condition, String order, Integer limit, Integer offset) {
        try {
            List<StdDataSetModel> datasetList = getDatasetList(StdDataSetModel.class, stdVersion, condition, order, limit, offset);
            List<Select2> detailModelList = new ArrayList<>();
            for (StdDataSetModel stdDataSetModel : datasetList) {
                Select2 select2 = new Select2();
                select2.setId(stdDataSetModel.getId() + SqlConstants.COMMA + stdDataSetModel.getCode());
                select2.setText(stdDataSetModel.getName());
                detailModelList.add(select2);
            }
            DetailModelResult detailModelResult = DetailModelResult.success("获取数据集下拉列表成功");
            detailModelResult.setDetailModelList(detailModelList);
            detailModelResult.setTotalCount(detailModelList.size());
            return detailModelResult;
        } catch (Exception e) {
            return DetailModelResult.error("获取数据集下拉列表失败");
        }
    }

    public List getAllVersionDatasetList() {
        // 导入的数据集列表应该只取集成标准已发布各版本的并集去重 @modify by yingjie chen
        Criteria criteria = getCurrentSession().createCriteria(StandardModel.class);
        StandardParamResultModel standardParam = standardService.getStandardParam(SqlConstants.STANDARD);
        String code = standardParam.getCode();
        criteria.add(Restrictions.eq("code", code));
        List<StandardModel> standardModelList = criteria.list();

        List<Long> idList = standardModelList.stream().map(StandardModel::getId).collect(Collectors.toList());
        try {
            List<STDVersion> versionModelList = new ArrayList<>();
            if (!CollectionUtil.isEmpty(idList)) {
                String hql = "FROM STDVersion WHERE commitTime is not null AND standardId IN :idList order by commitTime";
                Query query = getCurrentSession().createQuery(hql);
                query.setParameterList("idList", idList);
                versionModelList = query.list();
            }
            String sql = SqlConstants.EMPTY;
            BeanTransformer transformer = new BeanTransformer(StdDataSetModel.class);
            for (STDVersion versionModel : versionModelList) {
                SqlCreator sqlCreator = new SqlCreator(StdDataSetModel.class);
                StandardVersion version = new StandardVersion(versionModel.getVersion());
                sql = sql + sqlCreator.selectData(version.getDataSetTableName()) + SqlConstants.UNION;
            }

            if (!StringUtil.isEmpty(sql)) {
                sql = sql.substring(0, sql.length() - SqlConstants.UNION.length());
                SQLQuery sqlQuery = getCurrentSession().createSQLQuery(sql);
                sqlQuery.setResultTransformer(transformer);
                List<StdDataSetModel> modelList = sqlQuery.list();
                List<StdDataSetModel> distinctModelList = new ArrayList<>();
                Map<String, StdDataSetModel> datasetModelMap = new HashMap<>();
                for (StdDataSetModel model : modelList) {
                    datasetModelMap.put(model.getCode(), model);
                }
                for (String key : datasetModelMap.keySet()) {
                    distinctModelList.add(datasetModelMap.get(key));
                }
                return distinctModelList;
            } else {
                return new ArrayList<>();
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new ApiException(ErrorCode.GetDataSetListFailed);
        }
    }

    public List datasetVersionListByName(String rsResourceImportDatasetDialogName) {

        try {
            String hql = "FROM STDVersion WHERE commitTime is not null order by commitTime";
            Query query = getCurrentSession().createQuery(hql);
            List<STDVersion> versionModelList = query.list();
            String sql = SqlConstants.EMPTY;
            BeanTransformer transformer = new BeanTransformer(StdDataSetModel.class);
            for (STDVersion versionModel : versionModelList) {
                SqlCreator sqlCreator = new SqlCreator(StdDataSetModel.class);
                StandardVersion version = new StandardVersion(versionModel.getVersion());
                sql = sql + sqlCreator.selectData(version.getDataSetTableName())  + SqlConstants.UNION;
            }

            if (!StringUtil.isEmpty(sql)) {
                sql = sql.substring(0, sql.length() - SqlConstants.UNION.length());
                SQLQuery sqlQuery = getCurrentSession().createSQLQuery(sql);
                sqlQuery.setResultTransformer(transformer);
                List<StdDataSetModel> modelList = sqlQuery.list();
                List<StdDataSetModel> distinctModelList = new ArrayList<>();
                Map<String, StdDataSetModel> datasetModelMap = new HashMap<>();
                for (StdDataSetModel model : modelList) {
                    datasetModelMap.put(model.getCode(), model);
                }
                for (String key : datasetModelMap.keySet()) {
                    StdDataSetModel s=datasetModelMap.get(key);
                    if(s.getName().contains(rsResourceImportDatasetDialogName)){
                        distinctModelList.add(datasetModelMap.get(key));
                    }
                }
                return distinctModelList;
            } else {
                return new ArrayList<>();
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new ApiException(ErrorCode.GetDataSetListFailed);
        }
    }


    /* ehr 方法，有重复方法就去除*/
    public String getTaleName(String version) {
        return CDAVersionUtil.getDataSetTableName(version);
    }


    @Transactional(propagation = Propagation.REQUIRED)
    public boolean add(StdDataSetModel dataSet, String version) {
        //这里涉及到动态查询和对象拷贝，导致效率不高，现在用原始的jdbc查询
        String sql =
                "INSERT INTO " + getTaleName(version) +
                        "(code, name, ref_standard, std_version, summary, hash, document_id, lang, catalog, publisher) " +
                        "VALUES (?,?,?,?,?,?,?,?,?,?)";

        KeyHolder keyHolder = new GeneratedKeyHolder();
        jdbcTemplate.update(
                con -> {
                    PreparedStatement ps = jdbcTemplate.getDataSource()
                            .getConnection().prepareStatement(sql,
                                    new String[]{"id","code","name","version","summary","hashCode", "lang","catalog","publisher"});
                    ps.setString(1, dataSet.getCode());
                    ps.setString(2, dataSet.getName());
                    ps.setString(3, dataSet.getCode());
                    ps.setString(4, dataSet.getStdVersion());
                    ps.setInt(5, dataSet.getHashCode());
                    ps.setInt(6, dataSet.getLang());
                    ps.setInt(7, dataSet.getCatalog());
                    ps.setInt(8,dataSet.getPublisher());
                    return ps;
                }, keyHolder);
        System.out.println("自动插入id============================" + keyHolder.getKey().intValue());
        String key = keyHolder.getKey().toString();


        return true;
//        return result > 0;
    }


    @Transactional(propagation = Propagation.SUPPORTS)
    public Map<Long, String> getDataSetMapByIds(Object[] ids, String versionid) {
        Session session = currentSession();
        String dataSetTable = CDAVersionUtil.getDataSetTableName(versionid);
        String sql = "select id, name from " + dataSetTable;
        if (ids.length > 0)
            sql += " where id in(:ids) ";
        Query query = session.createSQLQuery(sql);
        if (ids.length > 0)
            query.setParameterList("ids", ids);
        List<Object> records = query.list();
        Map<Long, String> rs = new HashMap<>();
        for (int i = 0; i < records.size(); ++i) {
            Object[] record = (Object[]) records.get(i);
            rs.put(((Integer) record[0]).longValue(), (String) record[1]);
        }
        return rs;
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public void batchInsertDictsAndEntry(List<Map<String, Object>> models, String version) throws Exception {

        String sql = "INSERT INTO "+ CDAVersionUtil.getMetaDataTableName(version) +
                "(dataset_id, code, inner_code, name, type, format, dict_id, definition,nullable, column_type, " +
                "column_name, column_length, primary_key,std_id,std_version, hash) VALUES ";
        int i = 1;
        int j = 0;
        StringBuilder sb = new StringBuilder(sql);
        String tmp;
        int nullable;
        int primaryKey;
        for(Map<String, Object> map: models){
            long dataSetId = add(map, version);
            for(Map<String, Object> meta: (List<Map<String, Object>>) map.get("children")){
                sb.append("('"+ dataSetId +"',");
                sb.append("'"+ meta.get("code") +"',");
                sb.append("'"+ meta.get("innerCode") +"',");
                sb.append("'"+ meta.get("name") +"',");
                sb.append("'"+ meta.get("type") +"',");
                sb.append("'"+ meta.get("format") +"',");
                sb.append("'"+ (StringUtils.isEmpty((String) meta.get("dictId")) ? 0 : meta.get("dictId")) +"',");
                sb.append("'"+ meta.get("definition") +"',");
                tmp = (String) meta.get("nullable");
                nullable = StringUtils.isEmpty(tmp) ? 0: Integer.parseInt(tmp);
                sb.append(""+ nullable +",");
                sb.append("'"+ meta.get("columnType") +"',");
                sb.append("'"+ meta.get("columnName") +"',");
                sb.append("'"+ meta.get("columnLength") +"',");
                tmp = (String) meta.get("primaryKey");
                primaryKey = StringUtils.isEmpty(tmp) ? 0: Integer.parseInt(tmp);
                sb.append(""+ primaryKey +",");
                sb.append("'"+ meta.get("stdId") +"',");
                sb.append("'"+ meta.get("stdVersion") +"',");
                sb.append("'"+ getMetaHash(meta, dataSetId, nullable, primaryKey) +"')");
                j = 1;
                if(i%100==0){
                    currentSession().createSQLQuery(sb.toString()).executeUpdate();
                    sb = new StringBuilder(sql) ;
                    j = 0;
                }
                else
                    sb.append(",");
                i++;
            }
        }

        if(j==1)
            currentSession().createSQLQuery(sb.substring(0, sb.length()-1)).executeUpdate();
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public long add(Map<String, Object> dataSet, String version) throws Exception {
        StdDataSetModel data = new StdDataSetModel();
        data.setName(String.valueOf(dataSet.get("name")));
        data.setCode(String.valueOf(dataSet.get("code")));
        data.setStdVersion(version);
        data.setSummary(String.valueOf(dataSet.get("summary")));

        StandardVersion standardVersion = new StandardVersion(version);

        this.add(standardVersion.getDataSetTableName(),data);
        return data.getId();
        //这里涉及到动态查询和对象拷贝，导致效率不高，现在用原始的jdbc查询
//        String sql =
//                "INSERT INTO " + getTaleName(version) +
//                        "(code, name, ref_standard, std_version, summary, hash, document_id, lang, catalog, publisher) " +
//                        "VALUES (?,?,?,?,?,?,?,?,?,?)";
//
//        KeyHolder keyHolder = new GeneratedKeyHolder();
//        jdbcTemplate.update(
//                con -> {
//                    PreparedStatement ps = jdbcTemplate.getDataSource()
//                            .getConnection().prepareStatement(sql,
//                                    new String[]{"id","code","name","refStandard","version","summary","hashCode","documentId", "lang","catalog","publisher"});
//                    ps.setString(1, String.valueOf(dataSet.get("code")));
//                    ps.setString(2, String.valueOf(dataSet.get("name")));
//                    ps.setString(3, String.valueOf(dataSet.get("referenceId")));
//                    ps.setString(4, version);
//                    ps.setString(5, String.valueOf(dataSet.get("summary")));
//                    ps.setInt(6, getDataHash(dataSet, version));
//                    ps.setLong(7, 0);
//                    ps.setInt(8, 0);
//                    ps.setInt(9, 0);
//                    ps.setInt(10, 0);
//                    return ps;
//                }, keyHolder);
//        return keyHolder.getKey().intValue();
    }

    public int getDataHash(Map<String, Object> model, String version) {
        return Objects.hash(0, 0, 0, model.get("referenceId"),
                0, version, model.get("code"), model.get("name"), model.get("summary"));
    }

    public int getMetaHash(Map<String, Object> model, long dataSetId, int nullable, int primaryKey) {
        return  Objects.hash(dataSetId, model.get("dict_id"), model.get("code"), model.get("definition"), model.get("format"), model.get("inner_code"),
                model.get("name"),model.get("type"),model.get("columnLength"),model.get("columnType"), nullable, primaryKey);
    }

    /**
     * 通过版本号获取standardId
     * @param version
     * @return
     */
    private Long getByVersion(String version){
        String sql = "FROM STDVersion WHERE version = :version";
        Query query = getCurrentSession().createQuery(sql);
        query.setParameter("version", version);
        List<STDVersion> list = query.list();
        if(list==null || list.size()==0){
            return null;
        }
        return list.get(0).getStandardId();
    }

    public List<Map<String, Object>> getEhrAdapterMetadata(String version) {
        String sql = "SELECT  " +
                " sds. CODE dataSetCode,  " +
                " smd.inner_code metadaCode,  " +
                " smd. NAME metadaName " +
                "FROM  " +
                " std_metadata_"+version+" smd  " +
                "LEFT JOIN" +
                " std_dataset_"+version+" sds  " +
                "ON  " +
                " smd.dataset_id = sds.id";
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        return list;
    }

    public String getInsertSql(StandardVersion standardVersion, StdDataSetModel stdeDataset, String schema)throws IOException {
        SqlCreator sqlCreator = new SqlCreator(StdDataSetModel.class);
        JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(stdeDataset));
        String sql = sqlCreator.insertDataToSQL(standardVersion.getDataSetTableName(), jsonNode);
        if (org.apache.commons.lang3.StringUtils.isNotEmpty(schema)) {
            String myCatAnnotation = "/*#mycat:schema=" + schema + "*/ ";
            sql = myCatAnnotation + sql;
        }
        return sql;
    }

    public Long getIdByCode(String version, String code, String schema) {
        StandardVersion standardVersion = new StandardVersion(version);
        String searchSql = "select id from "+ standardVersion.getDataSetTableName() +" d where d.code=? ";
        String myCatAnnotation = "/*#mycat:schema=" + schema + "*/ ";
        searchSql = myCatAnnotation + searchSql;

        List<Long> ids = jdbcTemplate.queryForList(searchSql, Long.class, code);
        if(ids!=null && ids.size()>0){
            return ids.get(0);
        }
        return null;
    }
}

