package com.yihu.hos.standard.service.standard;


import com.fasterxml.jackson.databind.JsonNode;
import com.yihu.hos.core.datatype.CollectionUtil;
import com.yihu.hos.core.datatype.StringUtil;
import com.yihu.hos.standard.model.Select2;
import com.yihu.hos.standard.model.standard.STDVersion;
import com.yihu.hos.standard.model.standard.StdDictionaryEntryModel;
import com.yihu.hos.standard.model.standard.StdDictionaryModel;
import com.yihu.hos.standard.util.CDAVersionUtil;
import com.yihu.hos.web.framework.constant.ErrorCode;
import com.yihu.hos.web.framework.constant.SqlConstants;
import com.yihu.hos.web.framework.dao.SQLGeneralDAO;
import com.yihu.hos.web.framework.exception.ApiException;
import com.yihu.hos.web.framework.model.DetailModelResult;
import com.yihu.hos.web.framework.model.DictItem;
import com.yihu.hos.web.framework.model.DictionaryResult;
import com.yihu.hos.web.framework.model.Result;
import com.yihu.hos.web.framework.model.standard.StandardVersion;
import com.yihu.hos.web.framework.util.sql.BeanTransformer;
import com.yihu.hos.web.framework.util.sql.RequestParamTransformer;
import com.yihu.hos.web.framework.util.sql.SqlCreator;
import org.hibernate.Query;
import org.hibernate.SQLQuery;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.io.IOException;
import java.sql.Connection;
import java.util.*;

import static com.yihu.hos.web.framework.model.standard.StandardVersion.DictTable;
import static com.yihu.hos.web.framework.model.standard.StandardVersion.VersionBoundSymbol;

/**
 * @author Sand
 * @version 1.0
 * @created 14-7月-2015 15:44:21
 */
@Transactional
@Service("StdDictService")
public class StdDictService extends SQLGeneralDAO {

    public static final String BEAN_ID = "StdDictService";
    @Autowired
    StdDictEntryService dictEntryService;

    public StdDictService() {
    }

    public void finalize() throws Throwable {

    }

    public List<StdDictionaryModel> getList(String stdVersion, String condition, String order, Integer limit, Integer offset) {
        try{
            Map<String, Object> queryMap = RequestParamTransformer.parseJsonToMap(condition);
            Map<String, String> orderMap = RequestParamTransformer.parseJsonToMap(order);
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            return this.getList(standardVersion, queryMap, orderMap, limit, offset);
        }catch (Exception e){
            e.printStackTrace();
        }

        throw new ApiException(ErrorCode.StandardDictGetListFailed);
    }

    public Result add(String stdVersion, String dict) {
        Boolean existFlg = false;
        try {
            StandardVersion standardVersion = new StandardVersion(stdVersion);

            StdDictionaryModel stdDictionaryModel = objectMapper.readValue(dict, StdDictionaryModel.class);
            existFlg = dictCodeValidate(standardVersion, stdDictionaryModel.getCode());
            if (!existFlg) {
                //通过stdVersion查找standardId
                Long standardId = getByVersion(stdVersion);
                stdDictionaryModel.setStandardId(standardId);
                stdDictionaryModel.setStdVersion(stdVersion);
                stdDictionaryModel.setCreateDate(new Date());
                this.add(standardVersion, stdDictionaryModel);

                //取得新增字典表记录
                Map<String, Object> query = new HashMap<>();
                Map<String, String> order = new HashMap<>();
                query.put("code", stdDictionaryModel.getCode());

                List<StdDictionaryModel> list = this.getList(standardVersion, query, order, 1, 0);
                Long id = list.get(0).getId();
                Long parentId = list.get(0).getBaseDict();
                if (parentId != null) {
                    Map<String, Object> query1 = new HashMap<>();
                    Map<String, String> order1 = new HashMap<>();
                    query1.put("dictId", parentId);
                    List<StdDictionaryEntryModel> dicEntryList = this.getDicEntryList(standardVersion,query1,order,null,null);
                    for (StdDictionaryEntryModel entryModel : dicEntryList) {
                        StdDictionaryEntryModel model =  new StdDictionaryEntryModel();
                        model.setDictId(id);
                        model.setStandardId(entryModel.getStandardId());
                        model.setCode(entryModel.getCode());
                        model.setValue(entryModel.getValue());
                        model.setDesc(entryModel.getDesc());
                        dictEntryService.add(standardVersion,model);
                    }
                }

                stdDictionaryModel.setSuccessFlg(true);
                stdDictionaryModel.setMessage("保存标准字典成功");
                return stdDictionaryModel;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (existFlg) {
            return Result.error("已存在标准字典编码");
        } else {
            return Result.error("保存标准字典失败");
        }
    }

    public Boolean dictCodeValidate(StandardVersion standardVersion, String code) {
        Map<String, Object> query = new HashMap<>();
        Map<String, String> order = new HashMap<>();
        query.put("code", code);

        List<StdDictionaryModel> list = this.getList(standardVersion, query, order, 1, 0);
        if (!CollectionUtil.isEmpty(list)) {
            return true;
        }
        return false;
    }

    public Result delete(String stdVersion, Long dictId) {
        try {
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            if (dictId != null) {
                this.delete(standardVersion, dictId);
                return Result.success("删除标准字典成功");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Result.success("删除标准字典失败");
    }

    public Boolean dictCodeValidate(StandardVersion standardVersion, Long dictId, String code) {
        SqlCreator sqlCreator = new SqlCreator(StdDictionaryModel.class);
        sqlCreator.equalCondition("code", code);
        sqlCreator.neCondition("id", dictId);
        String sql = sqlCreator.selectData(standardVersion.getDictTableName());
        Query sessionQuery = getQuery(sqlCreator, sql);
        List<StdDictionaryModel> list = sessionQuery.list();
        if (!CollectionUtil.isEmpty(list)) {
            return true;
        }
        return false;
    }

    public Result update(String stdVersion, String dict) {
        Boolean existFlg = false;
        try {
            StandardVersion standardVersion = new StandardVersion(stdVersion);

            StdDictionaryModel stdDictionaryModel = objectMapper.readValue(dict, StdDictionaryModel.class);
            existFlg = dictCodeValidate(standardVersion, stdDictionaryModel.getId(), stdDictionaryModel.getCode());
            if (!existFlg) {
                this.modify(standardVersion, stdDictionaryModel);
                stdDictionaryModel.setSuccessFlg(true);
                stdDictionaryModel.setMessage("修改标准字典成功");
                return stdDictionaryModel;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (existFlg) {
            return Result.error("已存在标准字典编码");
        } else {
            return Result.error("修改标准字典失败");
        }
    }

    public Result get(String stdVersion, Long dictId) {
        try {
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            if (dictId != null) {
                StdDictionaryModel stdDictionaryModel = this.get(standardVersion, dictId);
                if(stdDictionaryModel==null){
                    stdDictionaryModel= new StdDictionaryModel();
                }
                stdDictionaryModel.setSuccessFlg(true);
                stdDictionaryModel.setMessage("获取标准字典成功");
                return stdDictionaryModel;
            }

        } catch (Exception e) {
            e.printStackTrace();
        }

        return Result.error("获取标准字典失败");
    }

    private StdDictionaryModel get(StandardVersion standardVersion, Long dictId) {

        Session session = getCurrentSession();
        String dictTableName = standardVersion.getDictTableName();

        String sql = "select id, std_id, code, name, define, description, parent_id from " + dictTableName + " where id = :id";
        Query query = session.createSQLQuery(sql);
        query.setLong("id", dictId);
        query.setResultTransformer(new BeanTransformer<>(StdDictionaryModel.class));
        return (StdDictionaryModel) query.uniqueResult();
    }

    public StdDictionaryModel findById(String stdVersion, Long dictId) {
        Session session = getCurrentSession();
        String dictTableName = DictTable + VersionBoundSymbol + stdVersion;

        String sql = "select id, std_id, code, name, define, description, parent_id from " + dictTableName + " where id = :id";
        Query query = session.createSQLQuery(sql);
        query.setLong("id", dictId);
        query.setResultTransformer(new BeanTransformer<>(StdDictionaryModel.class));
        return (StdDictionaryModel) query.uniqueResult();
    }

    private void modify(StandardVersion standardVersion, StdDictionaryModel stdDictionaryModel) throws IOException {
        String strTableName = standardVersion.getDictTableName();
        JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(stdDictionaryModel));
        SqlCreator sqlCreator = new SqlCreator(StdDictionaryModel.class);
        String sql = sqlCreator.updateDataByTableKey(strTableName, jsonNode);
        Query query = getExeuteQuery(sqlCreator, sql);
        query.executeUpdate();
    }

    private void delete(StandardVersion standardVersion, Long dictId) {
        Session session = getCurrentSession();
        String sql = "delete from " + standardVersion.getDictTableName() + " where id = :id";
        Query query = session.createSQLQuery(sql);
        query.setLong("id", dictId);
        query.executeUpdate();
        sql = "delete from " + standardVersion.getDictEntryTableName() + " where dict_id = :dictId";
        query = session.createSQLQuery(sql);
        query.setLong("dictId", dictId);
        query.executeUpdate();
    }

    public void add(StandardVersion standardVersion, StdDictionaryModel stdDictionaryModel) throws IOException {
        Session session = getCurrentSession();
        String strTableName = standardVersion.getDictTableName();
        String sql = null;
        sql = "select max(id) from " + strTableName;

        Query query = session.createSQLQuery(sql);
        Object object = query.uniqueResult();
        Long id = object == null ? 1 : Long.parseLong(object.toString()) + 1;
        stdDictionaryModel.setId(id);

        JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(stdDictionaryModel));
        SqlCreator sqlCreator = new SqlCreator(StdDictionaryModel.class);
        sql = sqlCreator.insertData(strTableName, jsonNode);
        query = getExeuteQuery(sqlCreator, sql);
        query.executeUpdate();
    }

    private List<StdDictionaryModel> getList(StandardVersion standardVersion, Map<String, Object> query, Map<String, String> order, Integer limit, Integer offset) {
        SqlCreator sqlCreator = new SqlCreator(StdDictionaryModel.class);
        for (String key : query.keySet()) {
            sqlCreator.equalCondition(key, query.get(key));
        }
        for (String key : order.keySet()) {
            sqlCreator.order(key, order.get(key));
        }

        String sql = sqlCreator.selectData(standardVersion.getDictTableName());
        Query sessionQuery = getQuery(sqlCreator, sql);
        if (limit != null) {
            sessionQuery.setMaxResults(limit);
            if (offset != null) {
                sessionQuery.setFirstResult((offset - 1) * limit);
            }
        }
        return (List<StdDictionaryModel>) sessionQuery.list();
    }

    public List<StdDictionaryEntryModel> getDicEntryList(StandardVersion standardVersion, Map<String, Object> query, Map<String, String> order, Integer limit, Integer offset) {
        SqlCreator sqlCreator = new SqlCreator(StdDictionaryEntryModel.class);
        if(null!=query){
            for (String key : query.keySet()) {
                sqlCreator.equalCondition(key, query.get(key));
            }
        }
        if(order!=null){
            for (String key : order.keySet()) {
                sqlCreator.order(key, order.get(key));
            }
        }

        String sql = sqlCreator.selectData(standardVersion.getDictEntryTableName());
        Query sessionQuery = getQuery(sqlCreator, sql);
        if (limit != null) {
            sessionQuery.setMaxResults(limit);
            if (offset != null) {
                sessionQuery.setFirstResult((offset - 1) * limit);
            }
        }
        return (List<StdDictionaryEntryModel>) sessionQuery.list();
    }

    public List<StdDictionaryEntryModel> getDicEntryList(String schemaName,StandardVersion standardVersion, Map<String, Object> query, Map<String, String> order, Integer limit, Integer offset) {
        SqlCreator sqlCreator = new SqlCreator(StdDictionaryEntryModel.class);
        if(null!=query){
            for (String key : query.keySet()) {
                sqlCreator.equalCondition(key, query.get(key));
            }
        }
        if(order!=null){
            for (String key : order.keySet()) {
                sqlCreator.order(key, order.get(key));
            }
        }

        String sql = sqlCreator.selectData(standardVersion.getDictEntryTableName());

        String myCatAnnotation = "/*#mycat:schema=" + schemaName + "*/ ";
        String completeSql = myCatAnnotation + sql;

        Query sessionQuery = getQuery(sqlCreator, completeSql);
        if (limit != null) {
            sessionQuery.setMaxResults(limit);
            if (offset != null) {
                sessionQuery.setFirstResult((offset - 1) * limit);
            }
        }
        return (List<StdDictionaryEntryModel>) sessionQuery.list();
    }

    public DictionaryResult getDictionaryList(String stdVersion, String query, String order, Integer rows, Integer page) {
        try {
            List<StdDictionaryModel> stdDictionaryModelList = getList(stdVersion, query, order, rows, page);
            List<DictItem> detailModelList = new ArrayList<DictItem>();
            DictItem item = new DictItem();
            item.setCode("");
            item.setValue("-无-");
            detailModelList.add(item);
            for (StdDictionaryModel stdDictionaryModel : stdDictionaryModelList) {
                DictItem items = new DictItem();
                Long  id = stdDictionaryModel.getId();
                String name = stdDictionaryModel.getName();
                items.setCode(id.toString());
                items.setValue(name);
                detailModelList.add(items);
            }
            DictionaryResult dictionaryResult = DictionaryResult.success("获取字典列表成功");
            dictionaryResult.setDetailModelList(detailModelList);
            return dictionaryResult;
        } catch (Exception e) {
            return DictionaryResult.error("获取字典列表失败");
        }
    }

    public DetailModelResult getDetailModelList(String stdVersion, String condition, String order, Integer rows, Integer page) {
        try {
            List<StdDictionaryModel> dictModelList = getDictList(StdDictionaryModel.class, stdVersion, condition, order, rows, page);
            Integer count = getDictInt(StdDictionaryModel.class, stdVersion, condition);
            DetailModelResult result = DetailModelResult.success("获取标准数据字典列表成功");
            result.setDetailModelList(dictModelList);
            result.setTotalCount(count);
            return result;
        } catch (Exception e) {
            return DetailModelResult.error("获取标准字典列表失败");
        }
    }

    public List getDictList(Class tClass, String stdVersion, String condition, String order, Integer limit, Integer offset) {
        StandardVersion standardVersion = new StandardVersion(stdVersion);
        String tableName = standardVersion.getDictTableName();
        SqlCreator sqlCreator = new SqlCreator(tClass);
        try {
            if (!StringUtil.isEmpty(condition)) {
                JsonNode jsonNode = objectMapper.readTree(condition);
                if(jsonNode.get("initVal")!=null && !jsonNode.get("initVal").asText().isEmpty()){
                    String initName = jsonNode.get("initVal").asText();
                    sqlCreator.equalCondition("name",initName);
                }else if(jsonNode.get("name")!=null && !jsonNode.get("name").asText().isEmpty()){
                    String name = jsonNode.get("name").asText();
                    sqlCreator.likeOrCondition("code", "name", name);
                }
            }
            if (!StringUtil.isEmpty(order)) {
                JsonNode jsonNode = objectMapper.readTree(order);
                Iterator<String> fieldNames = jsonNode.fieldNames();
                while (fieldNames.hasNext()) {
                    String fieldName = fieldNames.next();
                    String value = jsonNode.get(fieldName).asText().toUpperCase();
                    sqlCreator.order(fieldName, value);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        String sql = sqlCreator.selectData(tableName);
        Query query = getQuery(sqlCreator, sql);
        if (limit != null) {
            query.setMaxResults(limit);
            if (offset != null) {
                query.setFirstResult((offset - 1) * limit);
            }
        }

        return query.list();
    }

    public Integer getDictInt(Class tClass, String stdVersion, String condition) {
        StandardVersion standardVersion = new StandardVersion(stdVersion);
        String tableName = standardVersion.getDictTableName();
        SqlCreator sqlCreator = new SqlCreator(tClass);
        try {
            if (!StringUtil.isEmpty(condition)) {
                JsonNode jsonNode = objectMapper.readTree(condition);
                String name = jsonNode.get("name").asText();
                sqlCreator.likeOrCondition("code", "name", name);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        String sql = sqlCreator.countData(tableName);
        Query query = getCurrentSession().createSQLQuery(sql);
        for (String key : sqlCreator.getKeyValueMap().keySet()) {
            query.setParameter(key, sqlCreator.getKeyValueMap().get(key));
        }
        Integer count = Integer.parseInt(StringUtil.toString(query.list().get(0)));
        return count;
    }

    public DetailModelResult getDictSelectList(List<StdDictionaryModel> dictList) {
        try {
            List<Select2> detailModelList = new ArrayList<>();
            for (StdDictionaryModel dictModel : dictList) {
                Select2 select2 = new Select2();
                select2.setId(dictModel.getId() + SqlConstants.COMMA + dictModel.getCode());
                select2.setText(dictModel.getName());
                detailModelList.add(select2);
            }
            DetailModelResult detailModelResult = DetailModelResult.success("获取字典下来列表成功");
            detailModelResult.setDetailModelList(detailModelList);
            detailModelResult.setTotalCount(detailModelList.size());
            return detailModelResult;
        } catch (Exception e) {
            return DetailModelResult.error("获取字典下来列表失败");
        }
    }

    public StdDictionaryModel getAdapterDictByCode(String version, String dictCode) {
        try {
            SqlCreator sqlCreator = new SqlCreator(StdDictionaryModel.class);
            sqlCreator.equalCondition("code", dictCode);
            String sql = sqlCreator.selectData("std_dictentry_"+version);
            Query query = getQuery(sqlCreator, sql);
            return (StdDictionaryModel)query.uniqueResult();
        } catch (Exception e) {
            return null;
        }
    }

    /* 以下为ehr迁移的方法 */

    public String getTaleName(String version){

        return CDAVersionUtil.getDictTableName(version);
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public StdDictionaryModel getByCode(String code, String version){
        StandardVersion standardVersion = new StandardVersion(version);
        String hql = "select e from " + standardVersion.getDictTableName() + " e where e.code=:code";

        Query query = currentSession().createQuery(hql);
        query.setParameter("code", code);
        List<StdDictionaryModel> ls = query.list();
        return ls.size()>0? ls.get(0) : null;
    }

    public Map getDictMapByIds(String version, Long dataSetId, Long metaDataId) {
        Session session = currentSession();
        String dictTableName = CDAVersionUtil.getDictTableName(version);
        String metaTable = CDAVersionUtil.getMetaDataTableName(version);

        String sql =
                " select d.id as dictId, t.id as metaId, t.dataset_id from " + metaTable + " t " +
                        " left join " + dictTableName + " d on t.dict_id=d.id " +
                        " where d.id is not null and t.id = :metaDataId and t.dataset_id = :dataSetId ";

        Query query = session.createSQLQuery(sql);
        query.setParameter("metaDataId", metaDataId);
        query.setParameter("dataSetId", dataSetId);
        Object o = query.uniqueResult();
        Map map = null;
        if (o != null) {
            Object[] os = ((Object[]) o);
            map = new HashMap<>();
            map.put("dictId", os[0]);
            map.put("metaId", os[1]);
            map.put("datasetId", os[2]);
        }
        return map;
    }


    public List<StdDictionaryModel> getChildrensByParentId(long baseDictId, String version) {
        Session session = currentSession();
        StandardVersion standardVersion = new StandardVersion(version);
        String sql="";
        if(StringUtils.isEmpty(baseDictId)){
            sql += "select a.* from "+standardVersion.getDictTableName()+" a where 1=1 and (a.parent_id is null or a.parent_id='')";
        }else{
            sql += "select a.* from "+standardVersion.getDictTableName()+" a where 1=1 and a.parent_id =:parent_id";
        }
        Query query = session.createSQLQuery(sql);
        if(!StringUtils.isEmpty(baseDictId)){
            query.setLong("parent_id", baseDictId);
        }
        return query.list();
    }

    public List<StdDictionaryModel> getCdaTypeExcludeSelfAndChildren(String childrenIds, String version) {
        Session session = currentSession();
        StandardVersion standardVersion = new StandardVersion(version);
        String[] ids = childrenIds.split(",");

        Long[] idsL = new Long[ids.length];
        for (int i = 0; i < ids.length; i++) {
            idsL[i] = Long.parseLong(ids[i]);
        }

        String  hql = "select a.* from "+standardVersion.getDictTableName() +" a where 1=1 and a.id not in (:ids)";
        Query query = session.createSQLQuery(hql);
        query.setParameterList("ids",idsL);
        return query.list();
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public void batchInsertDictsAndEntry(List<Map<String, Object>> models,String version) throws Exception {
        StandardVersion standardVersion = new StandardVersion(version);
        String dictEntryTableName = CDAVersionUtil.getDictEntryTableName(version);
        Session session = null;
        Transaction transaction =null;
        Connection connection = null;
        try {
            List<Map<String, Object>> dictEntry = new ArrayList<>();
            //插入字典

            Map<String,StdDictionaryModel> hashMap = new HashMap<>();
            for(Map<String, Object> map: models){
                StdDictionaryModel dict =StdDictionaryModel.class.newInstance();
                dict.setStandardId(Long.parseLong(map.get("stdId")+""));
                dict.setCode(map.get("code")+"");
                dict.setName(map.get("name")+"");
                dict.setDescription(null2Space(map.get("description"))+"");
                dict.setStdVersion(version);
                dict.setCreateDate(new Date());
                dict.setHashCode((map.get("code") + "").hashCode());
                this.add(standardVersion,dict);
                hashMap.put(map.get("code") + "", dict);
                if(map.get("children")==null){
                    continue;
                }else{
                    dictEntry.addAll((List)map.get("children"));
                }
            }
            /**
             * 打破了完整性
             */
            //插入字典项
            session = currentSession().getSessionFactory().openSession();
            transaction = session.beginTransaction();

            String  title = "INSERT INTO "+dictEntryTableName+"(std_id,std_version,dict_id, code, value, description,hash) VALUES ";
            StringBuilder sql = new StringBuilder(title);
            int i = 1;
            StdDictionaryModel dict = null;
            for(Map<String, Object> map: dictEntry){
                if(dict==null||!map.get("dictCode").equals(dict.getCode())){
                    dict = hashMap.get(map.get("dictCode")+"");
                }
                String name =  map.get("name")+"";
                if(name.indexOf("'")>0){
                    name = name.replaceAll("'","\'");
                }
                sql.append("('" + map.get("stdId") +"'");
                sql.append(",'" + map.get("stdVersion") +"'");
                sql.append(",'" + dict.getId() +"'");
                sql.append(",'" + map.get("code") +"'");
                sql.append(",'" + name +"'");
                sql.append(",'"+  null2Space(map.get("description")) +"'");
                sql.append(",'"+  (map.get("code")+"").hashCode() +"')");
                if(i%100==0 || i == dictEntry.size()){
                    session.createSQLQuery(sql.toString()).executeUpdate();
                    sql = new StringBuilder(title) ;
                }
                else
                    sql.append(",");
                i++;
            }
            transaction.commit();
            session.close();
            //更新字典项关联字典字段
            //  sql = new StringBuilder("UPDATE "+dictEntryTableName+" e,"+dictTableName+" d SET e.dict_id =d.id WHERE e.dict_id=d.code AND (e.dict_id=0 OR e.dict_id IS NULL)");
            //jdbcTemplate.execute(sql.toString());
        } catch (Exception e){
            if(transaction!=null)transaction.rollback();
            if(session!=null) session.close();
            if(connection!=null)connection.close();
            throw e;
        }
    }
    public Object null2Space(Object str){
        return str==null? "" : str;
    }

    /**
     * 查询编码是否已存在， 返回已存在资源标准编码
     */
    public List codeExist(String[] codes,String version) {
        String dictTableName = CDAVersionUtil.getDictTableName(version);
        String sql = "SELECT DISTINCT code FROM "+dictTableName+" WHERE code in(:codes)";
        SQLQuery sqlQuery = currentSession().createSQLQuery(sql);
        sqlQuery.setParameterList("codes", codes);
        return sqlQuery.list();
    }

    /**
     * 通过版本号获取standardId
     * @param version
     * @return
     */
    private Long getByVersion(String version){
        String sql = "FROM STDVersion WHERE version = :version";
        Query query = getCurrentSession().createQuery(sql);
        query.setParameter("version", version);
        List<STDVersion> list = query.list();
        if(list==null || list.size()==0){
            return null;
        }
        return list.get(0).getStandardId();
    }

    public List<Map<String, Object>> getEhrAdapterDict(String version) {
        String sql =" SELECT " +
                " sd. CODE dictCode, " +
                " sde. CODE dictEntryCode, " +
                " sde. " +
                "VALUE dictEntryName " +
                " " +
                "FROM " +
                " std_dictionary_entry_"+version+" sde " +
                "LEFT JOIN" +
                " std_dictionary_"+version+" sd " +
                "ON " +
                " sde.dict_id = sd.id where sd.code is not null ";
        List<Map<String, Object>> list = jdbcTemplate.queryForList(sql);
        return list;

    }

    public String getInsertSql(StandardVersion standardVersion, StdDictionaryModel stdeDict, String schema) throws IOException {
        SqlCreator sqlCreator = new SqlCreator(StdDictionaryModel.class);
        JsonNode jsonNode = objectMapper.readTree(objectMapper.writeValueAsString(stdeDict));
        String sql = sqlCreator.insertDataToSQL(standardVersion.getDictTableName(), jsonNode);
        if (org.apache.commons.lang3.StringUtils.isNotEmpty(schema)) {
            String myCatAnnotation = "/*#mycat:schema=" + schema + "*/ ";
            sql = myCatAnnotation + sql;
        }
        return sql;
    }

    public Long getIdByCode(String version,String code,String schema){
        StandardVersion standardVersion = new StandardVersion(version);
        String searchSql = "select id from "+ standardVersion.getDictTableName() +" d where d.code=? ";
        String myCatAnnotation = "/*#mycat:schema=" + schema + "*/ ";
        searchSql = myCatAnnotation + searchSql;

        List<Long> ids = jdbcTemplate.queryForList(searchSql, Long.class, code);
        if(ids!=null && ids.size()>0){
            return ids.get(0);
        }
        return null;
    }

}
