package com.yihu.hos.system.controller;

import com.yihu.hos.core.constants.MicroServices;
import com.yihu.hos.standard.model.adapter.AdapterSchemeVersionModel;
import com.yihu.hos.standard.model.standard.STDVersion;
import com.yihu.hos.standard.service.adapter.AdapterSchemeVersionService;
import com.yihu.hos.standard.service.standard.STDVersionService;
import com.yihu.hos.system.model.SyncMessage;
import com.yihu.hos.system.model.SyncStandardMessage;
import com.yihu.hos.system.service.DataSyncService;
import com.yihu.hos.web.framework.util.controller.BaseController;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.text.DateFormat;
import java.util.Date;
import java.util.List;


@RestController("DataSyncEndPoint")
@RequestMapping(value = MicroServices.ApiVersion.Version1_0+"/dataSync")
public class DataSyncEndPoint extends BaseController {

    @Autowired
    private DataSyncService dataSyncService;
    @Autowired
    private STDVersionService stdVersionService;
    @Autowired
    private AdapterSchemeVersionService adapterSchemeVersionService;
    @Value("${hos.adminOutUrl}")
    private String url;

    /**
     *
     * @param type  同步类型
     *  type=sync   同步所有
     *  type=standard  同步标准(stdVersion 必须)
     *  type=adapter  同步适配(adapterVersion必须)
     *  type=system  同步系统配置
     *  type=job  同步任务
     *  @param stdVersion  标准版本
     *  @param adapterVersion  适配版本
     * @return
     */
    @RequestMapping(value="/sync", method = RequestMethod.GET)
    @ApiOperation(value = "数据同步",  produces = "application/json", notes = "数据同步")
    public boolean sync(@ApiParam(name = "type", value = "同步类型")
                           @RequestParam(value = "type") String type,
                            @ApiParam(name = "stdVersion", value = "标准版本")
                            @RequestParam(value = "stdVersion" ,required = false) String stdVersion,
                            @ApiParam(name = "adapterVersion", value = "适配版本")
                            @RequestParam(value = "adapterVersion",required = false) String adapterVersion,
                            HttpServletRequest request) throws Exception {

        boolean b = false;
        if("sync".equals(type) || "system".equals(type) || "job".equals(type)){
            Date date = new Date();
            DateFormat df = DateFormat.getDateTimeInstance();
            SyncMessage syncMessage  = new SyncMessage();
            syncMessage.setLastUpdate(df.format(date));
            syncMessage.setUrl(url);
            b = dataSyncService.sync(type, syncMessage);
        }else if("standard".equals(type)){
            if(StringUtils.isBlank(stdVersion)){
                throw new Exception("stdVersion 适配版本号不能为空");
            }
            List<STDVersion> versions = stdVersionService.search(null, "version=" + stdVersion+";inStage=false", null, null, null);
            if(CollectionUtils.isEmpty(versions)){
                throw new Exception("该版本号为空,或者还未发布");
            }
            Date commitTime = versions.get(0).getCommitTime();
            DateFormat df = DateFormat.getDateTimeInstance();
            SyncStandardMessage syncMessage  = new SyncStandardMessage();
            syncMessage.setLastUpdate(df.format(commitTime));
            syncMessage.setUrl(url);
            syncMessage.setVersion(stdVersion);
            b = dataSyncService.sync(type, syncMessage);
        }else if("adapter".equals(type)) {
            if(StringUtils.isBlank(adapterVersion)){
                throw new Exception("adapterVersion 标准版本号不能为空");
            }
            List<AdapterSchemeVersionModel> versions = adapterSchemeVersionService.search(null, "version=" + adapterVersion+";publishStatus=1", null, null, null);
            if(CollectionUtils.isEmpty(versions)){
                throw new Exception("该版本号为空,或者还未发布");
            }
            Date publishTime = versions.get(0).getPublishTime();
            DateFormat df = DateFormat.getDateTimeInstance();
            SyncStandardMessage syncMessage  = new SyncStandardMessage();
            syncMessage.setLastUpdate(df.format(publishTime));
            syncMessage.setUrl(url);
            syncMessage.setVersion(adapterVersion);
            b = dataSyncService.sync(type, syncMessage);
        }
        return b;
    }

    //@RequestMapping(value="/test", method = RequestMethod.GET)
    //@ApiOperation(value = "测试",  produces = "application/json", notes = "数据同步")
    //public void test() throws IOException, InterruptedException {
    //    Broker broker = new Broker("192.168.131.144:9020"); //!!!Share it in Application!!!
    //
    //    broker = new Broker("192.168.6.109:9020");
    //    Producer p = new Producer(broker);
    //    p.declareTopic("MyTopic");
    //
    //    Message msg = new Message();
    //    msg.setTopic("MyTopic");
    //    //msg.setTag("oo.account.pp");  //set tag for Consumer's filter
    //    msg.setBody("hello " + System.currentTimeMillis());
    //    Message res = p.publish(msg);
    //    System.out.println(res);
    //    broker.close();
    //
    //}

}
