package com.yihu.hos.system.controller;

import com.yihu.hos.core.constants.MicroServices;
import com.yihu.hos.model.standard.MConventionalDict;
import com.yihu.hos.system.model.SystemDict;
import com.yihu.hos.system.service.DictManager;
import com.yihu.hos.web.framework.util.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * 惯用字典接口，用于快速提取常用的字典项。  提供给ehr
 */
@RestController
@RequestMapping(MicroServices.ApiVersion.Version1_0)
@Api(value = "EIP-EHR-Dictionaries", description = "EHR调用EIP.获取常用字典项", tags = {"惯用字典"})
public class EsbDictEndPoint extends BaseController {

    @Resource(name = DictManager.BEAN_ID)
    private DictManager dictManager;

    @RequestMapping(value = "/dictionaries/app_catalog", method = RequestMethod.GET)
    @ApiOperation(value = "获取应用类别字典项", response = MConventionalDict.class)
    public MConventionalDict getAppCatalog(
            @ApiParam(name = "code", value = "字典代码", defaultValue = "")
            @RequestParam(value = "code") String code) {
        SystemDict dict = dictManager.getDictIetm("APP_TYPE", code);
        return changeToEhrDic(dict);
    }

    @RequestMapping(value = "/dictionaries/application_source", method = RequestMethod.GET)
    @ApiOperation(value = "应用来源", response = MConventionalDict.class)
    public MConventionalDict getApplicationSource(
            @ApiParam(name = "code", value = "字典代码", defaultValue = "")
            @RequestParam(value = "code") String code){
        SystemDict dict = dictManager.getDictIetm("SOURCE_TYPE", code);
        return changeToEhrDic(dict);
    }

    @RequestMapping(value = "/dictionaries/application_source_list", method = RequestMethod.GET)
    @ApiOperation(value = "应用来源", response = MConventionalDict.class)
    public Collection<MConventionalDict> getApplicationSourceList() {
        List<SystemDict> list = dictManager.getDictEntries("SOURCE_TYPE",null);
        List<MConventionalDict> result = new ArrayList<>();
        for(SystemDict systemDict:list){
            result.add(changeToEhrDic(systemDict));
        }
        return result;
    }

    @RequestMapping(value = "/dictionaries/app_status", method = RequestMethod.GET)
    @ApiOperation(value = "获取应用状态字典项", response = MConventionalDict.class)
    public MConventionalDict getAppStatus(
            @ApiParam(name = "code", value = "字典代码", defaultValue = "")
            @RequestParam(value = "code") String code) {
        SystemDict dict = dictManager.getDictIetm("APP_STATUS", code);
        return changeToEhrDic(dict);

    }

    @RequestMapping(value = "/dictionaries/application_menu_type", method = RequestMethod.GET)
    @ApiOperation(value = "应用菜单类型", response = SystemDict.class)
    public MConventionalDict getApplicationMenuType(
            @ApiParam(name = "code", value = "字典代码", defaultValue = "")
            @RequestParam(value = "code") String code){
        SystemDict dict = dictManager.getDictIetm("APP_MENU_TYPE", code);
        return changeToEhrDic(dict);
    }

    @RequestMapping(value = "/dictionaries/application_menu_type_list", method = RequestMethod.GET)
    @ApiOperation(value = "应用菜单类型", response = MConventionalDict.class)
    Collection<MConventionalDict> getApplicationMenuTypeList(){
        List<SystemDict> list = dictManager.getDictEntries("APP_MENU_TYPE",null);
        List<MConventionalDict> result = new ArrayList<>();
        for(SystemDict systemDict:list){
            result.add(changeToEhrDic(systemDict));
        }
        return result;
    }

    @RequestMapping(value = "/dictionaries/audit_level", method = RequestMethod.GET)
    @ApiOperation(value = "审计等级", response = SystemDict.class)
    public MConventionalDict getAuditLevel(
            @ApiParam(name = "code", value = "字典代码", defaultValue = "")
            @RequestParam(value = "code") String code){
        SystemDict dict =  dictManager.getDictIetm("AUDIT_STATUS", code);
        return changeToEhrDic(dict);
    }

    @RequestMapping(value = "/dictionaries/audit_level_list", method = RequestMethod.GET)
    @ApiOperation(value = "审计等级", response = MConventionalDict.class)
    Collection<MConventionalDict> getAuditLevelList(){
        List<SystemDict> list = dictManager.getDictEntries("AUDIT_STATUS",null);
        List<MConventionalDict> result = new ArrayList<>();
        for(SystemDict systemDict:list){
            result.add(changeToEhrDic(systemDict));
        }
        return result;
    }

    @RequestMapping(value = "/dictionaries/open_level", method = RequestMethod.GET)
    @ApiOperation(value = "开放等级", response = SystemDict.class)
    public MConventionalDict getOpenLevel(
            @ApiParam(name = "code", value = "字典代码", defaultValue = "")
            @RequestParam(value = "code") String code){
        SystemDict dict =  dictManager.getDictIetm("OPEN_LEVEL", code);
        return changeToEhrDic(dict);
    }

    @RequestMapping(value = "/dictionaries/open_level_list", method = RequestMethod.GET)
    @ApiOperation(value = "开放等级", response = MConventionalDict.class)
    Collection<MConventionalDict> getOpenLevelList(){
        List<SystemDict> list = dictManager.getDictEntries("OPEN_LEVEL",null);
        List<MConventionalDict> result = new ArrayList<>();
        for(SystemDict systemDict:list){
            result.add(changeToEhrDic(systemDict));
        }
        return result;
    }

    @RequestMapping(value = "/dictionaries/api_parameter_type", method = RequestMethod.GET)
    @ApiOperation(value = "API参数类型", response = SystemDict.class)
    public MConventionalDict getApiParameterType(
            @ApiParam(name = "code", value = "字典代码", defaultValue = "")
            @RequestParam(value = "code") String code){
        SystemDict dict =  dictManager.getDictIetm("APICSLX", code);
        return changeToEhrDic(dict);
    }

    @RequestMapping(value = "/dictionaries/api_parameter_type_list", method = RequestMethod.GET)
    @ApiOperation(value = "API参数类型", response = MConventionalDict.class)
    Collection<MConventionalDict> getApiParameterTypeList(){
        List<SystemDict> list = dictManager.getDictEntries("APICSLX",null);
        List<MConventionalDict> result = new ArrayList<>();
        for(SystemDict systemDict:list){
            result.add(changeToEhrDic(systemDict));
        }
        return result;
    }

    @RequestMapping(value = "/dictionaries/api_parameter_data_type", method = RequestMethod.GET)
    @ApiOperation(value = "API参数数据类型", response = SystemDict.class)
    public MConventionalDict getApiParameterDataType(
            @ApiParam(name = "code", value = "字典代码", defaultValue = "")
            @RequestParam(value = "code") String code){
        SystemDict dict =  dictManager.getDictIetm("APICSSJLX", code);
        return changeToEhrDic(dict);
    }

    @RequestMapping(value = "/dictionaries/api_parameter_data_type_list", method = RequestMethod.GET)
    @ApiOperation(value = "API参数数据类型", response = MConventionalDict.class)
    Collection<MConventionalDict> getApiParameterDataTypeList(){
        List<SystemDict> list = dictManager.getDictEntries("APICSSJLX",null);
        List<MConventionalDict> result = new ArrayList<>();
        for(SystemDict systemDict:list){
            result.add(changeToEhrDic(systemDict));
        }
        return result;
    }

    @RequestMapping(value = "/dictionaries/api_parameter_data_required", method = RequestMethod.GET)
    @ApiOperation(value = "API参数必输标识", response = SystemDict.class)
    public MConventionalDict getApiParameterDataRequired(
            @ApiParam(name = "code", value = "字典代码", defaultValue = "")
            @RequestParam(value = "code") String code){
        SystemDict dict =  dictManager.getDictIetm("APICSBSBS", code);
        return changeToEhrDic(dict);
    }

    @RequestMapping(value = "/dictionaries/api_parameter_data_required_list", method = RequestMethod.GET)
    @ApiOperation(value = "API参数必输标识", response = MConventionalDict.class)
    Collection<MConventionalDict> getApiParameterDataRequiredList(){
        List<SystemDict> list = dictManager.getDictEntries("APICSBSBS",null);
        List<MConventionalDict> result = new ArrayList<>();
        for(SystemDict systemDict:list){
            result.add(changeToEhrDic(systemDict));
        }
        return result;
    }

    @RequestMapping(value = "/dictionaries/api_protocol", method = RequestMethod.GET)
    @ApiOperation(value = "API协议", response = MConventionalDict.class)
    MConventionalDict getApiProtocol(
            @ApiParam(name = "code", value = "字典代码", defaultValue = "")
            @RequestParam(value = "code") String code){
        SystemDict dict =  dictManager.getDictIetm("APIXY", code);
        return changeToEhrDic(dict);
    }

    @RequestMapping(value = "/dictionaries/api_protocol_list", method = RequestMethod.GET)
    @ApiOperation(value = "API协议", response = MConventionalDict.class)
    Collection<MConventionalDict> getApiProtocolList(){
        List<SystemDict> list = dictManager.getDictEntries("APIXY",null);
        List<MConventionalDict> result = new ArrayList<>();
        for(SystemDict systemDict:list){
            result.add(changeToEhrDic(systemDict));
        }
        return result;
    }

    @RequestMapping(value = "/dictionaries/api_method", method = RequestMethod.GET)
    @ApiOperation(value = "API方法", response = MConventionalDict.class)
    MConventionalDict getApiMethod(
            @ApiParam(name = "code", value = "字典代码", defaultValue = "")
            @RequestParam(value = "code") String code){
        SystemDict dict =  dictManager.getDictIetm("HTTP_TYPE", code);
        return changeToEhrDic(dict);
    }

    @RequestMapping(value = "/dictionaries/api_method_list", method = RequestMethod.GET)
    @ApiOperation(value = "API方法", response = MConventionalDict.class)
    Collection<MConventionalDict> getApiMethodList(){
        List<SystemDict> list = dictManager.getDictEntries("HTTP_TYPE",null);
        List<MConventionalDict> result = new ArrayList<>();
        for(SystemDict systemDict:list){
            result.add(changeToEhrDic(systemDict));
        }
        return result;
    }

    @RequestMapping(value = "/dictionaries/api_type", method = RequestMethod.GET)
    @ApiOperation(value = "API类型", response = MConventionalDict.class)
    MConventionalDict getApiType(
            @ApiParam(name = "code", value = "字典代码", defaultValue = "")
            @RequestParam(value = "code") String code){
        SystemDict dict =  dictManager.getDictIetm("APP_API_TYPE", code);
        return changeToEhrDic(dict);
    }

    /**
     * 将SystemDict 转换成ehr的 MConventionalDict类型(未包含主键id,因类型不同      未包含catalog,因eip没有此字段)
     * @param dict
     * @return
     */
    private MConventionalDict changeToEhrDic(SystemDict dict){
        MConventionalDict mConventionalDict = new MConventionalDict();
        mConventionalDict.setCode(dict.getCode());
        mConventionalDict.setValue(dict.getValue());
        mConventionalDict.setPhoneticCode(dict.getPyCode());
        mConventionalDict.setSort(dict.getSort());
        return mConventionalDict;
    }

}
