package com.yihu.hos.system.controller;

import com.yihu.ehr.constants.ApiVersion;
import com.yihu.hos.model.standard.MDictionaryEntry;
import com.yihu.hos.system.model.SystemDict;
import com.yihu.hos.system.service.SystemDictService;
import com.yihu.hos.web.framework.util.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Sand
 * @version 1.0
 * @created 2016.02.15 18:25
 */
@RestController
@RequestMapping(ApiVersion.Version1_0)
@Api(value = "DictionaryEntry", description = "系统全局字典项管理", tags = {"系统字典-系统全局字典项管理"})
public class EsbSystemDictEntryEndPoint extends BaseController {

    @Autowired
    private SystemDictService systemDictService;

    @ApiOperation(value = "获取字典项列表")
    @RequestMapping(value = "/esb/dictionaries/entries", method = RequestMethod.GET)
    public List<MDictionaryEntry> getDictEntries(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段", defaultValue = "")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器", defaultValue = "")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序", defaultValue = "")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "size", value = "分页大小", defaultValue = "15")
            @RequestParam(value = "size", required = false) Integer size,
            @ApiParam(name = "page", value = "页码", defaultValue = "1")
            @RequestParam(value = "page", required = false) Integer page,
            HttpServletRequest request,
            HttpServletResponse response) throws Exception {
        List<SystemDict> list = systemDictService.search(fields,filters,sorts,page,size);
        pagedResponse(request, response,systemDictService.getCount(filters), page, size);
        List<MDictionaryEntry> result = new ArrayList<>();
        for(SystemDict systemDict:list){
            result.add(changeToEhrDic(systemDict));
        }
        return result;
    }

    /**
     * 将SystemDict 转换成ehr的 MConventionalDict类型(未包含主键id,因类型不同      未包含catalog,因eip没有此字段)
     * @param dict
     * @return
     */
    private MDictionaryEntry changeToEhrDic(SystemDict dict){
        MDictionaryEntry mConventionalDict = new MDictionaryEntry();
        mConventionalDict.setCode(dict.getCode());
        mConventionalDict.setValue(dict.getValue());
        mConventionalDict.setPhoneticCode(dict.getPyCode());
        mConventionalDict.setSort(dict.getSort());
        return mConventionalDict;
    }
}
