package com.yihu.hos.system.controller;

import com.yihu.hos.core.constants.MicroServices;
import com.yihu.hos.core.constants.ServiceApi;
import com.yihu.hos.model.syspermission.MSystemMenu;
import com.yihu.hos.system.entity.SystemMenu;
import com.yihu.hos.system.service.SystemMenuService;
import com.yihu.hos.web.framework.util.controller.BaseController;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 菜单管理
 */
@RequestMapping(MicroServices.ApiVersion.Version1_0)
@RestController
public class SystemMenuController extends BaseController {

    @Autowired
    private SystemMenuService systemMenuService;

    @ApiOperation(value = "获取菜单列表（分页）")
    @RequestMapping(value = ServiceApi.SystemMenu.List, method = RequestMethod.GET, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public Collection<MSystemMenu> getList(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "筛选条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "size", value = "分页大小", defaultValue = "15")
            @RequestParam(value = "size", required = false) Integer size,
            @ApiParam(name = "page", value = "页码", defaultValue = "1")
            @RequestParam(value = "page", required = false) Integer page,
            HttpServletRequest request,
            HttpServletResponse response) throws Exception {
        List<SystemMenu> systemMenusList = systemMenuService.search(fields, filters, sorts, page, size);
        pagedResponse(request, response, systemMenuService.getCount(filters), page, size);
        return convertToModels(systemMenusList, new ArrayList<>(systemMenusList.size()), MSystemMenu.class, fields);
    }

    @ApiOperation(value = "获取菜单列表（不分页）")
    @RequestMapping(value = ServiceApi.SystemMenu.ListNoPage, method = RequestMethod.GET, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public Collection<MSystemMenu> getListNoPage(
            @ApiParam(name = "filters", value = "筛选条件")
            @RequestParam(value = "filters", required = false) String filters) throws Exception {
        List<SystemMenu> systemMenusList = systemMenuService.search(filters);
        return convertToModels(systemMenusList, new ArrayList<>(systemMenusList.size()), MSystemMenu.class, "");
    }

    @ApiOperation(value = "根据用户权限获取菜单")
    @RequestMapping(value = ServiceApi.SystemMenu.ListByLoginCode, method = RequestMethod.GET, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public List<Map<String, Object>> getListByLoginCode(
            @ApiParam(name = "loginCode", value = "用户登录名", required = true)
            @RequestParam(value = "loginCode") String loginCode) throws Exception {
        return systemMenuService.getListByLoginCode(loginCode, -1);
    }

    @ApiOperation(value = "获取菜单")
    @RequestMapping(value = ServiceApi.SystemMenu.GetById, method = RequestMethod.GET, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public MSystemMenu getById(
            @ApiParam(name = "id", value = "菜单主键", required = true)
            @PathVariable Integer id) throws Exception {
        return convertToModel(systemMenuService.getById(id), MSystemMenu.class);
    }

    @ApiOperation(value = "新增菜单")
    @RequestMapping(value = ServiceApi.SystemMenu.Create, method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public MSystemMenu create(
            @ApiParam(name = "systemMenuJson", value = "菜单模型JSON字符串", required = true)
            @RequestBody String systemMenuJson) throws Exception {
        SystemMenu systemMenu = toEntity(systemMenuJson, SystemMenu.class);
        systemMenuService.create(systemMenu);
        return convertToModel(systemMenu, MSystemMenu.class);
    }

    @ApiOperation(value = "更新菜单")
    @RequestMapping(value = ServiceApi.SystemMenu.Update, method = RequestMethod.PUT, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public MSystemMenu update(
            @ApiParam(name = "systemMenuJson", value = "菜单模型JSON字符串", required = true)
            @RequestBody String systemMenuJson) throws Exception {
        SystemMenu systemMenu = toEntity(systemMenuJson, SystemMenu.class);
        systemMenuService.update(systemMenu);
        return convertToModel(systemMenu, MSystemMenu.class);
    }

    @ApiOperation(value = "删除菜单")
    @RequestMapping(value = ServiceApi.SystemMenu.Delete, method = RequestMethod.DELETE, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public boolean delete(
            @ApiParam(name = "ids", value = "菜单主键集（逗号分隔）", required = true)
            @RequestParam String ids) throws Exception {
        systemMenuService.delete(ids);
        return true;
    }

    @ApiOperation(value = "判断编码是否唯一")
    @RequestMapping(value = ServiceApi.SystemMenu.IsUniqueCode, method = RequestMethod.GET, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public boolean isUniqueCode(
            @ApiParam(name = "id", value = "菜单主键", required = true)
            @RequestParam int id,
            @ApiParam(name = "code", value = "菜单编码", required = true)
            @RequestParam String code) throws Exception {
        return systemMenuService.isUniqueCode(id, code);
    }

    @ApiOperation(value = "判断名称是否唯一")
    @RequestMapping(value = ServiceApi.SystemMenu.IsUniqueName, method = RequestMethod.GET, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public boolean isUniqueName(
            @ApiParam(name = "id", value = "菜单主键", required = true)
            @RequestParam int id,
            @ApiParam(name = "name", value = "菜单名称", required = true)
            @RequestParam String name) throws Exception {
        return systemMenuService.isUniqueName(id, name);
    }

}
