package com.yihu.hos.system.service;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.hos.core.log.Logger;
import com.yihu.hos.core.log.LoggerFactory;
import com.yihu.hos.system.model.SyncMessage;
import com.yihu.hos.web.framework.constant.ContextAttributes;
import com.yihu.hos.web.framework.thread.LocalContext;
import io.zbus.mq.Broker;
import io.zbus.mq.Message;
import io.zbus.mq.Producer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;

@Component
public class DataSyncService {

    private static final Logger logger = LoggerFactory.getLogger(DataSyncService.class);

    @Autowired
    private Broker broker;
    @Autowired
    private ObjectMapper objectMapper;


    public boolean sync(String type, SyncMessage syncMessage){
        boolean b = false;
        if (broker == null) {
            logger.error("broker is null.");
            return b;
        }
        try {
            String tenant = LocalContext.getContext().getAttachment(ContextAttributes.TENANT_NAME);
            if (tenant != null) {
                String topic = "sync@" + tenant;
                Producer producer = new Producer(broker);
                //确定为创建消息队列需要显示调用
                producer.declareTopic(topic);
                Message message = new Message();
                message.setTopic(topic);
                message.setHeader("type", type);
                String msg = objectMapper.writeValueAsString(syncMessage);
                message.setBody(msg);
                producer.publish(message);
                b=true;
            } else {
                logger.error("DataSyncService.sysnData Fail : tenant is null");
            }
        } catch (IOException | InterruptedException e) {
            logger.error(e.getMessage());
            e.printStackTrace();
        }
        return b;
    }
}
