package com.yihu.hos.system.service;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.yihu.hos.system.dao.SystemMenuDao;
import com.yihu.hos.system.entity.SystemMenu;
import com.yihu.hos.web.framework.baseservice.BaseService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 菜单管理 Service
 */
@Service
public class SystemMenuService extends BaseService<SystemMenu> {

    @Autowired
    private SystemMenuDao systemMenuDao;

    public SystemMenu getById(Integer id) throws Exception {
        return systemMenuDao.getEntity(SystemMenu.class, id);
    }

    /**
     * 根据用户权限获取菜单
     *
     * @param loginCode 用户登录code
     * @param pid       父级菜单主键，-1表示pid为null。
     * @return 菜单列表
     */
    public List<Map<String, Object>> getListByLoginCode(String loginCode, int pid) throws IOException {
        List<Map<String, Object>> menuList = new ArrayList<>();

        String sql = " SELECT DISTINCT sm.id AS id, sm.content AS content FROM system_menu sm ";
        // 由于登录功能历史原因，"admin" 账户不在数据库中，并且拥有最高权限，故单独获取全部有效菜单。
        if (!"admin".equals(loginCode)) {
            sql += " JOIN system_user su ON su.login_code = '" + loginCode + "' " +
                    " JOIN system_role_user sru ON sru.user_id = su.id " +
                    " JOIN system_role_menu srm ON srm.role_id = sru.role_id AND srm.menu_id = sm.id " +
                    " JOIN system_role sr ON sr.id = srm.role_id AND sr.valid = 'true' ";
        }
        sql += " WHERE sm.`status` = '1' AND sm.type <> '3' ";
        if (pid == -1) {
            sql += " AND sm.pid IS NULL ";
        } else {
            sql += " AND sm.pid=" + pid;
        }
        sql += " ORDER BY sm.sort ASC ";

        List<Map<String, Object>> tmpList = jdbcTemplate.queryForList(sql);
        for (Map<String, Object> item : tmpList) {
            Map<String, Object> menu = objectMapper.readValue(item.get("content").toString(), Map.class);
            menuList.add(menu);
            menuList.addAll(getListByLoginCode(loginCode, (int) item.get("id")));
        }

        return menuList;
    }

    @Transactional
    public SystemMenu create(SystemMenu systemMenu) throws Exception {
        systemMenuDao.saveEntity(systemMenu);
        joinContent(systemMenu);
        systemMenuDao.updateEntity(systemMenu);
        return systemMenu;
    }

    @Transactional
    public SystemMenu update(SystemMenu systemMenu) throws Exception {
        joinContent(systemMenu);
        systemMenuDao.updateEntity(systemMenu);
        return systemMenu;
    }

    @Transactional
    public void delete(String ids) throws Exception {
        for (String id : ids.split(",")) {
            systemMenuDao.deleteEntity(SystemMenu.class, Integer.parseInt(id));
        }
    }

    public boolean isUniqueCode(int id, String code) {
        return systemMenuDao.isUniqueCode(id, code);
    }

    public boolean isUniqueName(int id, String name) {
        return systemMenuDao.isUniqueName(id, name);
    }

    // 拼接菜单内容
    private void joinContent(SystemMenu menu) throws JsonProcessingException {
        Map<String, Object> contentMap = new HashMap<>();
        contentMap.put("id", menu.getId());
        contentMap.put("text", menu.getName());
        contentMap.put("type", menu.getType());
        if (menu.getPid() != null) {
            contentMap.put("pid", menu.getPid());
        }
        if (!menu.getType().equals("1") && StringUtils.isNotEmpty(menu.getUrl())) {
            // 这里"/esb"为 eip-manage 的上下文路径
            contentMap.put("url", "/esb" + menu.getUrl());
        }
        if (StringUtils.isNotEmpty(menu.getIcon())) {
            contentMap.put("iconUrl", menu.getIcon());
        }
        menu.setContent(objectMapper.writeValueAsString(contentMap));
    }

}
